require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"

Boss1 = Object:extend()


function Boss1.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function Boss1:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.boundsTop    = 0
	self.boundsBottom = 111
	
	self.speedX = 0.0 --take from levelsettings
	self.moveProgressX = 0.0
	
	self.speedY = 0.0 --take from levelsettings
	self.moveProgressY = 0.0
	
	self.startTimer = 0
	self.doesMovementY = false
	
	--####################
	--# Shooting         #
	--####################
	
	self.shootTimer = 0
	self.shootTimerMax = 50
	self.shootState = 0 --invalid value
		
end

function Boss1.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function Boss1.activate(self)
	
	self.active = true
	
	self.currentGph = enmBoss1Gph
	
	self.scoreReward = 1000
	
	self.enmWidth  = 41
	self.enmHeight = 39
	
	self.hitPoints = 200
	
	self.speedX = 2.0
	
	self.startTimer = 25
	
	self.shootTimer = 0 --shoot right when the starttimer has run up
	self.shootState = 1 --valid value
	
end

function Boss1.plantExplosion(self)

	
	globBoss1Explosion.activate(globBoss1Explosion, self.x, self.y)
	
end

function Boss1.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function Boss1.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function Boss1.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				print("Enemy dies")
				
				
				--if globIsInDemoMode == false then 
				--	impactSnd:stop()
				--	impactSnd:play()
				--end
				
				self.deactivate(self)
				
				
				globBossBeaten = true
				globBossBeatTimer = 250
				
				globLevelMusic:stop()
			else
				if globIsInDemoMode == false then 
					bossHitSnd:stop()
					bossHitSnd:play()
				end
			end
		end
	end
	
	--only if still active
	if self.active == true then
		self.executeFlyMove(self) --always as long as active: move along the x axis
	end
end

function Boss1.executeFlyMove(self)

	if self.doesMovementY == false then

		self.moveProgressX = self.moveProgressX - self.speedX
	
		while self.moveProgressX <= ((-1) * threshold) do --for negative x-Movement
			self.moveProgressX = self.moveProgressX + threshold
			self.x = self.x - 1
		end
		
		self.startTimer = self.startTimer - 1
		
		if self.startTimer == 0 then
			self.doesMovementY = true
			self.speedY = (-1)*1.7
		end
		
		
	else
	
		self.moveProgressY = self.moveProgressY + self.speedY
	
		while self.moveProgressY <= ((-1) * threshold) do --for negative x-Movement
			self.moveProgressY = self.moveProgressY + threshold
			self.y = self.y - 1
		end
		
		while self.moveProgressY >= threshold do --for negative x-Movement
			self.moveProgressY = self.moveProgressY - threshold
			self.y = self.y + 1
		end
		
		if self.y <= self.boundsTop then
			self.speedY = 1.7
		end
		
		if self.y >= self.boundsBottom then
			self.speedY = (-1)*1.7
		end
		
		
		if self.shootTimer > 0 then
			self.shootTimer = self.shootTimer - 1
		end
		
		if self.shootTimer <= 0 then
			self.shootTimer = self.shootTimerMax
			self.plantShot(self, self.shootState)
			self.shootState = self.shootState + 1
			if self.shootState > 4 then
				self.shootState = 1
			end
		end

		
	
	end
	
end

function Boss1.plantShot(self, aState)

	local lclLaunchPX = 0
	local lclLaunchPY = 0
	
	local lclXSpd = 0.0
	local lclYSpd = 0.0
	
	lclLaunchPX = self.x
	
	if aState == 1 or aState == 3 then
		lclLaunchPY = self.y + 18
	elseif aState == 2 then
		lclLaunchPY = self.y + 33
	elseif aState == 4 then
		lclLaunchPY = self.y + 3
	end
	
	lclXSpd = (-1)*2.3
	
	lclYSpd = (-1)*1.6
	
	globEnmBulletsB[globEnmBulletsBIndex].activate(globEnmBulletsB[globEnmBulletsBIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
	globEnmBulletsBIndex = globEnmBulletsBIndex + 1
	if globEnmBulletsBIndex >= globEnmBulletsBIndexMax then
		globEnmBulletsBIndex = 1
	end
	
	lclYSpd = (-1)*0.8
	
	globEnmBulletsB[globEnmBulletsBIndex].activate(globEnmBulletsB[globEnmBulletsBIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
	globEnmBulletsBIndex = globEnmBulletsBIndex + 1
	if globEnmBulletsBIndex >= globEnmBulletsBIndexMax then
		globEnmBulletsBIndex = 1
	end
	
	lclYSpd = 0.0
	
	globEnmBulletsB[globEnmBulletsBIndex].activate(globEnmBulletsB[globEnmBulletsBIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
	globEnmBulletsBIndex = globEnmBulletsBIndex + 1
	if globEnmBulletsBIndex >= globEnmBulletsBIndexMax then
		globEnmBulletsBIndex = 1
	end
	
	lclYSpd = 0.8
	
	globEnmBulletsB[globEnmBulletsBIndex].activate(globEnmBulletsB[globEnmBulletsBIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
	globEnmBulletsBIndex = globEnmBulletsBIndex + 1
	if globEnmBulletsBIndex >= globEnmBulletsBIndexMax then
		globEnmBulletsBIndex = 1
	end
	
	lclYSpd = 1.6
	
	globEnmBulletsB[globEnmBulletsBIndex].activate(globEnmBulletsB[globEnmBulletsBIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
	globEnmBulletsBIndex = globEnmBulletsBIndex + 1
	if globEnmBulletsBIndex >= globEnmBulletsBIndexMax then
		globEnmBulletsBIndex = 1
	end
	
	if globIsInDemoMode == false then 
		shotSnd:stop()
		shotSnd:play()
	end
	
end

function Boss1.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end







