require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"

Boss3 = Object:extend()


function Boss3.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function Boss3:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.moveState = 0 --invalid value
	self.movePtX = {}
	self.movePtY = {}
	self.movePtX[1] = 84
	self.movePtY[1] = 59
	self.movePtX[2] = 84
	self.movePtY[2] = 0
	self.movePtX[3] = 84
	self.movePtY[3] = 59
	self.movePtX[4] = 0
	self.movePtY[4] = 59
	self.movePtX[5] = 84
	self.movePtY[5] = 59
	self.movePtX[6] = 84
	self.movePtY[6] = 118
	self.movePtX[7] = 84
	self.movePtY[7] = 59
	self.movePtX[8] = 168
	self.movePtY[8] = 59
	
	--####################
	--# Shooting         #
	--####################
	
	self.shootTimer = 0
	self.shootTimerMax = 70
	
	self.shootState = 0 --invalid value
	self.shootPtX ={}
	self.shootPtY ={}
	self.shootPtX[1] = 36
	self.shootPtY[1] = 22
	self.shootPtX[2] = 158
	self.shootPtY[2] = 22
	self.shootPtX[3] = 36
	self.shootPtY[3] = 122
	self.shootPtX[4] = 158
	self.shootPtY[4] = 122
		
end

function Boss3.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function Boss3.activate(self)
	
	self.active = true
	
	self.currentGph = enmBoss3Gph
	
	self.scoreReward = 3000
	
	self.enmWidth  = 32
	self.enmHeight = 32
	
	self.hitPoints = 200
	
	self.shootTimer = self.shootTimerMax
	
	self.moveState = 1 --valid value bzw. init value
	self.shootState = 1--valid value bzw. init value
	
end

function Boss3.plantExplosion(self)

	
	globBoss3Explosion.activate(globBoss3Explosion, self.x, self.y)
	
end

function Boss3.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function Boss3.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function Boss3.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				print("Enemy dies")
				
				
				--if globIsInDemoMode == false then 
				--	impactSnd:stop()
				--	impactSnd:play()
				--end
				
				self.deactivate(self)
				
				
				globBossBeaten = true
				globBossBeatTimer = 250
				
				globLevelMusic:stop()
			else
				if globIsInDemoMode == false then 
					bossHitSnd:stop()
					bossHitSnd:play()
				end
			end
		end
	end
	
	--only if still active
	if self.active == true then
		self.executeFlyMove(self) --always as long as active: move along the x axis
	end
end

function Boss3.executeFlyMove(self)
		
	local lclDirectionVectorX = self.movePtX[self.moveState] - self.x
	local lclDirectionVectorY = self.movePtY[self.moveState] - self.y 
		
	local lclVectorLength = math.sqrt(lclDirectionVectorX * lclDirectionVectorX + lclDirectionVectorY * lclDirectionVectorY)
		
	if lclVectorLength < 2.0 then
		self.x = self.movePtX[self.moveState]
		self.y = self.movePtY[self.moveState]
		self.plantShot(self, self.moveState)
		self.moveState = self.moveState + 1
		
		if self.moveState > 8 then
			self.moveState = 1
		end
	else
		lclDirectionVectorX = lclDirectionVectorX/lclVectorLength
		lclDirectionVectorY = lclDirectionVectorY/lclVectorLength
		lclDirectionVectorX = lclDirectionVectorX*1.3
		lclDirectionVectorY = lclDirectionVectorY*1.3
		self.x = self.x + lclDirectionVectorX
		self.y = self.y + lclDirectionVectorY
	end
	
	
		
	if self.shootTimer > 0 then
		self.shootTimer = self.shootTimer - 1
	end
		
	if self.shootTimer <= 0 then
		self.shootTimer = self.shootTimerMax
		self.plantBigShot(self)
	end
	
end

function Boss3.plantBigShot(self)


	local lclLaunchPX = self.shootPtX[self.shootState]
	local lclLaunchPY = self.shootPtY[self.shootState]
	
	self.shootState = self.shootState + 1
	if self.shootState > 4 then
		self.shootState = 1
	end
	
	local lclXSpd = 0.0
	local lclYSpd = 0.0
	local lclSkip = false
	

	lclXSpd = globPlShip[1].x + 8 - lclLaunchPX
	lclYSpd = globPlShip[1].y + 6 - lclLaunchPY
		
	local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
		
	if lclVectorLength < epsilon then
		lclSkip = true
	else
		lclXSpd = lclXSpd/lclVectorLength
		lclYSpd = lclYSpd/lclVectorLength
			
		lclXSpd = lclXSpd*2.3
		lclYSpd = lclYSpd*2.3
	end
	
	if lclSkip == false then

		globEnmBulletsF[globEnmBulletsFIndex].activate(globEnmBulletsF[globEnmBulletsFIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
		globEnmBulletsFIndex = globEnmBulletsFIndex + 1
		if globEnmBulletsFIndex >= globEnmBulletsFIndexMax then
			globEnmBulletsFIndex = 1
		end
		
	else
	
		--in this case, dont really skip, dont allow shipposition to cancel out shot, instead just shoot forward, as so
		globEnmBulletsF[globEnmBulletsFIndex].activate(globEnmBulletsF[globEnmBulletsFIndex], lclLaunchPX, lclLaunchPY, 2.3, 0.0)
		
		globEnmBulletsFIndex = globEnmBulletsFIndex + 1
		if globEnmBulletsFIndex >= globEnmBulletsFIndexMax then
			globEnmBulletsFIndex = 1
		end
	end
	
	if globIsInDemoMode == false then 
		shotSnd:stop()
		shotSnd:play()
	end
end

function Boss3.plantShot(self, aState)

	local lclLaunchPX = self.x + 14
	local lclLaunchPY = self.y + 14
	
	local lclXSpd = 0.0
	local lclYSpd = 0.0
	local lclTotalSpeed = 2.7
	

	
	--left bullet
	lclXSpd = (-1)*1.0
	lclYSpd = 0.0
	local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
	lclXSpd = lclXSpd/lclVectorLength
	lclYSpd = lclYSpd/lclVectorLength	
	lclXSpd = lclXSpd*lclTotalSpeed
	lclYSpd = lclYSpd*lclTotalSpeed
	--
	globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
	--	
	globEnmBulletsAIndex = globEnmBulletsAIndex + 1
	if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
		globEnmBulletsAIndex = 1
	end
	
	--lower bullet
	lclXSpd = 0.0
	lclYSpd = 1.0
	local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
	lclXSpd = lclXSpd/lclVectorLength
	lclYSpd = lclYSpd/lclVectorLength	
	lclXSpd = lclXSpd*lclTotalSpeed
	lclYSpd = lclYSpd*lclTotalSpeed
	--
	globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
	--	
	globEnmBulletsAIndex = globEnmBulletsAIndex + 1
	if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
		globEnmBulletsAIndex = 1
	end
	
	--lower left bullet
	lclXSpd = (-1)*1.0
	lclYSpd = 1.0
	local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
	lclXSpd = lclXSpd/lclVectorLength
	lclYSpd = lclYSpd/lclVectorLength	
	lclXSpd = lclXSpd*lclTotalSpeed
	lclYSpd = lclYSpd*lclTotalSpeed
	--
	globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
	--	
	globEnmBulletsAIndex = globEnmBulletsAIndex + 1
	if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
		globEnmBulletsAIndex = 1
	end
	
	--right bullet
	lclXSpd = 1.0
	lclYSpd = 0.0
	local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
	lclXSpd = lclXSpd/lclVectorLength
	lclYSpd = lclYSpd/lclVectorLength	
	lclXSpd = lclXSpd*lclTotalSpeed
	lclYSpd = lclYSpd*lclTotalSpeed
	--
	globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
	--	
	globEnmBulletsAIndex = globEnmBulletsAIndex + 1
	if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
		globEnmBulletsAIndex = 1
	end
	
	--lower right bullet
	lclXSpd = 1.0
	lclYSpd = 1.0
	local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
	lclXSpd = lclXSpd/lclVectorLength
	lclYSpd = lclYSpd/lclVectorLength	
	lclXSpd = lclXSpd*lclTotalSpeed
	lclYSpd = lclYSpd*lclTotalSpeed
	--
	globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
	--	
	globEnmBulletsAIndex = globEnmBulletsAIndex + 1
	if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
		globEnmBulletsAIndex = 1
	end
	
	--upper bullet
	lclXSpd = 0.0
	lclYSpd = (-1)*1.0
	local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
	lclXSpd = lclXSpd/lclVectorLength
	lclYSpd = lclYSpd/lclVectorLength	
	lclXSpd = lclXSpd*lclTotalSpeed
	lclYSpd = lclYSpd*lclTotalSpeed
	--
	globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
	--	
	globEnmBulletsAIndex = globEnmBulletsAIndex + 1
	if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
		globEnmBulletsAIndex = 1
	end
	
	--upper right bullet
	lclXSpd = 1.0
	lclYSpd = (-1)*1.0
	local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
	lclXSpd = lclXSpd/lclVectorLength
	lclYSpd = lclYSpd/lclVectorLength	
	lclXSpd = lclXSpd*lclTotalSpeed
	lclYSpd = lclYSpd*lclTotalSpeed
	--
	globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
	--	
	globEnmBulletsAIndex = globEnmBulletsAIndex + 1
	if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
		globEnmBulletsAIndex = 1
	end

	--left upper bullet 
	lclXSpd = (-1)*1.0
	lclYSpd = (-1)*1.0
	local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
	lclXSpd = lclXSpd/lclVectorLength
	lclYSpd = lclYSpd/lclVectorLength	
	lclXSpd = lclXSpd*lclTotalSpeed
	lclYSpd = lclYSpd*lclTotalSpeed
	--
	globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
	--	
	globEnmBulletsAIndex = globEnmBulletsAIndex + 1
	if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
		globEnmBulletsAIndex = 1
	end
	
	if globIsInDemoMode == false then 
		shotSnd:stop()
		shotSnd:play()
	end
	
end

function Boss3.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end







