require "luas.shortcuts"
require "luas.globals"

DemoMode = Object:extend()

function DemoMode.new(self)

	self.leaveStateReady  = false
	self.leaveStateWithDevice = GLOB_INPUT_WAITING
	
	--need those to determine which input device was used to exit demo.
	--it will be the device that is used to control the 1st player.
	self.quitWithKeyboardFirePress = false
	self.quitWithGamepadFirePress  = false
	self.quitWithGamepad2FirePress = false
	self.quitWithGamepad3FirePress = false
	self.quitWithMouseFirePress    = false
	self.blockMouseOnWindowbarY    = 0	--must not start game or join player when window gains focus and
	                                    --mouse gets click on windowbar instead of playfield
	
	self.leaveStateDemoCountdown      = 0 --SEPARATE FROM GAMEOVER FLAG: start with gameover, block return to demo for short time
	self.LEAVE_STATE_LEAVE_DEMO_DELAY = 40--SEPARATE FROM GAMEOVER FLAG: min time before return to demo
	
	self.levelBackGph   = backGroundGph
	--self.levelBackFlickerGph   = backGroundFlickerGph
	
	self.bkg_x = 0
	self.bkg_y = 0
	
	-- with this, force smallUpdates only during playerDying Phase
	self.doUpdatesFlag = true
	
	--don't let player join when mouse-event on window-bar instead of playfield is worked through
	self.blockMouseOnWindowbarY = 0
	
	self.debug = 0
	
	self.cursorPosX = {}
	self.cursorPosY = {}
	
	self.cursorPosX[1] = 72
	self.cursorPosX[2] = 72
	self.cursorPosX[3] = 72
	self.cursorPosY[1] = 73
	self.cursorPosY[2] = 84
	self.cursorPosY[3] = 95
	
	self.currentCursorPosX = self.cursorPosX[globDifficulty]
	self.currentCursorPosY = self.cursorPosY[globDifficulty]
	
	self.upPress    = false
	self.downPress  = false
	self.firePress  = false
	self.upDownPressedCounter = 0
	self.oldMounseX = 0
	self.oldMouseY = 0
	
	self.musicPlaying = false
	
end


function DemoMode:update(dt)
	--do not depend on deltatime dt here, use doUpdate() instead
	

end

--replace update
function DemoMode:doUpdate()																	--Task AAQ (part 2) update demo mode

	if self.musicPlaying == false then
		self.musicPlaying = true
		titleMusic:stop()
		titleMusic:play()
	end

	if self.leaveStateDemoCountdown < self.LEAVE_STATE_LEAVE_DEMO_DELAY then
		--
		self.leaveStateDemoCountdown = self.leaveStateDemoCountdown + 1
	else
		self:keyProcess()
	end
	
	--update difficulty
	
	self.upPress    = false
	self.downPress  = false
	self.firePress  = false
	
					--if self.inputType == GLOB_INPUT_KEYBOARD then --all controls at once
	
	
	if lkeyBrd("up") == true then
		self.upPress = true
	end
	if lkeyBrd("down") == true then
		self.downPress = true
	end
						
	if lkeyBrd("z") == true or lkeyBrd("y") == true then
		self.firePress  = true
	end
	
			
	if globJoystick then --only react when joystick 1 is actually in usb 
					
		if globJoystick:isGamepadDown("dpup") == true then
			self.upPress = true
		end
		if globJoystick:isGamepadDown("dpdown") == true then
			self.downPress = true
		end

		if globJoystick:isGamepadDown("a", "b", "x", "y", "start") == true then
			self.firePress  = true
		end

	end
						
					
	if globJoystick2 then --only react when joystick 1 is actually in usb 
					
		if globJoystick2:isGamepadDown("dpup") == true then
			self.upPress = true
		end
		if globJoystick2:isGamepadDown("dpdown") == true then
			self.downPress = true
		end
								
		if globJoystick2:isGamepadDown("a", "b", "x", "y", "start") == true then
			self.firePress = true
		end
				
	end
					
	if globJoystick3 then 
					
		if globJoystick3:isGamepadDown("dpup") == true then
			self.upPress = true
		end
		if globJoystick3:isGamepadDown("dpdown") == true then
			self.downPress = true
		end
					
								
		if globJoystick3:isGamepadDown("a", "b", "x", "y", "start") == true then
			self.firePress = true
		end				
					
	end
	
	if self.downPress == true and self.upPress == true then --both directions pressed, cancel each other out
		self.downPress = false
		self.upPress = false
	end
	
	if self.downPress == true or self.upPress == true then --one of the both pressed
		if self.upDownPressedCounter == 0 then
		
			self.upDownPressedCounter = 50
			
			--react
			if self.downPress == true then
				globDifficulty = globDifficulty + 1
				
				if globDifficulty > GLOB_DIFFICULTY_HARD then --cycle through
					globDifficulty = GLOB_DIFFICULTY_EASY
				end
			end
			if self.upPress == true then
				globDifficulty = globDifficulty - 1
				
				if globDifficulty < GLOB_DIFFICULTY_EASY then --cycle through
					globDifficulty = GLOB_DIFFICULTY_HARD
				end
			end
		
		else
			self.upDownPressedCounter = self.upDownPressedCounter - 1
		end
	else
		self.upDownPressedCounter = 0
	end
	
					
	local lclMouseX = lMouseGetX()
	local lclMouseY = lMouseGetY()
	--transfrom mouse according to game resolution (window to big-pixel)
	lclMouseX = mouseX_Transformed(lclMouseX)
	lclMouseY = mouseY_Transformed(lclMouseY)
	
	if lclMouseX ~= self.oldMounseX or lclMouseY ~= self.oldMouseY then --if mouseposition has changed, test for setting cursor to difficulty according to mouseposition
		
		self.mouseX = lclMouseX
		self.mouseY = lclMouseY
		
		self.oldMounseX = self.mouseX
		self.oldMouseY = self.mouseY
		
		if self.mouseX >= self.cursorPosX[1] - 9 and self.mouseX <= self.cursorPosX[1] + 80 then
			if self.mouseY >= self.cursorPosY[1] and self.mouseY <= self.cursorPosY[1] + 7 then
				globDifficulty = 1
			end
		end
		
		if self.mouseX >= self.cursorPosX[2] - 9 and self.mouseX <= self.cursorPosX[2] + 80 then
			if self.mouseY >= self.cursorPosY[2] and self.mouseY <= self.cursorPosY[2] + 7 then
				globDifficulty = 2
			end
		end
		
		if self.mouseX >= self.cursorPosX[3] - 9 and self.mouseX <= self.cursorPosX[3] + 80 then
			if self.mouseY >= self.cursorPosY[3] and self.mouseY <= self.cursorPosY[3] + 7 then
				globDifficulty = 3
			end
		end

	end
					

	if lMouseDown("l") == true then
		self.firePress  = true --fire with left mousebutton down 
	end

	
	
	
	
	--update cursorpos
	self.currentCursorPosX = self.cursorPosX[globDifficulty]
	self.currentCursorPosY = self.cursorPosY[globDifficulty]
	
	
	
	
	--if self.isPaused == false then--no pause in demo
																--Task AAM (part 2) update game mode

		--##########################################################################
		--# fill level according to current global level number (NO-UPDATE-FUNCTION)
		--##########################################################################
		
	
		--#######################################################
		--# do joining of 2nd or 3rd player  (NO-UPDATE-FUNCTION)  --don't have this in demomode
		--#######################################################
	
	
		--#######################
		--# update player       #
		--#######################
	
	
		--#######################
		--# update player bullets
		--#######################
	
	
		--###############################################################
		--# do playerbullet-enemy collision handling (NO-UPDATE-FUNCTION)
		--###############################################################
	
	
		--#############################
		--# decide for enemies shooting
		--#############################
	
	
		--#######################
		--# update enemies 
		--#######################
	
	
	
		--#######################
		--# update big enemies 
		--#######################
	


		--#######################
		--# update enemy bullets
		--#######################
	
	
		--##########################
		--# update enemy BIG bullets
		--##########################
	
	
		--#######################
		--# update explosions
		--#######################
	
	
		--#######################
		--# update BIG explosions
		--#######################
	
	
		--##########################
		--# update Player explosions
		--##########################
	
	
		--#####################################
		--#check for player taking a bullet hit
		--#####################################
		--
	
	
		--######################################
		--#check for player colliding with enemy
		--######################################
		--
	
	
		--########################
		--#check for level cleared
		--########################

	
	
		--########################
		--#update mask printer
		--########################
	
	
		--#########################################################
		--# have background flicker on playerdeath only for so long
		--#########################################################
	
	
		--#########################################
		--#Do Gameover-checks and Gameover handling
		--#########################################
	
		--self.doGameOverCheck(self) --no gameover in demo mode
		
	--end --no pause in demo
	
	
	--self.checkPauseUnPause(self) --no pause in demo
	
	
	--no gameover in demomode, keyprocess is called above
	--
	--if self.gameOverFlag == true then --
		--
	--	if self.leaveStateGameOverCountdown < self.LEAVE_STATE_GAME_OVER_DELAY then
	--		self.leaveStateGameOverCountdown = self.leaveStateGameOverCountdown + 1
	--	else
			--##################################################################################
			--#	Read and handle Controls that need to be processed in Gamestate (for State exit)
			--##################################################################################
			--
	--		self:keyProcess()
	--	end
		--
	--end
	
	
	
	
	--if self.doUpdatesFlag == false then 
										  
	--	self.playerDeathHandling(self)
		
	--end
	
	--#########################################################
	--# DEMO only: update titlescreen, logos, press fire prompt
	--#########################################################
	
	--self.doTitlescreenHandling(self)
	
	
end

function DemoMode:keyProcess()
	--
	--
	self.quitWithKeyboardFirePress = lkeyBrd("z") or lkeyBrd("y")
	self.quitWithMouseFirePress    = lMouseDown("l")
	--
	local lclOldMousePos = self.blockMouseOnWindowbarY
	self.blockMouseOnWindowbarY = lMouseGetY()
	
	if self.quitWithMouseFirePress == true then
		print("mouse start on coordinate"..self.blockMouseOnWindowbarY)
	end
	
	if self.blockMouseOnWindowbarY <= 0 then --if mousefire outside playfield (happens on windowbar gaining focus), 
		self.quitWithMouseFirePress = false  --then DON'T react to it
	elseif lclOldMousePos <= 0 and self.blockMouseOnWindowbarY > 0 then
		self.quitWithMouseFirePress = false --even when just re-entering window and mouseevent still keeps prending
		                                    --when is was a click on windowbar then DON'T REACT TO THAT FUCKING MOUSEPRESS
		--TODO: still fail, also see gameMode (checkPlayerJoin)
	end
	
	
	if globJoystick then --condition for joystick plugged in, tested in globals
			
		--dleft  = globJoystick:isGamepadDown("dpleft")
		--dright = globJoystick:isGamepadDown("dpright")
		
		--TODO can do for now, but need to find start button
		--Btns 1,2,3 directional (Number for up-Button Missing?)
		--Btns 8, 9 on top
		--Btns 10, 11, 12, 13 Roundbuttons (Fire)
		--other buttons not yet found up until index 25
		
		--test
		--print("hasjoystick")
		
		--if globJoystick:isDown(10) then
		--	print("button 10 down")
		--end

		--down10 = globJoystick:isDown(10) --use all 4 buttons as fire button 
		--down11 = globJoystick:isDown(11) --use all 4 buttons as fire button 
		--down12 = globJoystick:isDown(12) --use all 4 buttons as fire button 
		--down13 = globJoystick:isDown(13) --use all 4 buttons as fire button 
		
		--if down10 == true or down11 == true or down12 == true or down13 == true then
		--	self.quitWithGamepadFirePress = true
		--end
		
		self.quitWithGamepadFirePress = globJoystick:isGamepadDown("a", "b", "x", "y", "start")
		
		
		--if globJoystick:isDown(1) then
		--else
		--end
	else
		--print("No Joystick")
	end
	
	
	if globJoystick2 then --condition for joystick plugged in, tested in globals
		
		self.quitWithGamepad2FirePress = globJoystick2:isGamepadDown("a", "b", "x", "y", "start")

	else
		--print("No Joystick2")
	end
	
	if globJoystick3 then --condition for joystick plugged in, tested in globals
		
		self.quitWithGamepad3FirePress = globJoystick3:isGamepadDown("a", "b", "x", "y", "start")

	else
		--print("No Joystick2")
	end
	
	
	
	--self.quitWithGamepadFirePress  = --TODO Gamepad
	--self.quitWithMouseFirePress    = --TODO mouse
	
	--device to be carried over to game (use for 1st player)
	if self.quitWithKeyboardFirePress == true then
	
		self.leaveStateReady  = true
		self.leaveStateWithDevice = GLOB_INPUT_KEYBOARD
		print("DemoMode: Keyboard input detected")
		titleMusic:stop()
		
	elseif self.quitWithGamepadFirePress == true then
	
		self.leaveStateReady  = true
		self.leaveStateWithDevice = GLOB_INPUT_GAMEPAD
		print("DemoMode: Gamepad input detected")
		titleMusic:stop()
		
	elseif self.quitWithGamepad2FirePress == true then
	
		self.leaveStateReady  = true
		self.leaveStateWithDevice = GLOB_INPUT_GAMEPAD2
		print("DemoMode: Gamepad2 input detected")
		titleMusic:stop()
	
	elseif self.quitWithGamepad3FirePress == true then
	
		self.leaveStateReady  = true
		self.leaveStateWithDevice = GLOB_INPUT_GAMEPAD3
		print("DemoMode: Gamepad3 input detected")
		titleMusic:stop()
	
	elseif self.quitWithMouseFirePress == true then
	
		self.leaveStateReady  = true
		self.leaveStateWithDevice = GLOB_INPUT_MOUSE
		print("DemoMode: Mouse input detected")
		titleMusic:stop()
		
	end
end




function DemoMode.playerBulletEnmCollisionHandling(self)

	
end


function DemoMode.prepareEnemyShooting_Lv1(self)


end



function DemoMode.decideShootingLv2(self)


end


function DemoMode.prepareEnemyShooting_Lv2(self)


end



function DemoMode.levelClearAndFadeoutHandling(self)
	
	
end



function DemoMode.enmBulletPlayerCollisionHandling(self)


end



function DemoMode.enemyPlayerCollisionHandling(self)

	
end



function DemoMode.deathFlickerUseUp(self)


end


function DemoMode.doTitlescreenHandling(self)


end


function DemoMode:draw()																			--Task AAT (part 3) draw canvas demo mode

	--if self.playerDyingCountDown == 0 then
		lg.draw( self.levelBackGph, self.bkg_x, self.bkg_y, 0, 1, 1)
	--else
	--	lg.draw( self.levelBackFlickerGph, self.bkg_x, self.bkg_y, 0, 1, 1)
	--end
	--lg.draw( self.levelGroundGph, self.bkg_x, self.bkg_y + 125, 0, 1, 1) --debug, otherwise below to omit other objects
	
	lg.draw( gameLogoGph, self.bkg_x, self.bkg_y, 0, 1, 1)
	
	
	lg.draw( cursorGph, self.currentCursorPosX, self.currentCursorPosY, 0, 1, 1)
	
	
	--#######################
	--# draw player 
	--#######################
	

	
	--#######################
	--# draw enemy bullets
	--#######################
	
		
	--#######################
	--# draw enemies 
	--#######################
	
	
	--#######################
	--# draw player bullets
	--#######################

	
	--#######################
	--# draw explosions
	--#######################
	
		
	--#################################
	--# draw player explosion
	--#################################
	
	--done above
	
	--################################
	--# draw Maskprint/Statusbar
	--################################
	

	--################################
	--# draw Game Logos
	--################################
	
	
end

