require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"

E_STATE_UNDEFINED  = 0
E_STATE_MOTHERSHIP = 1
E_STATE_COMMENTS   = 2
E_STATE_FINALSCORE = 3


Ending = Object:extend()


function Ending.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function Ending:reset()

	self.active = false	
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.state = E_STATE_UNDEFINED
	
	self.counterMothership = 0
	self.counterMothershipMax = 400
	--
	self.counterComments = 0
	self.counterCommentsMax = 750
	--
	self.eShipSmallPosX = (-20)
	self.eShipSmallPosY = 0
	--
	self.eShipPosX = (-50)
	self.eShipPosY = 0
	--
	self.commentIndex = {}
	self.commentIndex[1] = 0 --invalid value
	self.commentIndex[2] = 0 --invalid value
	self.commentIndex[3] = 0 --invalid value
	self.commentIndex[4] = 0 --invalid value
	
	self.showFinalScreen = false
end



function Ending.activate(self)
	
	self.active = true
	
	self.currentGph = backGroundGph
	
	self.state = E_STATE_MOTHERSHIP
	
	self.commentIndex[1] = 1 --valid value
	self.commentIndex[2] = 1 --valid value
	self.commentIndex[3] = 1 --valid value
	self.commentIndex[4] = 1 --valid value
	
	self.showFinalScreen = false
	
	self.setConsts(self)
	
end

function Ending.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function Ending.doUpdate(self)


	if self.active == true then
	
		self.executePlot(self) --always as long as active: move along the x axis
	end
end

function Ending.executePlot(self)

	if self.state == E_STATE_MOTHERSHIP then
		
		self.counterMothership = self.counterMothership + 1
		
		if self.counterMothership == 1 then
			globExplosionMothership.activate(globExplosionMothership, 82, 57)
		end
		
		if self.counterMothership == 120 then
		
			self.eShipSmallPosX = 77
			self.eShipSmallPosY = 73
			
			if globIsInDemoMode == false then
				escapeSnd:stop()
				escapeSnd:play()
			end
		end
		
		if self.counterMothership > 120 then
			self.eShipSmallPosX = self.eShipSmallPosX - 1
		end
		
		if self.counterMothership == self.counterMothershipMax then
			self.state = E_STATE_COMMENTS
		end
		
	elseif self.state == E_STATE_COMMENTS then
	
		self.counterComments = self.counterComments + 1
		
		if self.counterComments == 1 then
			self.eShipPosX = 92
			self.eShipPosY = 80
			
			endingMusic:stop()
			endingMusic:play()
		end
		
		if self.counterComments >= 30 and self.counterComments < 60 then
			self.commentIndex[1] = 2
		elseif self.counterComments >= 60 and self.counterComments < 90 then
			self.commentIndex[1] = 3
		elseif self.counterComments >= 90 and self.counterComments < 120 then
			self.commentIndex[1] = 4
		elseif self.counterComments >= 120 and self.counterComments < 150 then
			self.commentIndex[2] = 2
		elseif self.counterComments >= 150 and self.counterComments < 180 then
			self.commentIndex[2] = 3
		elseif self.counterComments >= 180 and self.counterComments < 210 then
			self.commentIndex[2] = 4
		elseif self.counterComments >= 210 and self.counterComments < 240 then
			self.commentIndex[3] = 2
		elseif self.counterComments >= 240 and self.counterComments < 270 then
			self.commentIndex[3] = 3
		elseif self.counterComments >= 270 and self.counterComments < 300 then
			self.commentIndex[3] = 4
		elseif self.counterComments >= 300 and self.counterComments < 330 then
			self.commentIndex[4] = 2
		elseif self.counterComments >= 330 and self.counterComments < 360 then
			self.commentIndex[4] = 3
		elseif self.counterComments >= 360 and self.counterComments < 390 then
			self.commentIndex[4] = 4
		elseif self.counterComments >= 600 and self.counterComments < 630 then
			--reset comments
			self.commentIndex[1] = 1
			self.commentIndex[2] = 1
			self.commentIndex[3] = 1
			self.commentIndex[4] = 1
		elseif self.counterComments >= 630 then
			self.eShipPosX = self.eShipPosX + 1
		end
		
		if self.counterComments == self.counterCommentsMax then
			self.state = E_STATE_FINALSCORE
		end
		
	elseif self.state == E_STATE_FINALSCORE then
		
		self.showFinalScreen = true
		
	end

	
end


function Ending.setConsts(self)

	END_OFFSET_DIGITSYMBOL = 6
	--
	--positions for scoredigits
	END_SCORE_DIGIT_PX = {}
	END_SCORE_DIGIT_PY = {}
	--
	END_SCORE_DIGIT_PX[1] = 110 - 1--108 - 1
	END_SCORE_DIGIT_PX[2] = END_SCORE_DIGIT_PX[1] - END_OFFSET_DIGITSYMBOL
	END_SCORE_DIGIT_PX[3] = END_SCORE_DIGIT_PX[2] - END_OFFSET_DIGITSYMBOL
	END_SCORE_DIGIT_PX[4] = END_SCORE_DIGIT_PX[3] - END_OFFSET_DIGITSYMBOL
	END_SCORE_DIGIT_PX[5] = END_SCORE_DIGIT_PX[4] - END_OFFSET_DIGITSYMBOL
	END_SCORE_DIGIT_PX[6] = END_SCORE_DIGIT_PX[5] - END_OFFSET_DIGITSYMBOL
	END_SCORE_DIGIT_PX[7] = END_SCORE_DIGIT_PX[6] - END_OFFSET_DIGITSYMBOL
	--
	END_SCORE_DIGIT_PY[1] = 85
	END_SCORE_DIGIT_PY[2] = 85
	END_SCORE_DIGIT_PY[3] = 85
	END_SCORE_DIGIT_PY[4] = 85
	END_SCORE_DIGIT_PY[5] = 85
	END_SCORE_DIGIT_PY[6] = 85
	END_SCORE_DIGIT_PY[7] = 85
	


end



function Ending.printScore(self)

	--print score without leading zeros
	sDigit1= globScore % 10
	sDigit2= ((globScore - sDigit1) % 100) / 10
	sDigit3= ((globScore - sDigit2*10 - sDigit1) % 1000) / 100
	sDigit4= ((globScore - sDigit3*100 - sDigit2*10 - sDigit1) % 10000) / 1000 
	sDigit5= ((globScore - sDigit4*1000 - sDigit3*100 - sDigit2*10 - sDigit1) % 100000) / 10000
	sDigit6= ((globScore - sDigit5*10000 - sDigit4*1000 - sDigit3*100 - sDigit2*10 - sDigit1) % 1000000) / 100000 
	sDigit7= ((globScore - sDigit6*100000 - sDigit5*10000 - sDigit4*1000 - sDigit3*100 - sDigit2*10 - sDigit1) / 1000000)
		
		
	--always print first digit even when 0
	lg.draw(digitGph[sDigit1 + 1], END_SCORE_DIGIT_PX[1], END_SCORE_DIGIT_PY[1], 0, 1, 1)
		
	if globScore >= 10 then
		lg.draw(digitGph[sDigit2 + 1], END_SCORE_DIGIT_PX[2], END_SCORE_DIGIT_PY[2], 0, 1, 1)
	end
	if globScore >= 100 then
		lg.draw(digitGph[sDigit3 + 1], END_SCORE_DIGIT_PX[3], END_SCORE_DIGIT_PY[3], 0, 1, 1)
	end
	if globScore >= 1000 then
		lg.draw(digitGph[sDigit4 + 1], END_SCORE_DIGIT_PX[4], END_SCORE_DIGIT_PY[4], 0, 1, 1)
	end
	if globScore >= 10000 then
		lg.draw(digitGph[sDigit5 + 1], END_SCORE_DIGIT_PX[5], END_SCORE_DIGIT_PY[5], 0, 1, 1)
	end
	if globScore >= 100000 then
		lg.draw(digitGph[sDigit6 + 1], END_SCORE_DIGIT_PX[6], END_SCORE_DIGIT_PY[6], 0, 1, 1)
	end
	if globScore >= 1000000 then
		lg.draw(digitGph[sDigit7 + 1], END_SCORE_DIGIT_PX[7], END_SCORE_DIGIT_PY[7], 0, 1, 1)
	end
	
end


function Ending.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
		
		lg.draw(smallShipGph,self.eShipSmallPosX,self.eShipSmallPosY, 0, 1, 1)
		
		lg.draw(commentGph[1][self.commentIndex[1]],0,8, 0, 1, 1)
		lg.draw(commentGph[2][self.commentIndex[2]],0,20, 0, 1, 1)
		lg.draw(commentGph[3][self.commentIndex[3]],0,32, 0, 1, 1)
		lg.draw(commentGph[4][self.commentIndex[4]],0,44, 0, 1, 1)
		
		lg.draw(playerShipGph[1][2],self.eShipPosX,self.eShipPosY, 0, 1, 1)
		
		if self.showFinalScreen == true then
		
			lg.draw(difficultyGph[globDifficulty],0,35, 0, 1, 1)
			lg.draw(finalScoreGph,0,70, 0, 1, 1)
			self.printScore(self)
			lg.draw(theEndGph,0,110, 0, 1, 1)
			
		end

	end
end








