require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"

EnemyA = Object:extend()


function EnemyA.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function EnemyA:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.boundsLeft  = (-1)*14
	
	self.speedX = 0.0 --take from levelsettings
	self.moveProgressX = 0.0
	
	--####################
	--# Shooting         #
	--####################
	
	self.plantShotFlag    = false
	self.aimAtPlyrFlag    = false
	self.shotSpdX = 0.0
	self.shotSpdY = 0.0
	
	
	self.oid = 0

	
end

function EnemyA.setOid(self, aOid)
	self.oid = aOid
end

function EnemyA.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function EnemyA.activate(self, aSpd)
	
	self.active = true
	
	self.currentGph = enmAGph
	
	self.scoreReward = 20
	
	self.enmWidth  = 14
	self.enmHeight = 11
	
	self.hitPoints = 1
	
	self.speedX = aSpd
	
end

function EnemyA.plantExplosion(self)

	globSmallExplosions[globSmallExplosionIndex].activate(globSmallExplosions[globSmallExplosionIndex], self.x - 2, self.y - 2)
	globSmallExplosionIndex = globSmallExplosionIndex + 1
	if globSmallExplosionIndex > globSmallExplosionMax then
		globSmallExplosionIndex = 1
	end
	
end

function EnemyA.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function EnemyA.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function EnemyA.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				print("Enemy dies, oid:"..self.oid)
				
				
				if globIsInDemoMode == false then 
					enemyDie1Snd:stop()
					enemyDie1Snd:play()
				end
				
				self.deactivate(self)
			end
		end
	end
	
	--only if still active
	if self.active == true then
		self.executeFlyMove(self) --always as long as active: move along the x axis
		
		if self.plantShotFlag == true then
			self.plantShot(self)
			self.plantShotFlag = false --flamethrower effect when line is left out :-)
		end
	end
end

function EnemyA.prepareShoot(self, aSpdX, aSpdY, aAimPlyr, aParam1, aParam2)
	print("Prepare shooting")
	
	--keep info that shooting needs to occur separate from actual shooting.
	--do the shooting in next update
	self.plantShotFlag = true
	self.aimAtPlyrFlag    = aAimPlyr
	self.shotSpdX = aSpdX
	self.shotSpdY = aSpdY
	
end

function EnemyA.executeFlyMove(self)

	--scrolling movement pixel
	self.x = self.x - 1

	self.moveProgressX = self.moveProgressX - self.speedX
	
	while self.moveProgressX <= ((-1) * threshold) do --for negative x-Movement
		self.moveProgressX = self.moveProgressX + threshold
		self.x = self.x - 1
	end
	
	if self.x < self.boundsLeft then
		self.deactivate(self)
	end 
	
end

function EnemyA.plantShot(self)
	
	local lclLaunchPX = self.x + 5 
	local lclLaunchPY = self.y + 4
	
	--self.aimAtPlyrFlag    = false
	--self.shotSpdX = 0.0
	--self.shotSpdY = 0.0
	
	local lclXSpd = 0.0
	local lclYSpd = 0.0
	local lclSkip = false
	
	if self.aimAtPlyrFlag == true then
	
		lclXSpd = globPlShip[1].x + 8 - lclLaunchPX
		lclYSpd = globPlShip[1].y + 6 - lclLaunchPY
		
		local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
		
		if lclVectorLength < epsilon then
			lclSkip = true
		else
			lclXSpd = lclXSpd/lclVectorLength
			lclYSpd = lclYSpd/lclVectorLength
			
			lclXSpd = lclXSpd*2.0
			lclYSpd = lclYSpd*2.0
		end
	
	else
		lclXSpd = self.shotSpdX
		lclYSpd = self.shotSpdY
		
		local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
		
		lclXSpd = lclXSpd/lclVectorLength
		lclYSpd = lclYSpd/lclVectorLength
			
		lclXSpd = lclXSpd*2.0
		lclYSpd = lclYSpd*2.0
	end
	
	if lclSkip == false then

		globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
		globEnmBulletsAIndex = globEnmBulletsAIndex + 1
		if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
			globEnmBulletsAIndex = 1
		end
	
	end
	
end

function EnemyA.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end







