require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"

EnemyB = Object:extend()


function EnemyB.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function EnemyB:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.boundsLeft  = (-1)*13
	
	self.destPtX = {}
	self.destPtY = {}
	--
	self.destPtX[1] = 0
	self.destPtY[1] = 0
	--
	self.destPtX[2] = 0
	self.destPtY[2] = 0
	--
	self.destPtX[3] = 0
	self.destPtY[3] = 0
	
	self.speed = 0.0
	self.moveProgressX = 0.0
	self.moveProgressY = 0.0
	
	self.movePhase = 0 --invalid value
	
	--animation
	self.anmIndex = 0 --invalid value
	self.anmIndexMax = 8
	self.anmStep = 0 --ineffective value
	self.anmProgress = 0
	self.anmThresh = 6
	
	self.frames = {}
	
	self.frames[1] = enmBGph[1]
	self.frames[2] = enmBGph[2]
	self.frames[3] = enmBGph[3]
	self.frames[4] = enmBGph[4]
	self.frames[5] = enmBGph[5]
	self.frames[6] = enmBGph[4]
	self.frames[7] = enmBGph[3]
	self.frames[8] = enmBGph[2]
	
	--####################
	--# Shooting         #
	--####################
	
	self.plantShotFlag    = false
	self.aimAtPlyrFlag    = false
	self.shotSpdX = 0.0
	self.shotSpdY = 0.0
	
	self.oid = 0
	
end

function EnemyB.setOid(self, aOid)
	self.oid = aOid
end


function EnemyB.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function EnemyB.activate(self, aSpd, aXReflect, aYReflect)
	
	self.active = true
	
	self.movePhase = 1
	
	self.anmIndex = 1 --valid value
	self.anmStep  = 1  --effective value
	
	self.currentGph = self.frames[self.anmIndex]
	
	self.scoreReward = 30
	
	self.enmWidth  = 13
	self.enmHeight = 11
	
	self.hitPoints = 1
	
	self.speed = aSpd
	
	self.destPtX[1] = 0
	self.destPtY[1] = self.y
	--
	self.destPtX[2] = aXReflect
	self.destPtY[2] = aYReflect
	--
	self.destPtX[3] = (-1)*100
	self.destPtY[3] = aYReflect
	
end

function EnemyB.plantExplosion(self)

	globSmallExplosions[globSmallExplosionIndex].activate(globSmallExplosions[globSmallExplosionIndex], self.x , self.y + 1)
	globSmallExplosionIndex = globSmallExplosionIndex + 1
	if globSmallExplosionIndex > globSmallExplosionMax then
		globSmallExplosionIndex = 1
	end
	
end

function EnemyB.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function EnemyB.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function EnemyB.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				print("Enemy dies, oid:"..self.oid)
				
				
				if globIsInDemoMode == false then 
					enemyDie1Snd:stop()
					enemyDie1Snd:play()
				end
				
				self.deactivate(self)
			end
		end
	end
	
	--only if still active
	if self.active == true then
		self.doAnimation(self)
		self.executeFlyMove(self) --always as long as active: move along the x axis
		
		if self.plantShotFlag == true then
			self.plantShot(self)
			self.plantShotFlag = false --flamethrower effect when line is left out :-)
		end
	end
end

function EnemyB.prepareShoot(self, aSpdX, aSpdY, aAimPlyr, aParam1, aParam2)
	print("Prepare shooting")
	
	--keep info that shooting needs to occur separate from actual shooting.
	--do the shooting in next update
	self.plantShotFlag = true
	self.aimAtPlyrFlag    = aAimPlyr
	self.shotSpdX = aSpdX
	self.shotSpdY = aSpdY
	
end

function EnemyB.doAnimation(self)

	self.anmProgress = self.anmProgress + self.anmStep
	
	if self.anmProgress >= self.anmThresh then
	
		self.anmProgress = 0
		
		self.anmIndex = self.anmIndex + 1
		
		if self.anmIndex > self.anmIndexMax then
			self.anmIndex = 1
		end
		
		self.currentGph = self.frames[self.anmIndex]
		
	end

end

function EnemyB.executeFlyMove(self)

	local lclDirectionVectorX = self.destPtX[self.movePhase] - self.x
	local lclDirectionVectorY = self.destPtY[self.movePhase] - self.y 
	
	local lclVectorLength = math.sqrt(lclDirectionVectorX * lclDirectionVectorX + lclDirectionVectorY * lclDirectionVectorY)
	
	if lclVectorLength < 2.0 then
		self.x = self.destPtX[self.movePhase]
		self.y = self.destPtY[self.movePhase]
		
		if self.movePhase < 3 then --just to be extra safe
			self.movePhase = self.movePhase + 1
		end
	else
		lclDirectionVectorX = lclDirectionVectorX/lclVectorLength
		lclDirectionVectorY = lclDirectionVectorY/lclVectorLength
		
		local lclSpeedX = lclDirectionVectorX*self.speed
		local lclSpeedY = lclDirectionVectorY*self.speed
		
		self.moveProgressX = self.moveProgressX + lclSpeedX
		
		while self.moveProgressX >= threshold do --for positive x-Movement
			self.moveProgressX = self.moveProgressX - threshold
			self.x = self.x + 1
		end
		while self.moveProgressX <= ((-1) * threshold) do --for negative x-Movement
			self.moveProgressX = self.moveProgressX + threshold
			self.x = self.x - 1
		end
		
		self.moveProgressY = self.moveProgressY + lclSpeedY
		
		while self.moveProgressY >= threshold do --for positive y-Movement (not used in this game)
			self.moveProgressY = self.moveProgressY - threshold
			self.y = self.y + 1
		end
		while self.moveProgressY <= ((-1) * threshold) do --for negative y-Movement (standard case)
			self.moveProgressY = self.moveProgressY + threshold
			self.y = self.y - 1
		end
		
		
	end
	
	if self.x < self.boundsLeft then
		self.deactivate(self)
	end 
	
end

function EnemyB.plantShot(self)
	
	
	local lclLaunchPX = self.x + 5 
	local lclLaunchPY = self.y + 4
	
	--self.aimAtPlyrFlag    = false
	--self.shotSpdX = 0.0
	--self.shotSpdY = 0.0
	
	local lclXSpd = 0.0
	local lclYSpd = 0.0
	local lclSkip = false
	
	if self.aimAtPlyrFlag == true then
	
		lclXSpd = globPlShip[1].x + 8 - lclLaunchPX
		lclYSpd = globPlShip[1].y + 6 - lclLaunchPY
		
		local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
		
		if lclVectorLength < epsilon then
			lclSkip = true
		else
			lclXSpd = lclXSpd/lclVectorLength
			lclYSpd = lclYSpd/lclVectorLength
			
			lclXSpd = lclXSpd*2.3
			lclYSpd = lclYSpd*2.3
		end
	
	else
		lclXSpd = self.shotSpdX
		lclYSpd = self.shotSpdY
		
		local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
		
		lclXSpd = lclXSpd/lclVectorLength
		lclYSpd = lclYSpd/lclVectorLength
			
		lclXSpd = lclXSpd*2.3
		lclYSpd = lclYSpd*2.3
	end
	
	if lclSkip == false then

		globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
		globEnmBulletsAIndex = globEnmBulletsAIndex + 1
		if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
			globEnmBulletsAIndex = 1
		end
	
	end
	
	
end

function EnemyB.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end







