require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"

EnemyD = Object:extend()


function EnemyD.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function EnemyD:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.boundsLeft  = (-1)*14
	
	self.speedY = 0.0 --take from levelsettings
	self.moveProgressY = 0.0

	
	--animation
	self.anmIndex = 0 --invalid value
	self.anmIndexMax = 8
	self.anmStep = 0 --ineffective value
	self.anmProgress = 0
	self.anmThresh = 7
	
	self.frames = {}
	
	self.frames[1] = enmDGph[1]
	self.frames[2] = enmDGph[2]
	self.frames[3] = enmDGph[3]
	self.frames[4] = enmDGph[4]
	self.frames[5] = enmDGph[5]
	self.frames[6] = enmDGph[4]
	self.frames[7] = enmDGph[3]
	self.frames[8] = enmDGph[2]
	
	--####################
	--# Shooting         #
	--####################
	
	self.plantShotFlag    = false
	self.aimAtPlyrFlag    = false
	self.shotSpdX = 0.0
	self.shotSpdY = 0.0
	
	self.oid = 0
	
end

function EnemyD.setOid(self, aOid)
	self.oid = aOid
end

function EnemyD.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function EnemyD.activate(self, aSpd)
	
	self.active = true
	
	self.anmIndex = 1 --valid value
	self.anmStep = 1  --effective value
	
	self.currentGph = self.frames[self.anmIndex]
	
	self.scoreReward = 70
	
	self.enmWidth  = 14
	self.enmHeight = 11
	
	self.hitPoints = 3
	
	self.speedY = aSpd
	
end

function EnemyD.plantExplosion(self)

	globMediumExplosions[globMediumExplosionIndex].activate(globMediumExplosions[globMediumExplosionIndex], self.x - 2, self.y - 4)
	globMediumExplosionIndex = globMediumExplosionIndex + 1
	if globMediumExplosionIndex > globMediumExplosionMax then
		globMediumExplosionIndex = 1
	end
	
end

function EnemyD.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function EnemyD.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function EnemyD.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				print("Enemy dies, oid:"..self.oid)
				
				
				if globIsInDemoMode == false then 
					enemyDie3Snd:stop()
					enemyDie3Snd:play()
				end
				
				self.deactivate(self)
			else
				
				if globIsInDemoMode == false then 
					enemyHitSnd:stop()
					enemyHitSnd:play()
				end
			end
		end
	end
	
	--only if still active
	if self.active == true then
		self.doAnimation(self)
		self.executeFlyMove(self) --always as long as active: move along the x axis
		
		if self.plantShotFlag == true then
			self.plantShot(self)
			self.plantShotFlag = false --flamethrower effect when line is left out :-)
		end
	end
end

function EnemyD.prepareShoot(self, aSpdX, aSpdY, aAimPlyr, aParam1, aParam2)
	print("Prepare shooting")
	
	--keep info that shooting needs to occur separate from actual shooting.
	--do the shooting in next update
	self.plantShotFlag = true
	self.aimAtPlyrFlag    = aAimPlyr
	self.shotSpdX = aSpdX
	self.shotSpdY = aSpdY
	
end

function EnemyD.doAnimation(self)

	self.anmProgress = self.anmProgress + self.anmStep
	
	if self.anmProgress >= self.anmThresh then
	
		self.anmProgress = 0
		
		self.anmIndex = self.anmIndex + 1
		
		if self.anmIndex > self.anmIndexMax then
			self.anmIndex = 1
		end
		
		self.currentGph = self.frames[self.anmIndex]
		
	end

end

function EnemyD.executeFlyMove(self)

	--scrolling movement pixel
	self.x = self.x - 1

	self.moveProgressY = self.moveProgressY + self.speedY
	
	while self.moveProgressY >= threshold do --for positive y-Movement
		self.moveProgressY = self.moveProgressY - threshold
		self.y = self.y + 1
	end
	
	while self.moveProgressY <= ((-1) * threshold) do --for negative y-Movement
		self.moveProgressY = self.moveProgressY + threshold
		self.y = self.y - 1
	end
	
	--could restrict y as well but since x changes to negative as time passes this is sufficient
	if self.x < self.boundsLeft then
		self.deactivate(self)
	end 
	
end

function EnemyD.plantShot(self)
	
	
	local lclLaunchPX = self.x + 5 
	local lclLaunchPY = self.y + 4
	
	--self.aimAtPlyrFlag    = false
	--self.shotSpdX = 0.0
	--self.shotSpdY = 0.0
	
	local lclXSpd = 0.0
	local lclYSpd = 0.0
	local lclSkip = false
	
	if self.aimAtPlyrFlag == true then
	
		lclXSpd = globPlShip[1].x + 8 - lclLaunchPX
		lclYSpd = globPlShip[1].y + 6 - lclLaunchPY
		
		local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
		
		if lclVectorLength < epsilon then
			lclSkip = true
		else
			lclXSpd = lclXSpd/lclVectorLength
			lclYSpd = lclYSpd/lclVectorLength
			
			lclXSpd = lclXSpd*2.4
			lclYSpd = lclYSpd*2.4
		end
	
	else
		lclXSpd = self.shotSpdX
		lclYSpd = self.shotSpdY
		
		local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
		
		lclXSpd = lclXSpd/lclVectorLength
		lclYSpd = lclYSpd/lclVectorLength
			
		lclXSpd = lclXSpd*2.4
		lclYSpd = lclYSpd*2.4
	end
	
	if lclSkip == false then

		globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
		globEnmBulletsAIndex = globEnmBulletsAIndex + 1
		if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
			globEnmBulletsAIndex = 1
		end
	
	end
	
	
end

function EnemyD.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end







