require "luas.shortcuts"
require "luas.boundingBox"
require "luas.extendedLoader"
require "luas.point"

EnemyE = Object:extend()


function EnemyE.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end

function EnemyE:reset()

	self.active = false	
	
	self.direction = 0 --invalid value
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	--####################
	--# Shooting         #
	--####################
	
	self.plantShotFlag    = false
	
	--action point
	self.actionPt = {}
	for i=1,7,1 do
		self.actionPt[i] = Point()
	end
	self.actionPt[1].putPosition(self.actionPt[1], (-1), 3)
	self.actionPt[2].putPosition(self.actionPt[2], 1, 1)
	self.actionPt[3].putPosition(self.actionPt[3], 3, (-1))
	self.actionPt[4].putPosition(self.actionPt[4], 6, (-1))
	self.actionPt[5].putPosition(self.actionPt[5], 9, (-1))
	self.actionPt[6].putPosition(self.actionPt[6], 11, 1)
	self.actionPt[7].putPosition(self.actionPt[7], 13, 3)
	
	self.aimPt = Point()
	
	self.aimIndex = 1 --default
	
	self.oid = 0
	
end

function EnemyE.setOid(self, aOid)
	self.oid = aOid
end

function EnemyE.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function EnemyE.activate(self)

	self.active = true
	
	self.direction = love.math.random(7)
	
	self.hitPoints = 1 --basic before activation

	self.scoreReward = 50 
	
	self.enmWidth  = 16 
	self.enmHeight = 11 
	
	self.currentGph  = enmEGph[self.direction]
	--self.currentBBox = BoundingBox(enmCannonBoxGph) --need to put above so group can be determined after putting startposition
	
	self.aimIndex = 1

end


function EnemyE.plantExplosion(self)

	globSmallExplosions[globSmallExplosionIndex].activate(globSmallExplosions[globSmallExplosionIndex], self.x + 4, self.y + 5)
	globSmallExplosionIndex = globSmallExplosionIndex + 1
	if globSmallExplosionIndex > globSmallExplosionMax then
		globSmallExplosionIndex = 1
	end
	
end


function EnemyE.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function EnemyE.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end


function EnemyE.doUpdate(self)


	if self.active == true then
	
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				--give score
				--globScore = globScore + self.scoreReward
				--
				--nope: increasing score must do more than that
				--Therefore
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				print("Enemy dies, oid:"..self.oid)
					
				if globIsInDemoMode == false then 
					enemyDie2Snd:stop()
					enemyDie2Snd:play()
				end
				
				self.deactivate(self)
			end
		end
		
	end	
	
	if self.active == true then --if still active, procceed with the rest
	
		self.executeMove(self)
		
		--self.determineAiming(self)--in this game always aim at player 1
		
		--if self.isAiming == true then --in this game always do aiming
			self.aimAtPlayer(self)
		--end
		
		if self.plantShotFlag == true then
			self.plantShot(self)
			self.plantShotFlag = false --flamethrower effect when line is left out :-)
		end
	end
	
end


function EnemyE.prepareShoot(self)
	print("Prepare shooting")
	
	--keep info that shooting needs to occur separate from actual shooting.
	--do the shooting in next update
	self.plantShotFlag = true
	
end

function EnemyE.executeMove(self)

	--scroll with playfield
	self.x = self.x - 1
end

function EnemyE.aimAtPlayer(self)

	if self.aimIndex > 0 then
		
		self.aimPt.putPosition(self.aimPt, globPlShip[self.aimIndex].x + 8, globPlShip[self.aimIndex].y + 6)
		
		local lclRefX = self.x + self.enmWidth/2
		local lclRefY = self.y + 5
		local lclDeltaX = lclRefX - self.aimPt.x
		local lclDeltaY = lclRefY - self.aimPt.y
		
		if lclDeltaY < 0 then --if player is below bottom of cannon, must not mess up plus or minus which must be according to deltaX
			lclDeltaY = lclDeltaY*(-1)
		end
		
		if lclDeltaX == 0 then
		
			self.direction = 4
			print("Direction set to N")
		else
			local lclIndicator = lclDeltaY/lclDeltaX
			
			if lclIndicator > 0 and lclIndicator < 0.5 then
				self.direction = 1
			elseif lclIndicator >= 0.5 and lclIndicator < 2.0 then
				self.direction = 2
			elseif lclIndicator >= 2.0 and lclIndicator < 4.0 then
				self.direction = 3
			elseif lclIndicator >= 4.0 then
				self.direction = 4
			elseif lclIndicator <= (-1)*4.0 then
				self.direction = 4
			elseif lclIndicator > (-1)*4.0 and lclIndicator <= (-1)*2.0 then
				self.direction = 5
			elseif lclIndicator > (-1)*2.0 and lclIndicator <= (-1)*0.5 then
				self.direction = 6
			elseif lclIndicator > (-1)*0.5 and lclIndicator < 0 then
				self.direction = 7
			end
			
		end
		
		self.currentGph  = enmEGph[self.direction]
	end
--self.aimIndex = 1
end

function EnemyE.plantShot(self)
	
	local lclLaunchPX = self.x + self.actionPt[self.direction].x
	local lclLaunchPY = self.y + self.actionPt[self.direction].y
	
	--self.aimAtPlyrFlag    = false
	--self.shotSpdX = 0.0
	--self.shotSpdY = 0.0
	
	local lclXSpd = 0.0
	local lclYSpd = 0.0
	local lclSkip = false
	
	
	lclXSpd = globPlShip[1].x + 8 - lclLaunchPX
	lclYSpd = globPlShip[1].y + 6 - lclLaunchPY
		
	local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
		
	if lclVectorLength < epsilon then
		lclSkip = true
	else
		lclXSpd = lclXSpd/lclVectorLength
		lclYSpd = lclYSpd/lclVectorLength
			
		lclXSpd = lclXSpd*2.4
		lclYSpd = lclYSpd*2.4
	end

	
	if lclSkip == false then

		globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
		globEnmBulletsAIndex = globEnmBulletsAIndex + 1
		if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
			globEnmBulletsAIndex = 1
		end
	
	end
	
end

function EnemyE.draw(self)

	if self.active then	
		--take graphics directly from extendedloader for this one
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
		--lg.draw(self.currentGph, self.catmullRomPts[4].x, self.catmullRomPts[4].y, 0, 1, 1)
		
		--debug
		--self.currentBBox.draw(self.currentBBox)
	end
end









