require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"

EnemyG = Object:extend()


function EnemyG.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function EnemyG:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.boundsLeft  = (-1)*14
	
	self.speedX = 0.0 --take from levelsettings
	self.moveProgressX = 0.0
	
	self.speedY = 0.0 --take from levelsettings
	self.moveProgressY = 0.0
	
	self.reverseTimer = 0
	self.reverseTimerMax = 30
	
	--####################
	--# Shooting         #
	--####################
	
	self.plantShotFlag    = false
	
	self.oid = 0
	
	self.powerUpType = 0

	
end

function EnemyG.setOid(self, aOid)
	self.oid = aOid
end

function EnemyG.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function EnemyG.setPowerUpType(self, aType)
	self.powerUpType = aType
end

function EnemyG.activate(self)
	
	self.active = true
	
	self.currentGph = enmGGph
	
	self.scoreReward = 0
	
	self.enmWidth  = 14
	self.enmHeight = 13
	
	self.hitPoints = 1
	
	self.speedX = (-1)*1.5
	self.speedY = 1.0
	
	self.reverseTimer = self.reverseTimerMax 
	
end

function EnemyG.plantExplosion(self)

	globMediumExplosions[globMediumExplosionIndex].activate(globMediumExplosions[globMediumExplosionIndex], self.x - 2, self.y - 2)
	globMediumExplosionIndex = globMediumExplosionIndex + 1
	if globMediumExplosionIndex > globMediumExplosionMax then
		globMediumExplosionIndex = 1
	end
	
end

function EnemyG.plantPowerUp(self)
	globPowerup.setType(globPowerup, self.powerUpType)
	globPowerup.putStartPosition(globPowerup, self.x + 1, self.y)
	globPowerup.activate(globPowerup)
end

function EnemyG.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function EnemyG.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function EnemyG.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				self.plantPowerUp(self)
				
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				print("Enemy dies, oid:"..self.oid)
				
				
				if globIsInDemoMode == false then 
					enemyDie1Snd:stop()
					enemyDie1Snd:play()
				end
				
				self.deactivate(self)
			end
		end
	end
	
	--only if still active
	if self.active == true then
		self.executeFlyMove(self) --always as long as active: move along the x axis
		
		if self.plantShotFlag == true then
			self.plantShot(self)
			self.plantShotFlag = false --flamethrower effect when line is left out :-)
		end
	end
end

function EnemyG.prepareShoot(self, aSpdX, aSpdY, aAimPlyr, aParam1, aParam2)
	print("Prepare shooting")
	
	--keep info that shooting needs to occur separate from actual shooting.
	--do the shooting in next update
	self.plantShotFlag = true
	
end

function EnemyG.executeFlyMove(self)

	self.moveProgressX = self.moveProgressX + self.speedX
	
	while self.moveProgressX <= ((-1) * threshold) do --for negative x-Movement
		self.moveProgressX = self.moveProgressX + threshold
		self.x = self.x - 1
	end
	
	while self.moveProgressX >= threshold do --for negative x-Movement
		self.moveProgressX = self.moveProgressX - threshold
		self.x = self.x + 1
	end
	
	self.moveProgressY = self.moveProgressY + self.speedY
	
	while self.moveProgressY <= ((-1) * threshold) do --for negative x-Movement
		self.moveProgressY = self.moveProgressY + threshold
		self.y = self.y - 1
	end
	
	while self.moveProgressY >= threshold do --for negative x-Movement
		self.moveProgressY = self.moveProgressY - threshold
		self.y = self.y + 1
	end
	
	if self.reverseTimer > 0 then
		self.reverseTimer = self.reverseTimer - 1
	else
		self.reverseTimer = self.reverseTimerMax 
		self.speedY = self.speedY*(-1)
	end
	
	
	if self.x < self.boundsLeft then
		self.deactivate(self)
	end 
	
end

function EnemyG.plantShot(self)
	
	dummy = 1/0 --make it crash, since this enemy should never actually get a shooting command
	
end

function EnemyG.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end







