require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"
require "luas.point"
require "luas.catmullRomSpline"
require "luas.track"

EnemyH = Object:extend()


function EnemyH.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function EnemyH:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.boundsLeft  = (-1)*15
	
	self.roamIndex = 0 --invalid value
	self.roamTrack = nil
	self.roamGoodDist = 1.0 --default
	self.roamStretchFactor = 1.0
	
	--####################
	--# Shooting         #
	--####################
	
	self.plantShotFlag    = false
	
	self.oid = 0

	
end

function EnemyH.setOid(self, aOid)
	self.oid = aOid
end

function EnemyH.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function EnemyH.activate(self, aGoodDist, aStretchFactor)
	
	self.active = true
	
	self.currentGph = enmHGph
	
	self.scoreReward = 60
	
	self.enmWidth  = 15
	self.enmHeight = 8
	
	self.hitPoints = 3
	
	self.roamGoodDist = aGoodDist
	self.roamStretchFactor = aStretchFactor
	
	self.calculateTrack(self)
	
end


function EnemyH.calculateTrack(self)


	self.roamIndex = 1 --set to valid value for upcoming movements
			
	--create new roamtrack
	self.roamTrack = Track()
			
			
	--points for catmull-rom spline
	local lclP1X = 0
	local lclP1Y = 0
	local lclP2X = 0
	local lclP2Y = 0
	local lclP3X = 0
	local lclP3Y = 0
	local lclP4X = 0
	local lclP4Y = 0
	local lclP5X = 0
	local lclP5Y = 0
	local lclP6X = 0
	local lclP6Y = 0
	local lclP7X = 0
	local lclP7Y = 0
	
	--use relative coordinates relative to startPosX and startPosY, so it starts with 0
	--for the spline, have the first two points always the same, start with enemy position for the spline-track
	lclP1X = 0
	lclP1Y = 0--self.startPosY --should be at startPosY on beginning of roamingtack anyway? no, consider when reaching left or right boundaries at beginning of turning may start with different y
	lclP2X = 0
	lclP2Y = 0--self.startPosY	
	--
	lclP3X = (-1)*20
	lclP3Y = 40*self.roamStretchFactor
	lclP4X = (-1)*60
	lclP4Y = 80*self.roamStretchFactor
	lclP5X = (-1)*140
	lclP5Y = 120*self.roamStretchFactor
	lclP6X = (-1)*250
	lclP6Y = 130*self.roamStretchFactor
	--	
	lclP7X = lclP6X
	lclP7Y = lclP6Y
			
			
	--build the splines 
	local lclPts = {}
	lclPts[1] = Point()
	lclPts[2] = Point()
	lclPts[3] = Point()
	lclPts[4] = Point()
	lclPts[5] = Point()
	lclPts[6] = Point()
	lclPts[7] = Point()
	lclPts[1].putPosition(lclPts[1], lclP1X, lclP1Y)
	lclPts[2].putPosition(lclPts[2], lclP2X, lclP2Y)
	lclPts[3].putPosition(lclPts[3], lclP3X, lclP3Y)
	lclPts[4].putPosition(lclPts[4], lclP4X, lclP4Y)
	lclPts[5].putPosition(lclPts[5], lclP5X, lclP5Y)
	lclPts[6].putPosition(lclPts[6], lclP6X, lclP6Y)
	lclPts[7].putPosition(lclPts[7], lclP7X, lclP7Y)

	
	local lclSplines = {}
	--
	lclSplines[1] =  CatmullRomSpline()
	lclSplines[1].setGoodDist(lclSplines[1], self.roamGoodDist)
	lclSplines[1].setRefPoints(lclSplines[1], lclPts[1], lclPts[2], lclPts[3], lclPts[4])
	lclSplines[1].calculate(lclSplines[1])
	--
	lclSplines[2] =  CatmullRomSpline()
	lclSplines[2].setGoodDist(lclSplines[2], self.roamGoodDist)
	lclSplines[2].setRefPoints(lclSplines[2], lclPts[2], lclPts[3], lclPts[4], lclPts[5])
	lclSplines[2].calculate(lclSplines[2])
	--
	lclSplines[3] =  CatmullRomSpline()
	lclSplines[3].setGoodDist(lclSplines[3], self.roamGoodDist)
	lclSplines[3].setRefPoints(lclSplines[3], lclPts[3], lclPts[4], lclPts[5], lclPts[6])
	lclSplines[3].calculate(lclSplines[3])
	--
	lclSplines[4] =  CatmullRomSpline()
	lclSplines[4].setGoodDist(lclSplines[4], self.roamGoodDist)
	lclSplines[4].setRefPoints(lclSplines[4], lclPts[4], lclPts[5], lclPts[6], lclPts[7])
	lclSplines[4].calculate(lclSplines[4])
			
			
	--put splines together to track
	self.roamTrack.addSpline(self.roamTrack, lclSplines[1])
	self.roamTrack.addSpline(self.roamTrack, lclSplines[2])
	self.roamTrack.addSpline(self.roamTrack, lclSplines[3])
	self.roamTrack.addSpline(self.roamTrack, lclSplines[4])


end


function EnemyH.plantExplosion(self)

	globSmallExplosions[globSmallExplosionIndex].activate(globSmallExplosions[globSmallExplosionIndex], self.x + 5, self.y - 2)
	globSmallExplosionIndex = globSmallExplosionIndex + 1
	if globSmallExplosionIndex > globSmallExplosionMax then
		globSmallExplosionIndex = 1
	end
	
end

function EnemyH.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function EnemyH.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function EnemyH.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				print("Enemy dies, oid:"..self.oid)
				
				
				if globIsInDemoMode == false then 
					enemyDie4Snd:stop()
					enemyDie4Snd:play()
				end
				
				self.deactivate(self)
			else
				
				if globIsInDemoMode == false then 
					enemyHitSnd:stop()
					enemyHitSnd:play()
				end
			end
		end
	end
	
	--only if still active
	if self.active == true then
		self.executeFlyMove(self) --always as long as active: move along the x axis
		
		if self.plantShotFlag == true then
			self.plantShot(self)
			self.plantShotFlag = false --flamethrower effect when line is left out :-)
		end
	end
end

function EnemyH.prepareShoot(self, aSpdX, aSpdY, aAimPlyr, aParam1, aParam2)
	print("Prepare shooting")
	
	--keep info that shooting needs to occur separate from actual shooting.
	--do the shooting in next update
	self.plantShotFlag = true
	
end

function EnemyH.executeFlyMove(self)

	--do move relative to startpos
	self.x = self.startPosX + self.roamTrack.points[self.roamIndex].x
	self.y = self.startPosY + self.roamTrack.points[self.roamIndex].y

	self.roamIndex = self.roamIndex + 1
	
	if self.x < self.boundsLeft then
		self.deactivate(self)
	end 
	
end

function EnemyH.plantShot(self)
	
	local lclLaunchPX = self.x
	local lclLaunchPY = self.y + 3
	
	local lclXSpd = (-1)*2.2
	local lclYSpd = 0.0

	globEnmBulletsD[globEnmBulletsDIndex].activate(globEnmBulletsD[globEnmBulletsDIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
	globEnmBulletsDIndex = globEnmBulletsDIndex + 1
	if globEnmBulletsDIndex >= globEnmBulletsDIndexMax then
		globEnmBulletsDIndex = 1
	end
	
	if globIsInDemoMode == false then 
		shotSnd:stop()
		shotSnd:play()
	end

	
end

function EnemyH.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end







