require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"

EnemyI = Object:extend()


function EnemyI.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function EnemyI:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.boundsLeft  = (-1)*17
	
	self.speedX = 0.0 --take from levelsettings
	self.moveProgressX = 0.0
	
	--####################
	--# Shooting         #
	--####################
	
	self.plantShotFlag    = false
	
	self.oid = 0

	
end

function EnemyI.setOid(self, aOid)
	self.oid = aOid
end

function EnemyI.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function EnemyI.activate(self, aSpd)
	
	self.active = true
	
	self.currentGph = enmIGph
	
	self.scoreReward = 30
	
	self.enmWidth  = 17
	self.enmHeight = 7
	
	self.hitPoints = 1
	
	self.speedX = aSpd
	
end

function EnemyI.plantExplosion(self)

	globSmallExplosions[globSmallExplosionIndex].activate(globSmallExplosions[globSmallExplosionIndex], self.x + 5, self.y - 2)
	globSmallExplosionIndex = globSmallExplosionIndex + 1
	if globSmallExplosionIndex > globSmallExplosionMax then
		globSmallExplosionIndex = 1
	end
	
end

function EnemyI.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function EnemyI.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function EnemyI.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				print("Enemy dies, oid:"..self.oid)
				
				
				if globIsInDemoMode == false then 
					enemyDie2Snd:stop()
					enemyDie2Snd:play()
				end
				
				self.deactivate(self)
			end
		end
	end
	
	--only if still active
	if self.active == true then
		self.executeFlyMove(self) --always as long as active: move along the x axis
		
		if self.plantShotFlag == true then
			self.plantShot(self)
			self.plantShotFlag = false --flamethrower effect when line is left out :-)
		end
	end
end

function EnemyI.prepareShoot(self, aSpdX, aSpdY, aAimPlyr, aParam1, aParam2)
	print("Prepare shooting")
	
	--keep info that shooting needs to occur separate from actual shooting.
	--do the shooting in next update
	self.plantShotFlag = true
	
end

function EnemyI.executeFlyMove(self)

	--scrolling movement pixel
	self.x = self.x - 1

	self.moveProgressX = self.moveProgressX - self.speedX
	
	while self.moveProgressX <= ((-1) * threshold) do --for negative x-Movement
		self.moveProgressX = self.moveProgressX + threshold
		self.x = self.x - 1
	end
	
	if self.x < self.boundsLeft then
		self.deactivate(self)
	end 
	
end

function EnemyI.plantShot(self)
	
	local lclLaunchPX = self.x + 3
	local lclLaunchPY = self.y + 2
	
	local lclXSpd = (-1)*2.4
	local lclYSpd = 0.0

	globEnmBulletsB[globEnmBulletsBIndex].activate(globEnmBulletsB[globEnmBulletsBIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
	globEnmBulletsBIndex = globEnmBulletsBIndex + 1
	if globEnmBulletsBIndex >= globEnmBulletsBIndexMax then
		globEnmBulletsBIndex = 1
	end
	
	if globIsInDemoMode == false then 
		shotSnd:stop()
		shotSnd:play()
	end
	
end

function EnemyI.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end







