require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"

EnemyK = Object:extend()


function EnemyK.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function EnemyK:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.boundsLeft  = (-1)*26
	
	self.track = nil
	self.trackIndex = 0 --invalid values
	
	
	--####################
	--# Shooting         #
	--####################
	
	self.plantShotFlag    = false
	self.shootUpper = false
	self.shootLower = false
	
	
	self.oid = 0

	
end

function EnemyK.setOid(self, aOid)
	self.oid = aOid
end

function EnemyK.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function EnemyK.setTrack(self,aTrack)
	self.trackIndex = 1
	self.track = aTrack
end


function EnemyK.activate(self)
	
	self.active = true
	
	self.currentGph = enmKGph
	
	self.scoreReward = 140
	
	self.enmWidth  = 26
	self.enmHeight = 20
	
	self.hitPoints = 8
	
end

function EnemyK.plantExplosion(self)

	globMediumExplosions[globMediumExplosionIndex].activate(globMediumExplosions[globMediumExplosionIndex], self.x + 3, self.y - 3)
	globMediumExplosionIndex = globMediumExplosionIndex + 1
	if globMediumExplosionIndex > globMediumExplosionMax then
		globMediumExplosionIndex = 1
	end
	
	globMediumExplosions[globMediumExplosionIndex].activate(globMediumExplosions[globMediumExplosionIndex], self.x + 3, self.y + 12)
	globMediumExplosionIndex = globMediumExplosionIndex + 1
	if globMediumExplosionIndex > globMediumExplosionMax then
		globMediumExplosionIndex = 1
	end
	
end

function EnemyK.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function EnemyK.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function EnemyK.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				print("Enemy dies, oid:"..self.oid)
				
				
				if globIsInDemoMode == false then 
					enemyDie5Snd:stop()
					enemyDie5Snd:play()
				end
				
				self.deactivate(self)
			else
				
				if globIsInDemoMode == false then
					enemyHitSnd:stop()
					enemyHitSnd:play()
				end
			end
		end
	end
	
	--only if still active
	if self.active == true then
		self.executeFlyMove(self) --always as long as active: move along the x axis
		
		if self.plantShotFlag == true then
			self.plantShot(self)
			self.plantShotFlag = false --flamethrower effect when line is left out :-)
		end
	end
end

function EnemyK.prepareShoot(self, aSpdX, aSpdY, aAimPlyr, aParam1, aParam2)
	print("Prepare shooting")
	
	--keep info that shooting needs to occur separate from actual shooting.
	--do the shooting in next update
	self.plantShotFlag = true
	
	if aParam1 == 1 then
		self.shootUpper = true
		self.shootLower = false
	elseif aParam1 == 2 then
		self.shootUpper = true
		self.shootLower = true
	else
		self.shootUpper = false
		self.shootLower = true
	end
	
end

function EnemyK.executeFlyMove(self)

	--scrolling movement pixel
	self.x = self.track.points[self.trackIndex].x
	self.y = self.track.points[self.trackIndex].y

	self.trackIndex = self.trackIndex + 1
	
	if self.trackIndex > self.track.ptIndexMax then
		self.deactivate(self)
	end
	
	if self.x < self.boundsLeft then
		self.deactivate(self)
	end 
	
end

function EnemyK.plantShot(self)
	
	local lclLaunchPX = 0
	local lclLaunchPY = 0
	
	local lclXSpd = 0.0
	local lclYSpd = 0.0
	
	
	--self.shootUpper = false
	--self.shootLower = false
	
	if self.shootUpper == true then
	
		lclLaunchPX = self.x + 3
		lclLaunchPY = self.y + 2
	
		lclXSpd = (-1)*2.1
		lclYSpd = 0.0

		globEnmBulletsE[globEnmBulletsEIndex].activate(globEnmBulletsE[globEnmBulletsEIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
		globEnmBulletsEIndex = globEnmBulletsEIndex + 1
		if globEnmBulletsEIndex >= globEnmBulletsEIndexMax then
			globEnmBulletsEIndex = 1
		end
	end
	
	if self.shootLower == true then
	
		lclLaunchPX = self.x + 3
		lclLaunchPY = self.y + 13
	
		lclXSpd = (-1)*2.1
		lclYSpd = 0.0

		globEnmBulletsE[globEnmBulletsEIndex].activate(globEnmBulletsE[globEnmBulletsEIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
		globEnmBulletsEIndex = globEnmBulletsEIndex + 1
		if globEnmBulletsEIndex >= globEnmBulletsEIndexMax then
			globEnmBulletsEIndex = 1
		end
	end
	
	if globIsInDemoMode == false then 
		shotSnd:stop()
		shotSnd:play()
	end
	
end

function EnemyK.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end







