require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"

EnemyL = Object:extend()


function EnemyL.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function EnemyL:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.boundsLeft  = (-1)*23
	
	self.speedX = 0.0 --take from levelsettings
	self.moveProgressX = 0.0
	
	
	self.isStarting = false
	self.startTimer = 0
	self.isHalting = false
	self.haltTimer = 0
	self.isContinuing = false
	
	
	--####################
	--# Shooting         #
	--####################
	
	--self.plantShotFlag    = false
	
	--animation
	self.anmIndex = 0 --invalid value
	self.anmIndexMax = 2
	self.anmStep = 0 --ineffective value
	self.anmProgress = 0
	self.anmThresh = 10
	
	self.oid = 0

	
end

function EnemyL.setStartTimer(self, aTimer)
	self.startTimer = aTimer
end

function EnemyL.setOid(self, aOid)
	self.oid = aOid
end

function EnemyL.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function EnemyL.activate(self, aSpd)
	
	self.active = true
	
	self.isStarting = true
	
	self.anmIndex = 1 --valid value
	self.anmStep = 1 --effective value
	
	self.currentGph = enmLGph[self.anmIndex]
	
	self.scoreReward = 180
	
	self.enmWidth  = 23
	self.enmHeight = 13
	
	self.hitPoints = 7
	
	self.speedX = aSpd
	
end

function EnemyL.plantExplosion(self)

	globMediumExplosions[globMediumExplosionIndex].activate(globMediumExplosions[globMediumExplosionIndex], self.x + 1, self.y - 2)
	globMediumExplosionIndex = globMediumExplosionIndex + 1
	if globMediumExplosionIndex > globMediumExplosionMax then
		globMediumExplosionIndex = 1
	end
	
end

function EnemyL.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function EnemyL.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function EnemyL.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				print("Enemy dies, oid:"..self.oid)
				
				
				if globIsInDemoMode == false then 
					enemyDie7Snd:stop()
					enemyDie7Snd:play()
				end
				
				self.deactivate(self)
			else
				
				if globIsInDemoMode == false then
					enemyHitSnd:stop()
					enemyHitSnd:play()
				end
			end
		end
	end
	
	--only if still active
	if self.active == true then
	
		self.doAnimation(self)
	
		if self.isStarting == true then
			self.executeStartMove(self)
		end
		
		if self.isHalting == true then
			self.executeHaltMove(self)
		end
		
		if self.isContinuing == true then
			self.executeContinueMove(self)
		end
		
	end
end

function EnemyL.prepareShoot(self, aSpdX, aSpdY, aAimPlyr, aParam1, aParam2)
	--do nothing
end

function EnemyL.doAnimation(self)

	self.anmProgress = self.anmProgress + self.anmStep
	
	if self.anmProgress >= self.anmThresh then
		self.anmProgress = 0 
		self.anmIndex = self.anmIndex + 1
		
		if self.anmIndex > self.anmIndexMax then
			self.anmIndex = 1
		end
		
		self.currentGph = enmLGph[self.anmIndex]
	end

end

function EnemyL.executeStartMove(self)

	self.startTimer = self.startTimer - 1

	if self.startTimer > 0 then
	
		--scrolling movement pixel
		self.x = self.x - 1

		self.moveProgressX = self.moveProgressX - self.speedX
	
		while self.moveProgressX <= ((-1) * threshold) do --for negative x-Movement
			self.moveProgressX = self.moveProgressX + threshold
			self.x = self.x - 1
		end
	
		--if self.x < self.boundsLeft then --not on startmove
		--	self.deactivate(self)
		--end 
	
	else
		self.switchToHalt(self)
	end
	
end


function EnemyL.executeHaltMove(self)

	self.haltTimer = self.haltTimer - 1
	
	if self.haltTimer == 50 then
		self.plantShot(self, 1)
	elseif self.haltTimer == 42 then
		self.plantShot(self, 2)
	elseif self.haltTimer == 34 then
		self.plantShot(self, 3)
	elseif self.haltTimer == 26 then
		self.plantShot(self, 4)
	elseif self.haltTimer == 0 then
		self.switchToContinue(self)
	end
end

function EnemyL.executeContinueMove(self)
	
	--scrolling movement pixel
	self.x = self.x - 1

	self.moveProgressX = self.moveProgressX - self.speedX
	
	while self.moveProgressX <= ((-1) * threshold) do --for negative x-Movement
		self.moveProgressX = self.moveProgressX + threshold
		self.x = self.x - 1
	end
	
	if self.x < self.boundsLeft then --here it applies
		self.deactivate(self)
	end 

end

function EnemyL.switchToHalt(self)

	self.isStarting = false
	self.isHalting = true
	self.isContinuing = false
	
	self.haltTimer = 80
end

function EnemyL.switchToContinue(self)

	self.isStarting = false
	self.isHalting = false
	self.isContinuing = true
end

function EnemyL.plantShot(self, aAngleIndex)
	
	local lclLaunchPX = 0
	local lclLaunchPY = 0
	
	local lclXSpd = 0.0
	local lclYSpd = 0.0
	
	
	lclLaunchPX = self.x + 8
	lclLaunchPY = self.y + 2
	
	if aAngleIndex == 1 then
		lclXSpd = (-1)*0.75
		lclYSpd = (-1)*1.0
	elseif aAngleIndex == 2 then
		lclXSpd = (-1)*1.5
		lclYSpd = (-1)*1.0
	elseif aAngleIndex == 3 then
		lclXSpd = (-1)*3.0
		lclYSpd = (-1)*1.0
	else
		lclXSpd = (-1)*1.0
		lclYSpd = 0.0
	end
	
	local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
	
	lclXSpd = lclXSpd/lclVectorLength
	lclYSpd = lclYSpd/lclVectorLength
	
	lclXSpd = lclXSpd*2.4
	lclYSpd = lclYSpd*2.4

	globEnmBulletsB[globEnmBulletsBIndex].activate(globEnmBulletsB[globEnmBulletsBIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
	globEnmBulletsBIndex = globEnmBulletsBIndex + 1
	if globEnmBulletsBIndex >= globEnmBulletsBIndexMax then
		globEnmBulletsBIndex = 1
	end
	
	
	lclLaunchPX = self.x + 8
	lclLaunchPY = self.y + 8
	
	if aAngleIndex == 1 then
		lclXSpd = (-1)*0.75
		lclYSpd = 1.0
	elseif aAngleIndex == 2 then
		lclXSpd = (-1)*1.5
		lclYSpd = 1.0
	elseif aAngleIndex == 3 then
		lclXSpd = (-1)*3.0
		lclYSpd = 1.0
	else
		lclXSpd = (-1)*1.0
		lclYSpd = 0.0
	end
	
	local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
	
	lclXSpd = lclXSpd/lclVectorLength
	lclYSpd = lclYSpd/lclVectorLength
	
	lclXSpd = lclXSpd*2.4
	lclYSpd = lclYSpd*2.4

	globEnmBulletsB[globEnmBulletsBIndex].activate(globEnmBulletsB[globEnmBulletsBIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
	globEnmBulletsBIndex = globEnmBulletsBIndex + 1
	if globEnmBulletsBIndex >= globEnmBulletsBIndexMax then
		globEnmBulletsBIndex = 1
	end
	
end

function EnemyL.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end







