require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"
require "luas.point"
require "luas.catmullRomSpline"
require "luas.track"

EnemyM = Object:extend()


function EnemyM.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function EnemyM:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.boundsLeft  = (-1)*21
	
	self.speedX = 0.0 --take from levelsettings
	self.moveProgressX = 0.0
	
	
	self.isStarting = false
	self.isHalting = false
	self.haltTimer = 0
	self.isContinuing = false
	
	self.startOffset = 0
	self.startBowDown = false
	self.startTrackIndex = 0 --invalid value
	self.startTrack = nil
	self.startGoodDist = 1.0 --default
	
	--####################
	--# Shooting         #
	--####################
	
	self.plantShotFlag    = false
	self.shotSpdFactor = 0.0
	
	self.oid = 0

	
end

function EnemyM.setStartOffset(self, aOffset)
	self.startOffset = aOffset
end

function EnemyM.setStartBowDown(self, aBowDown)
	self.startBowDown = aBowDown
end

function EnemyM.setShotSpdFactor(self, aSSF)
	self.shotSpdFactor = aSSF
end

function EnemyM.setOid(self, aOid)
	self.oid = aOid
end

function EnemyM.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function EnemyM.activate(self)
	
	self.active = true
	
	self.isStarting = true
	
	self.currentGph = enmMGph
	
	self.scoreReward = 240
	
	self.enmWidth  = 21
	self.enmHeight = 11
	
	self.hitPoints = 10
	
	self.speedX = 1.7
	
	self.startGoodDist = 2.0
	
	self.x = self.x + self.startOffset
	self.startPosX = self.startPosX + self.startOffset
	
	self.calculateTrack(self)
	
end

function EnemyM.calculateTrack(self)
	
	
	
	self.startTrackIndex = 1 --set to valid value for upcoming movements
			
	--create new roamtrack
	self.startTrack = Track()
			
			
	--points for catmull-rom spline
	local lclP1X = 0
	local lclP1Y = 0
	local lclP2X = 0
	local lclP2Y = 0
	local lclP3X = 0
	local lclP3Y = 0
	local lclP4X = 0
	local lclP4Y = 0
	local lclP5X = 0
	local lclP5Y = 0
	local lclP6X = 0
	local lclP6Y = 0
	local lclP7X = 0
	local lclP7Y = 0
	
	--use relative coordinates relative to startPosX and startPosY, so it starts with 0
	--for the spline, have the first two points always the same, start with enemy position for the spline-track
	lclP1X = 0
	lclP1Y = 0--self.startPosY --should be at startPosY on beginning of roamingtack anyway? no, consider when reaching left or right boundaries at beginning of turning may start with different y
	lclP2X = 0
	lclP2Y = 0--self.startPosY	
	--
	lclP3X = (-1)*75
	lclP3Y = 0
	lclP4X = (-1)*84
	lclP4Y = 7
	lclP5X = (-1)*92
	lclP5Y = 18
	lclP6X = (-1)*100
	lclP6Y = 30
	--	
	lclP7X = lclP6X
	lclP7Y = lclP6Y
	
	
	if self.startBowDown == false then --invert bow upwards
		lclP1Y = lclP1Y*(-1)
		lclP2Y = lclP2Y*(-1)
		lclP3Y = lclP3Y*(-1)
		lclP4Y = lclP4Y*(-1)
		lclP5Y = lclP5Y*(-1)
		lclP6Y = lclP6Y*(-1)
		lclP7Y = lclP7Y*(-1)
	end
			
			
	--build the splines 
	local lclPts = {}
	lclPts[1] = Point()
	lclPts[2] = Point()
	lclPts[3] = Point()
	lclPts[4] = Point()
	lclPts[5] = Point()
	lclPts[6] = Point()
	lclPts[7] = Point()
	lclPts[1].putPosition(lclPts[1], lclP1X, lclP1Y)
	lclPts[2].putPosition(lclPts[2], lclP2X, lclP2Y)
	lclPts[3].putPosition(lclPts[3], lclP3X, lclP3Y)
	lclPts[4].putPosition(lclPts[4], lclP4X, lclP4Y)
	lclPts[5].putPosition(lclPts[5], lclP5X, lclP5Y)
	lclPts[6].putPosition(lclPts[6], lclP6X, lclP6Y)
	lclPts[7].putPosition(lclPts[7], lclP7X, lclP7Y)

	
	local lclSplines = {}
	--
	lclSplines[1] =  CatmullRomSpline()
	lclSplines[1].setGoodDist(lclSplines[1], self.startGoodDist)
	lclSplines[1].setRefPoints(lclSplines[1], lclPts[1], lclPts[2], lclPts[3], lclPts[4])
	lclSplines[1].calculate(lclSplines[1])
	--
	lclSplines[2] =  CatmullRomSpline()
	lclSplines[2].setGoodDist(lclSplines[2], self.startGoodDist)
	lclSplines[2].setRefPoints(lclSplines[2], lclPts[2], lclPts[3], lclPts[4], lclPts[5])
	lclSplines[2].calculate(lclSplines[2])
	--
	lclSplines[3] =  CatmullRomSpline()
	lclSplines[3].setGoodDist(lclSplines[3], self.startGoodDist)
	lclSplines[3].setRefPoints(lclSplines[3], lclPts[3], lclPts[4], lclPts[5], lclPts[6])
	lclSplines[3].calculate(lclSplines[3])
	--
	lclSplines[4] =  CatmullRomSpline()
	lclSplines[4].setGoodDist(lclSplines[4], self.startGoodDist)
	lclSplines[4].setRefPoints(lclSplines[4], lclPts[4], lclPts[5], lclPts[6], lclPts[7])
	lclSplines[4].calculate(lclSplines[4])
			
			
	--put splines together to track
	self.startTrack.addSpline(self.startTrack, lclSplines[1])
	self.startTrack.addSpline(self.startTrack, lclSplines[2])
	self.startTrack.addSpline(self.startTrack, lclSplines[3])
	self.startTrack.addSpline(self.startTrack, lclSplines[4])
	
	

end

function EnemyM.plantExplosion(self)

	globSmallExplosions[globSmallExplosionIndex].activate(globSmallExplosions[globSmallExplosionIndex], self.x, self.y)
	globSmallExplosionIndex = globSmallExplosionIndex + 1
	if globSmallExplosionIndex > globSmallExplosionMax then
		globSmallExplosionIndex = 1
	end
	
	globSmallExplosions[globSmallExplosionIndex].activate(globSmallExplosions[globSmallExplosionIndex], self.x + 10, self.y)
	globSmallExplosionIndex = globSmallExplosionIndex + 1
	if globSmallExplosionIndex > globSmallExplosionMax then
		globSmallExplosionIndex = 1
	end
	
end

function EnemyM.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function EnemyM.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function EnemyM.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				print("Enemy dies, oid:"..self.oid)
				
				
				if globIsInDemoMode == false then 
					enemyDie7Snd:stop()
					enemyDie7Snd:play()
				end
				
				self.deactivate(self)
			else
				
				if globIsInDemoMode == false then
					enemyHitSnd:stop()
					enemyHitSnd:play()
				end
			end
		end
	end
	
	--only if still active
	if self.active == true then
	
		if self.plantShotFlag == true then
			self.plantShotFlag = false
			self.plantShot(self)
		end
	
		if self.isStarting == true then
			self.executeStartMove(self)
		end
		
		if self.isHalting == true then
			self.executeHaltMove(self)
		end
		
		if self.isContinuing == true then
			self.executeContinueMove(self)
		end
		
	end
end

function EnemyM.prepareShoot(self, aSpdX, aSpdY, aAimPlyr, aParam1, aParam2)
	print("Prepare shooting")
	
	--keep info that shooting needs to occur separate from actual shooting.
	--do the shooting in next update
	self.plantShotFlag = true
	
end

function EnemyM.executeStartMove(self)

	--do move relative to startpos
	self.x = self.startPosX + self.startTrack.points[self.startTrackIndex].x
	self.y = self.startPosY + self.startTrack.points[self.startTrackIndex].y

	self.startTrackIndex = self.startTrackIndex + 1
	
	if self.startTrackIndex == self.startTrack.ptIndexMax then
		self.switchToHalt(self)
	end

	
end


function EnemyM.executeHaltMove(self)

	self.haltTimer = self.haltTimer - 1
	
	if self.haltTimer == 25 then
		self.plantShot(self)
	elseif self.haltTimer == 0 then
		self.switchToContinue(self)
	end
end

function EnemyM.executeContinueMove(self)
	
	--scrolling movement pixel
	self.x = self.x - 1

	self.moveProgressX = self.moveProgressX - self.speedX
	
	while self.moveProgressX <= ((-1) * threshold) do --for negative x-Movement
		self.moveProgressX = self.moveProgressX + threshold
		self.x = self.x - 1
	end
	
	if self.x < self.boundsLeft then --here it applies
		self.deactivate(self)
	end 

end

function EnemyM.switchToHalt(self)

	self.isStarting = false
	self.isHalting = true
	self.isContinuing = false
	
	self.haltTimer = 50
end

function EnemyM.switchToContinue(self)

	self.isStarting = false
	self.isHalting = false
	self.isContinuing = true
end

function EnemyM.plantShot(self)
	
	local lclLaunchPX = 0
	local lclLaunchPY = 0
	
	local lclXSpd = 0.0
	local lclYSpd = 0.0
	
	local lclSpdFactor = self.shotSpdFactor
	
	lclLaunchPX = self.x + 3
	lclLaunchPY = self.y + 3
	
	lclXSpd = (-1)*2.5*lclSpdFactor
	lclYSpd = 0.0
	
	--straight shot
	globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
	globEnmBulletsAIndex = globEnmBulletsAIndex + 1
	if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
		globEnmBulletsAIndex = 1
	end
	
	lclXSpd = (-1)*2.5*lclSpdFactor -  (-1)*0.2*lclSpdFactor
	lclYSpd = 1.5*lclSpdFactor
		
	--first left shot
	globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
	globEnmBulletsAIndex = globEnmBulletsAIndex + 1
	if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
		globEnmBulletsAIndex = 1
	end
	
	lclXSpd = (-1)*2.5*lclSpdFactor -  (-1)*0.4*lclSpdFactor
	lclYSpd = (-1)*1.5*lclSpdFactor
		
	--first right shot
	globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
	globEnmBulletsAIndex = globEnmBulletsAIndex + 1
	if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
		globEnmBulletsAIndex = 1
	end
	
	lclXSpd = (-1)*2.5*lclSpdFactor -  (-1)*0.2*lclSpdFactor
	lclYSpd = 3.0*lclSpdFactor
	
	--second left shot
	globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
	globEnmBulletsAIndex = globEnmBulletsAIndex + 1
	if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
		globEnmBulletsAIndex = 1
	end
	
	lclXSpd = (-1)*2.5*lclSpdFactor -  (-1)*0.2*lclSpdFactor
	lclYSpd = (-1)*3.0*lclSpdFactor
		
	--second right shot
	globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
	globEnmBulletsAIndex = globEnmBulletsAIndex + 1
	if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
		globEnmBulletsAIndex = 1
	end
	
	if globIsInDemoMode == false then 
		shotSnd:stop()
		shotSnd:play()
	end
	
end

function EnemyM.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end







