require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"

EnemyN = Object:extend()


function EnemyN.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function EnemyN:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.frames = {}
	self.frames[1] = enmNGph[1]
	self.frames[2] = enmNGph[2]
	self.frames[3] = enmNGph[3]
	self.frames[4] = enmNGph[4]
	self.frames[5] = enmNGph[3]
	self.frames[6] = enmNGph[2]
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.boundsTop  = (-1)*12
	self.boundsBottom  = 151
	
	self.speedY = 0.0 --take from levelsettings
	self.moveProgressY = 0.0
	
	
	self.isStarting = false
	self.isHalting = false
	self.haltTimer = 0
	self.isContinuing = false
	
	self.fromBelow = false
	
	
	--####################
	--# Shooting         #
	--####################
	
	--self.plantShotFlag    = false
	
	--animation
	self.anmIndex = 0 --invalid value
	self.anmIndexMax = 6
	self.anmStep = 0 --ineffective value
	self.anmProgress = 0
	self.anmThresh = 4
	self.anmOverflows = 0
	
	self.totalShotSpd = 0.0 --ineffective value
	self.ShotSpdX = {}
	self.ShotSpdY = {}
	self.ShotSpdX[1] = (-1)*2.0
	self.ShotSpdY[1] = 1.0
	self.ShotSpdX[2] = (-1)*1.0
	self.ShotSpdY[2] = 0.0
	self.ShotSpdX[3] = (-1)*2.0
	self.ShotSpdY[3] = (-1)*1.0
	self.ShotSpdX[4] = (-1)*1.0
	self.ShotSpdY[4] = (-1)*1.0
	self.ShotSpdX[5] = (-1)*1.0
	self.ShotSpdY[5] = (-1)*2.0
	self.ShotSpdX[6] = 0.0
	self.ShotSpdY[6] = (-1)*1.0
	self.ShotSpdX[7] = 1.0
	self.ShotSpdY[7] = (-1)*2.0
	self.ShotSpdX[8] = 1.0
	self.ShotSpdY[8] = (-1)*1.0
	self.ShotSpdX[9] = 2.0
	self.ShotSpdY[9] = (-1)*1.0
	self.ShotSpdX[10] = 1.0
	self.ShotSpdY[10] = 0.0
	self.ShotSpdX[11] = 2.0
	self.ShotSpdY[11] = 1.0
	
	self.oid = 0

	
end

function EnemyN.setOid(self, aOid)
	self.oid = aOid
end

function EnemyN.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function EnemyN.activate(self, aSpd)
	
	self.active = true
	
	self.isStarting = true
	
	self.totalShotSpd = 2.2 --chosen default value
	
	self.anmIndex = 1 --valid value
	self.anmStep = 1 --effective value
	
	self.currentGph = self.frames[self.anmIndex]
	
	self.scoreReward = 90
	
	self.enmWidth  = 9
	self.enmHeight = 12
	
	self.hitPoints = 1
	
	self.speedY = aSpd
	
	if self.speedY > 0 then 
		self.fromBelow = false
	else
		self.fromBelow = true
		self.speedY = self.speedY*(-1) --have speed always have positive value, direction information is in self.fromBelow
	end
	
	if self.fromBelow == false then --invert shot y directions when enemy is coming from above
		self.ShotSpdY[1] = self.ShotSpdY[1]*(-1)
		self.ShotSpdY[2] = self.ShotSpdY[2]*(-1)
		self.ShotSpdY[3] = self.ShotSpdY[3]*(-1)
		self.ShotSpdY[4] = self.ShotSpdY[4]*(-1)
		self.ShotSpdY[5] = self.ShotSpdY[5]*(-1)
		self.ShotSpdY[6] = self.ShotSpdY[6]*(-1)
		self.ShotSpdY[7] = self.ShotSpdY[7]*(-1)
		self.ShotSpdY[8] = self.ShotSpdY[8]*(-1)
		self.ShotSpdY[9] = self.ShotSpdY[9]*(-1)
		self.ShotSpdY[10] = self.ShotSpdY[10]*(-1)
		self.ShotSpdY[11] = self.ShotSpdY[11]*(-1)
	end
	
end

function EnemyN.plantExplosion(self)

	globSmallExplosions[globSmallExplosionIndex].activate(globSmallExplosions[globSmallExplosionIndex], self.x, self.y + 2)
	globSmallExplosionIndex = globSmallExplosionIndex + 1
	if globSmallExplosionIndex > globSmallExplosionMax then
		globSmallExplosionIndex = 1
	end
	
end

function EnemyN.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function EnemyN.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function EnemyN.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				print("Enemy dies, oid:"..self.oid)
				
				
				if globIsInDemoMode == false then 
					enemyDie8Snd:stop()
					enemyDie8Snd:play()
				end
				
				self.deactivate(self)
			end
		end
	end
	
	--only if still active
	if self.active == true then
	
		if self.isStarting == true then
			self.executeStartMove(self)
		end
		
		if self.isHalting == true then
			self.executeHaltMove(self)
		end
		
		if self.isContinuing == true then
			self.executeContinueMove(self)
		end
		
	end
end

function EnemyN.executeStartMove(self)

	--scrolling movement pixel
	self.x = self.x - 1
	
	if self.fromBelow == true then

		self.moveProgressY = self.moveProgressY - self.speedY
	
		while self.moveProgressY <= ((-1) * threshold) do --for negative x-Movement
			self.moveProgressY = self.moveProgressY + threshold
			self.y = self.y - 1
		end
	else
		self.moveProgressY = self.moveProgressY + self.speedY
	
		while self.moveProgressY >= threshold do --for negative x-Movement
			self.moveProgressY = self.moveProgressY - threshold
			self.y = self.y + 1
		end
	end
	
	--do animation and checking if we need to switch to different movement according to animation progress
	self.anmProgress = self.anmProgress + self.anmStep
	if self.anmProgress > self.anmThresh then
		--Coding error, would have to set anmProgress to zero here, but keep it like this because the effect is as desired
		self.anmIndex = self.anmIndex + 1
		
		if self.anmIndex > self.anmIndexMax then
			self.anmIndex = 1
			self.anmOverflows = self.anmOverflows + 1
		end
		
		self.currentGph = self.frames[self.anmIndex]
	end 
	
	if self.anmOverflows >= 2 then --after 2 animation cycles, halt
		self.switchToHalt(self)
	end
	
end


function EnemyN.executeHaltMove(self)

	--scrolling movement pixel
	self.x = self.x - 1

	self.haltTimer = self.haltTimer - 1
	
	if self.haltTimer == 20 then
		self.plantShot(self)
	elseif self.haltTimer == 0 then
		self.switchToContinue(self)
	end
end

function EnemyN.executeContinueMove(self)
	
	--scrolling movement pixel
	self.x = self.x - 1
	
	if self.fromBelow == true then
	
		self.moveProgressY = self.moveProgressY + self.speedY
	
		while self.moveProgressY >= threshold do --for negative x-Movement
			self.moveProgressY = self.moveProgressY - threshold
			self.y = self.y + 1
		end
		
	else
		
		self.moveProgressY = self.moveProgressY - self.speedY
	
		while self.moveProgressY <= ((-1) * threshold) do --for negative x-Movement
			self.moveProgressY = self.moveProgressY + threshold
			self.y = self.y - 1
		end
	end
	
	--do animation and checking if we need to switch to different movement according to animation progress
	self.anmProgress = self.anmProgress + self.anmStep
	if self.anmProgress > self.anmThresh then
		self.anmIndex = self.anmIndex + 1
		
		if self.anmIndex > self.anmIndexMax then
			self.anmIndex = 1
			--self.anmOverflows = self.anmOverflows + 1 --no need to count overflows anymore
		end
		
		self.currentGph = self.frames[self.anmIndex]
	end 
	
	if self.y < self.boundsTop then --here it applies
		self.deactivate(self)
	end 
	
	if self.y > self.boundsBottom then --here it applies
		self.deactivate(self)
	end 

end

function EnemyN.switchToHalt(self)

	self.isStarting = false
	self.isHalting = true
	self.isContinuing = false
	
	self.haltTimer = 40
end

function EnemyN.switchToContinue(self)

	self.isStarting = false
	self.isHalting = false
	self.isContinuing = true
end

function EnemyN.plantShot(self)
	
	local lclLaunchPX = 0
	local lclLaunchPY = 0
	

	local lclShotStartIndex = 0 --invalid value
	
	
	lclLaunchPX = self.x + 2
	lclLaunchPY = self.y + 4
	
	local lclDirX = globPlShip[1].x + 8 - lclLaunchPX
	local lclDirY = globPlShip[1].y + 6 - lclLaunchPY
	
	if self.fromBelow == true then
		if lclDirX <= 0 and lclDirY >= 0 then
			lclShotStartIndex = 1
		elseif  lclDirX >= 0 and lclDirY >= 0 then
			lclShotStartIndex = 9
		elseif lclDirX == 0 then
			lclShotStartIndex = 5
		else
		
			local lclDiv = lclDirY/lclDirX
			
			if lclDiv > 0.0 and lclDiv <= 0.5 then
				lclShotStartIndex = 2
			elseif lclDiv > 0.5 and lclDiv <= 1.5 then
				lclShotStartIndex = 3
			elseif lclDiv > 1.5 and lclDiv <= 3.0 then
				lclShotStartIndex = 4
			elseif lclDiv > 3.0 then
				lclShotStartIndex = 5
			elseif lclDiv < (-1)*3.0 then
				lclShotStartIndex = 5
			elseif lclDiv >= (-1)*3.0 and lclDiv < (-1)*1.5 then
				lclShotStartIndex = 6
			elseif lclDiv >= (-1)*1.5 and lclDiv < (-1)*0.5 then
				lclShotStartIndex = 7
			elseif lclDiv >= (-1)*0.5 and lclDiv < 0.0 then
				lclShotStartIndex = 8
			end
		
		end
	else
		if lclDirX <= 0 and lclDirY <= 0 then
			lclShotStartIndex = 1
		elseif  lclDirX >= 0 and lclDirY <= 0 then
			lclShotStartIndex = 9
		elseif lclDirX == 0 then
			lclShotStartIndex = 5
		else
		
			local lclDiv = lclDirY/lclDirX
			
			if lclDiv > 0.0 and lclDiv <= 0.5 then
				lclShotStartIndex = 8
			elseif lclDiv > 0.5 and lclDiv <= 1.5 then
				lclShotStartIndex = 7
			elseif lclDiv > 1.5 and lclDiv <= 3.0 then
				lclShotStartIndex = 6
			elseif lclDiv > 3.0 then
				lclShotStartIndex = 5
			elseif lclDiv < (-1)*3.0 then
				lclShotStartIndex = 5
			elseif lclDiv >= (-1)*3.0 and lclDiv < (-1)*1.5 then
				lclShotStartIndex = 4
			elseif lclDiv >= (-1)*1.5 and lclDiv < (-1)*0.5 then
				lclShotStartIndex = 3
			elseif lclDiv >= (-1)*0.5 and lclDiv < 0.0 then
				lclShotStartIndex = 2
			end
		
		end
	end
	
	
	if lclShotStartIndex > 0 then --valid direction determined
	
			local lclXSpd = 0.0
			local lclYSpd = 0.0
			
			local lclVectorLength = 1.0 --default
	
			--first bullet
			lclXSpd = self.ShotSpdX[lclShotStartIndex]
			lclYSpd = self.ShotSpdY[lclShotStartIndex]
			
			lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
			
			lclXSpd = lclXSpd/lclVectorLength
			lclYSpd = lclYSpd/lclVectorLength
			
			lclXSpd = lclXSpd*self.totalShotSpd
			lclYSpd = lclYSpd*self.totalShotSpd
	
			globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
			globEnmBulletsAIndex = globEnmBulletsAIndex + 1
			if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
				globEnmBulletsAIndex = 1
			end
			
			--second bullet
			lclXSpd = self.ShotSpdX[lclShotStartIndex + 1]
			lclYSpd = self.ShotSpdY[lclShotStartIndex + 1]
			
			lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
			
			lclXSpd = lclXSpd/lclVectorLength
			lclYSpd = lclYSpd/lclVectorLength
			
			lclXSpd = lclXSpd*self.totalShotSpd
			lclYSpd = lclYSpd*self.totalShotSpd
	
			globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
			globEnmBulletsAIndex = globEnmBulletsAIndex + 1
			if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
				globEnmBulletsAIndex = 1
			end
			
			--third bullet
			lclXSpd = self.ShotSpdX[lclShotStartIndex + 2]
			lclYSpd = self.ShotSpdY[lclShotStartIndex + 2]
			
			lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
			
			lclXSpd = lclXSpd/lclVectorLength
			lclYSpd = lclYSpd/lclVectorLength
			
			lclXSpd = lclXSpd*self.totalShotSpd
			lclYSpd = lclYSpd*self.totalShotSpd
	
			globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
			globEnmBulletsAIndex = globEnmBulletsAIndex + 1
			if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
				globEnmBulletsAIndex = 1
			end
	
	end
	
	if globIsInDemoMode == false then 
		shotSnd:stop()
		shotSnd:play()
	end
	
	
end

function EnemyN.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end







