require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"

EnemyO = Object:extend()


function EnemyO.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function EnemyO:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.boundsLeft  = (-1)*14
	
	self.speedX = 0.0 --take from levelsettings
	self.moveProgressX = 0.0
	
	self.speedY = 0.0
	self.moveProgressY = 0.0
	
	--####################
	--# Shooting         #
	--####################
	
	self.plantShotFlag    = false
	self.aimAtPlyrFlag    = false
	self.shotSpdX = 0.0
	self.shotSpdY = 0.0
	
	--up and down movement
	self.isMovingUp = false
	self.isMovingDown = false
	self.moveUpTimer = 0
	self.moveDownTimer = 0
	
	--animation
	self.frames = {}
	self.frames[1] = enmOGph[1]
	self.frames[2] = enmOGph[2]
	self.frames[3] = enmOGph[3]
	self.frames[4] = enmOGph[4]
	self.frames[5] = enmOGph[5]
	self.frames[6] = enmOGph[4]
	self.frames[7] = enmOGph[3]
	self.frames[8] = enmOGph[2]
	--
	self.anmIndex = 0 --invalid value
	self.anmIndexMax = 8
	self.anmStep = 0 --ineffective value
	self.anmProgress = 0
	self.anmThresh = 7
	
	
	self.startOffset = 0
	
	
	self.oid = 0

	
end

function EnemyO.setOid(self, aOid)
	self.oid = aOid
end

function EnemyO.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function EnemyO.setStartOffset(self, aOffset)
	self.startOffset = aOffset
end

function EnemyO.activate(self, aSpd)
	
	self.active = true
	
	self.anmIndex = 1 --valid value
	self.anmStep = 1  --effective value

	self.currentGph = self.frames[self.anmIndex]
	
	self.scoreReward = 70
	
	self.enmWidth  = 12
	self.enmHeight = 11
	
	self.hitPoints = 1
	
	self.speedX = aSpd
	
	self.speedY = 1.0 + aSpd --plus 1.0 because we want same speed vertically as horizontally and horizontally we also have 1 pixel per update extra in code below to follow scrolling
	
	self.isMovingUp = true
	self.moveUpTimer = 12
	
	self.x = self.x + self.startOffset
	self.startPosX = self.startPosX + self.startOffset
	
end

function EnemyO.plantExplosion(self)

	globSmallExplosions[globSmallExplosionIndex].activate(globSmallExplosions[globSmallExplosionIndex], self.x, self.y)
	globSmallExplosionIndex = globSmallExplosionIndex + 1
	if globSmallExplosionIndex > globSmallExplosionMax then
		globSmallExplosionIndex = 1
	end
	
end

function EnemyO.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function EnemyO.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function EnemyO.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				print("Enemy dies, oid:"..self.oid)
				
				
				if globIsInDemoMode == false then 
					enemyDie8Snd:stop()
					enemyDie8Snd:play()
				end
				
				self.deactivate(self)
			end
		end
	end
	
	--only if still active
	if self.active == true then
		self.doAnimation(self)
		self.executeFlyMove(self) --always as long as active: move along the x axis
		
		if self.plantShotFlag == true then
			self.plantShot(self)
			self.plantShotFlag = false --flamethrower effect when line is left out :-)
		end
	end
end

function EnemyO.prepareShoot(self, aSpdX, aSpdY, aAimPlyr, aParam1, aParam2)
	print("Prepare shooting")
	
	--keep info that shooting needs to occur separate from actual shooting.
	--do the shooting in next update
	self.plantShotFlag = true
	self.aimAtPlyrFlag    = aAimPlyr
	self.shotSpdX = aSpdX
	self.shotSpdY = aSpdY
	
end

function EnemyO.doAnimation(self)

	self.anmProgress = self.anmProgress + self.anmStep
	
	if self.anmProgress >= self.anmThresh then
		self.anmProgress = 0 
		self.anmIndex = self.anmIndex + 1
		
		if self.anmIndex > self.anmIndexMax then
			self.anmIndex = 1
		end
		
		self.currentGph = self.frames[self.anmIndex]
	end
end

function EnemyO.executeFlyMove(self)

	--vertical movement
	
	if self.isMovingUp == true then
	
		self.moveUpTimer = self.moveUpTimer - 1
		
		self.moveProgressY = self.moveProgressY - self.speedY
	
		while self.moveProgressY <= ((-1) * threshold) do --for negative x-Movement
			self.moveProgressY = self.moveProgressY + threshold
			self.y = self.y - 1
		end
		
		if self.moveUpTimer <= 0 then
			self.isMovingUp = false
			self.isMovingDown = true
			self.moveDownTimer = 24
		end 
	
	elseif self.isMovingDown == true then
	
		self.moveDownTimer = self.moveDownTimer - 1
		
		self.moveProgressY = self.moveProgressY + self.speedY
	
		while self.moveProgressY >= threshold do --for negative x-Movement
			self.moveProgressY = self.moveProgressY - threshold
			self.y = self.y + 1
		end
		
		if self.moveDownTimer <= 0 then
			self.isMovingUp = true
			self.isMovingDown = false
			self.moveUpTimer = 24
		end 
	
	end

	--horizontal movement
	--scrolling movement pixel
	self.x = self.x - 1

	self.moveProgressX = self.moveProgressX - self.speedX
	
	while self.moveProgressX <= ((-1) * threshold) do --for negative x-Movement
		self.moveProgressX = self.moveProgressX + threshold
		self.x = self.x - 1
	end
	
	if self.x < self.boundsLeft then
		self.deactivate(self)
	end 
	
end

function EnemyO.plantShot(self)
	
	local lclLaunchPX = self.x + 4 
	local lclLaunchPY = self.y + 3
	
	--self.aimAtPlyrFlag    = false
	--self.shotSpdX = 0.0
	--self.shotSpdY = 0.0
	
	local lclXSpd = 0.0
	local lclYSpd = 0.0
	local lclSkip = false
	
	if self.aimAtPlyrFlag == true then
	
		lclXSpd = globPlShip[1].x + 8 - lclLaunchPX
		lclYSpd = globPlShip[1].y + 6 - lclLaunchPY
		
		local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
		
		if lclVectorLength < epsilon then
			lclSkip = true
		else
			lclXSpd = lclXSpd/lclVectorLength
			lclYSpd = lclYSpd/lclVectorLength
			
			lclXSpd = lclXSpd*2.0
			lclYSpd = lclYSpd*2.0
		end
	
	else
		lclXSpd = self.shotSpdX
		lclYSpd = self.shotSpdY
		
		local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
		
		lclXSpd = lclXSpd/lclVectorLength
		lclYSpd = lclYSpd/lclVectorLength
			
		lclXSpd = lclXSpd*2.0
		lclYSpd = lclYSpd*2.0
	end
	
	if lclSkip == false then

		globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
		globEnmBulletsAIndex = globEnmBulletsAIndex + 1
		if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
			globEnmBulletsAIndex = 1
		end
	
	end
	
end

function EnemyO.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end







