require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"
require "luas.point"
require "luas.catmullRomSpline"
require "luas.track"

EnemyP = Object:extend()


function EnemyP.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function EnemyP:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.boundsLeft  = (-1)*17
	
	self.roamIndex = 0 --invalid value
	self.roamTrack = nil
	self.roamGoodDist = 1.0 --default
	self.roamStretchFactor = 1.0
	
	--####################
	--# Shooting         #
	--####################
	
	self.plantShotFlag    = false
	
	self.startOffset = 0
	
	self.oid = 0

	
end

function EnemyP.setOid(self, aOid)
	self.oid = aOid
end

function EnemyP.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function EnemyP.setStartOffset(self, aOffset)
	self.startOffset = aOffset
end

function EnemyP.activate(self, aGoodDist, aStretchFactor)
	
	self.active = true
	
	self.currentGph = enmPGph
	
	self.scoreReward = 250
	
	self.enmWidth  = 17
	self.enmHeight = 15
	
	self.hitPoints = 11
	
	self.roamGoodDist = aGoodDist
	self.roamStretchFactor = aStretchFactor
	
	self.x = self.x + self.startOffset
	self.startPosX = self.startPosX + self.startOffset
	
	self.calculateTrack(self)
	
end


function EnemyP.calculateTrack(self)


	self.roamIndex = 1 --set to valid value for upcoming movements
			
	--create new roamtrack
	self.roamTrack = Track()
			
			
	--points for catmull-rom spline
	local lclP1X = 0
	local lclP1Y = 0
	local lclP2X = 0
	local lclP2Y = 0
	local lclP3X = 0
	local lclP3Y = 0
	local lclP4X = 0
	local lclP4Y = 0
	local lclP5X = 0
	local lclP5Y = 0
	local lclP6X = 0
	local lclP6Y = 0
	local lclP7X = 0
	local lclP7Y = 0
	local lclP8X = 0
	local lclP8Y = 0
	local lclP9X = 0
	local lclP9Y = 0
	local lclP10X = 0
	local lclP10Y = 0
	local lclP11X = 0
	local lclP11Y = 0
	local lclP12X = 0
	local lclP12Y = 0
	
	--use relative coordinates relative to startPosX and startPosY, so it starts with 0
	--for the spline, have the first two points always the same, start with enemy position for the spline-track
	lclP1X = 0
	lclP1Y = 0--self.startPosY --should be at startPosY on beginning of roamingtack anyway? no, consider when reaching left or right boundaries at beginning of turning may start with different y
	lclP2X = 0
	lclP2Y = 0--self.startPosY	
	--
	lclP3X = 0
	lclP3Y = (-1)*25*self.roamStretchFactor
	--
	lclP4X = (-1)*10
	lclP4Y = (-1)*35*self.roamStretchFactor
	--
	lclP5X = (-1)*20
	lclP5Y = (-1)*35*self.roamStretchFactor
	--
	lclP6X = (-1)*30
	lclP6Y = (-1)*25*self.roamStretchFactor
	--
	lclP7X = (-1)*30
	lclP7Y = 25*self.roamStretchFactor
	--
	lclP8X = (-1)*40
	lclP8Y = 35*self.roamStretchFactor
	--
	lclP9X = (-1)*50
	lclP9Y = 35*self.roamStretchFactor
	--
	lclP10X = (-1)*60
	lclP10Y = 25*self.roamStretchFactor
	--
	lclP11X = (-1)*60
	lclP11Y = 0
	--	
	lclP12X = lclP11X
	lclP12Y = lclP11Y
			
			
	--build the splines 
	local lclPts = {}
	lclPts[1] = Point()
	lclPts[2] = Point()
	lclPts[3] = Point()
	lclPts[4] = Point()
	lclPts[5] = Point()
	lclPts[6] = Point()
	lclPts[7] = Point()
	lclPts[8] = Point()
	lclPts[9] = Point()
	lclPts[10] = Point()
	lclPts[11] = Point()
	lclPts[12] = Point()
	lclPts[1].putPosition(lclPts[1], lclP1X, lclP1Y)
	lclPts[2].putPosition(lclPts[2], lclP2X, lclP2Y)
	lclPts[3].putPosition(lclPts[3], lclP3X, lclP3Y)
	lclPts[4].putPosition(lclPts[4], lclP4X, lclP4Y)
	lclPts[5].putPosition(lclPts[5], lclP5X, lclP5Y)
	lclPts[6].putPosition(lclPts[6], lclP6X, lclP6Y)
	lclPts[7].putPosition(lclPts[7], lclP7X, lclP7Y)
	lclPts[8].putPosition(lclPts[8], lclP8X, lclP8Y)
	lclPts[9].putPosition(lclPts[9], lclP9X, lclP9Y)
	lclPts[10].putPosition(lclPts[10], lclP10X, lclP10Y)
	lclPts[11].putPosition(lclPts[11], lclP11X, lclP11Y)
	lclPts[12].putPosition(lclPts[12], lclP12X, lclP12Y)

	
	local lclSplines = {}
	--
	lclSplines[1] =  CatmullRomSpline()
	lclSplines[1].setGoodDist(lclSplines[1], self.roamGoodDist)
	lclSplines[1].setRefPoints(lclSplines[1], lclPts[1], lclPts[2], lclPts[3], lclPts[4])
	lclSplines[1].calculate(lclSplines[1])
	--
	lclSplines[2] =  CatmullRomSpline()
	lclSplines[2].setGoodDist(lclSplines[2], self.roamGoodDist)
	lclSplines[2].setRefPoints(lclSplines[2], lclPts[2], lclPts[3], lclPts[4], lclPts[5])
	lclSplines[2].calculate(lclSplines[2])
	--
	lclSplines[3] =  CatmullRomSpline()
	lclSplines[3].setGoodDist(lclSplines[3], self.roamGoodDist)
	lclSplines[3].setRefPoints(lclSplines[3], lclPts[3], lclPts[4], lclPts[5], lclPts[6])
	lclSplines[3].calculate(lclSplines[3])
	--
	lclSplines[4] =  CatmullRomSpline()
	lclSplines[4].setGoodDist(lclSplines[4], self.roamGoodDist)
	lclSplines[4].setRefPoints(lclSplines[4], lclPts[4], lclPts[5], lclPts[6], lclPts[7])
	lclSplines[4].calculate(lclSplines[4])
	--
	lclSplines[5] =  CatmullRomSpline()
	lclSplines[5].setGoodDist(lclSplines[5], self.roamGoodDist)
	lclSplines[5].setRefPoints(lclSplines[5], lclPts[5], lclPts[6], lclPts[7], lclPts[8])
	lclSplines[5].calculate(lclSplines[5])
	--
	lclSplines[6] =  CatmullRomSpline()
	lclSplines[6].setGoodDist(lclSplines[6], self.roamGoodDist)
	lclSplines[6].setRefPoints(lclSplines[6], lclPts[6], lclPts[7], lclPts[8], lclPts[9])
	lclSplines[6].calculate(lclSplines[6])
	--
	lclSplines[7] =  CatmullRomSpline()
	lclSplines[7].setGoodDist(lclSplines[7], self.roamGoodDist)
	lclSplines[7].setRefPoints(lclSplines[7], lclPts[7], lclPts[8], lclPts[9], lclPts[10])
	lclSplines[7].calculate(lclSplines[7])
	--
	lclSplines[8] =  CatmullRomSpline()
	lclSplines[8].setGoodDist(lclSplines[8], self.roamGoodDist)
	lclSplines[8].setRefPoints(lclSplines[8], lclPts[8], lclPts[9], lclPts[10], lclPts[11])
	lclSplines[8].calculate(lclSplines[8])
	--
	lclSplines[9] =  CatmullRomSpline()
	lclSplines[9].setGoodDist(lclSplines[9], self.roamGoodDist)
	lclSplines[9].setRefPoints(lclSplines[9], lclPts[9], lclPts[10], lclPts[11], lclPts[12])
	lclSplines[9].calculate(lclSplines[9])

	
			
	--put splines together to track
	self.roamTrack.addSpline(self.roamTrack, lclSplines[1])
	self.roamTrack.addSpline(self.roamTrack, lclSplines[2])
	self.roamTrack.addSpline(self.roamTrack, lclSplines[3])
	self.roamTrack.addSpline(self.roamTrack, lclSplines[4])
	self.roamTrack.addSpline(self.roamTrack, lclSplines[5])
	self.roamTrack.addSpline(self.roamTrack, lclSplines[6])
	self.roamTrack.addSpline(self.roamTrack, lclSplines[7])
	self.roamTrack.addSpline(self.roamTrack, lclSplines[8])
	self.roamTrack.addSpline(self.roamTrack, lclSplines[9])

end


function EnemyP.plantExplosion(self)

	globMediumExplosions[globMediumExplosionIndex].activate(globMediumExplosions[globMediumExplosionIndex], self.x, self.y - 1)
	globMediumExplosionIndex = globMediumExplosionIndex + 1
	if globMediumExplosionIndex > globMediumExplosionMax then
		globMediumExplosionIndex = 1
	end
	
end

function EnemyP.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function EnemyP.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function EnemyP.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				print("Enemy dies, oid:"..self.oid)
				
				
				if globIsInDemoMode == false then 
					enemyDie5Snd:stop()
					enemyDie5Snd:play()
				end
				
				self.deactivate(self)
			else
				
				if globIsInDemoMode == false then
					enemyHitSnd:stop()
					enemyHitSnd:play()
				end
			end
		end
	end
	
	--only if still active
	if self.active == true then
		self.executeFlyMove(self) --always as long as active: move along the x axis
		
		if self.plantShotFlag == true then
			self.plantShot(self)
			self.plantShotFlag = false --flamethrower effect when line is left out :-)
		end
	end
end

function EnemyP.prepareShoot(self, aSpdX, aSpdY, aAimPlyr, aParam1, aParam2)
	print("Prepare shooting")
	
	--keep info that shooting needs to occur separate from actual shooting.
	--do the shooting in next update
	self.plantShotFlag = true
	
end

function EnemyP.executeFlyMove(self)

	--do move relative to startpos
	self.x = self.startPosX + self.roamTrack.points[self.roamIndex].x
	self.y = self.startPosY + self.roamTrack.points[self.roamIndex].y

	if self.roamIndex == self.roamTrack.ptIndexMax then
		--start movement over
		self.roamIndex = 1
		self.startPosX = self.x
		self.startPosY = self.y
	else
		self.roamIndex = self.roamIndex + 1
	end
	
	if self.x < self.boundsLeft then
		self.deactivate(self)
	end 
	
end

function EnemyP.plantShot(self)
	
	local lclLaunchPX = self.x
	local lclLaunchPY = self.y + 5
	
	local lclXSpd = (-1)*2.4
	local lclYSpd = 0.0

	globEnmBulletsE[globEnmBulletsEIndex].activate(globEnmBulletsE[globEnmBulletsEIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
	globEnmBulletsEIndex = globEnmBulletsEIndex + 1
	if globEnmBulletsEIndex >= globEnmBulletsEIndexMax then
		globEnmBulletsEIndex = 1
	end
	
	if globIsInDemoMode == false then 
		shotSnd:stop()
		shotSnd:play()
	end

	
end

function EnemyP.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end







