require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"

EnemyQ = Object:extend()


function EnemyQ.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function EnemyQ:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.boundsLeft   = (-1)*11
	self.boundsTop    = (-1)*7
	self.boundsBottom = 151
	
	self.speedX = 0.0 --take from levelsettings
	self.moveProgressX = 0.0
	
	self.speedY = 0.0 --take from levelsettings
	self.moveProgressY = 0.0
	
	self.doesMovementY = false
	
	--####################
	--# Shooting         #
	--####################
	
	self.plantShotFlag    = false
	
	
	self.oid = 0

	
end

function EnemyQ.setOid(self, aOid)
	self.oid = aOid
end

function EnemyQ.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function EnemyQ.activate(self, aSpd)
	
	self.active = true
	
	self.currentGph = enmQGph
	
	self.scoreReward = 60
	
	self.enmWidth  = 11
	self.enmHeight = 7
	
	self.hitPoints = 1
	
	self.speedX = aSpd
	
end

function EnemyQ.plantExplosion(self)

	globSmallExplosions[globSmallExplosionIndex].activate(globSmallExplosions[globSmallExplosionIndex], self.x, self.y - 3)
	globSmallExplosionIndex = globSmallExplosionIndex + 1
	if globSmallExplosionIndex > globSmallExplosionMax then
		globSmallExplosionIndex = 1
	end
	
end

function EnemyQ.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function EnemyQ.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function EnemyQ.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				print("Enemy dies, oid:"..self.oid)
				
				
				if globIsInDemoMode == false then 
					enemyDie2Snd:stop()
					enemyDie2Snd:play()
				end
				
				self.deactivate(self)
			end
		end
	end
	
	--only if still active
	if self.active == true then
		self.executeFlyMove(self) --always as long as active: move along the x axis
		
		if self.plantShotFlag == true then
			self.plantShot(self)
			self.plantShotFlag = false --flamethrower effect when line is left out :-)
		end
	end
end

function EnemyQ.prepareShoot(self, aSpdX, aSpdY, aAimPlyr, aParam1, aParam2)
	print("Prepare shooting")
	
	--keep info that shooting needs to occur separate from actual shooting.
	--do the shooting in next update
	self.plantShotFlag = true
	
end

function EnemyQ.executeFlyMove(self)

	if self.doesMovementY == false then
		--scrolling movement pixel
		self.x = self.x - 1

		self.moveProgressX = self.moveProgressX - self.speedX
	
		while self.moveProgressX <= ((-1) * threshold) do --for negative x-Movement
			self.moveProgressX = self.moveProgressX + threshold
			self.x = self.x - 1
		end
		
		if globPlShip[1].active == true then
			if globPlShip[1].x >= (self.x - 12) and globPlShip[1].x <= (self.x + 6) then
			
				self.doesMovementY = true
			
				if globPlShip[1].y < (self.y + 3) then
					self.speedY = (-1)*2.5
				else
					self.speedY = 2.5
				end
			end
		end
	else
	
		self.moveProgressY = self.moveProgressY + self.speedY
	
		while self.moveProgressY <= ((-1) * threshold) do --for negative x-Movement
			self.moveProgressY = self.moveProgressY + threshold
			self.y = self.y - 1
		end
		
		while self.moveProgressY >= threshold do --for negative x-Movement
			self.moveProgressY = self.moveProgressY - threshold
			self.y = self.y + 1
		end
	
	end
	
	if self.x < self.boundsLeft then
		self.deactivate(self)
	end 
	
	if self.y < self.boundsTop then
		self.deactivate(self)
	end
	
	if self.y > self.boundsBottom then
		self.deactivate(self)
	end
	
end

function EnemyQ.plantShot(self)
	
	--this enemy doesn't shoot
	
end

function EnemyQ.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end







