require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"

EnemyS = Object:extend()


function EnemyS.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function EnemyS:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.boundsLeft  = (-1)*21
	
	self.speedX = 0.0 
	self.moveProgressX = 0.0
	
	self.speedY = 0.0 --for dangling
	self.moveProgressY = 0.0
	
	
	self.isStarting = false
	self.startTimer = 0
	self.isDangling = false
	self.dangleTimer = 0 
	self.isContinuing = false
	
	
	--####################
	--# Shooting         #
	--####################
	
	self.shootTimer = 0
	self.shootTimerMax = 30
	
	--####################
	--# deflecting/dangle#
	--####################
	
	self.deflection = 0	--magnitude/2 of deflection while dangling
	self.deflectMagnitude = 0 --total magnitude in complete move
	self.deflectProgress = 0
	self.deflectUp = false

	
	
	
	self.oid = 0

	
end

function EnemyS.setStartTimer(self, aTimer)
	self.startTimer = aTimer
end

function EnemyS.setDeflection(self, aDeflect)
	self.deflection = aDeflect
end

function EnemyS.setDangleTimer(self, aTimer)
	self.dangleTimer = aTimer
end

function EnemyS.setOid(self, aOid)
	self.oid = aOid
end

function EnemyS.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function EnemyS.activate(self, aSpd)
	
	self.active = true
	
	self.isStarting = true
	
	self.currentGph = enmSGph
	
	self.scoreReward = 230
	
	self.enmWidth  = 21
	self.enmHeight = 12
	
	self.hitPoints = 25
	
	self.speedX = aSpd
	
end

function EnemyS.plantExplosion(self)

	globMediumExplosions[globMediumExplosionIndex].activate(globMediumExplosions[globMediumExplosionIndex], self.x - 1, self.y - 1)
	globMediumExplosionIndex = globMediumExplosionIndex + 1
	if globMediumExplosionIndex > globMediumExplosionMax then
		globMediumExplosionIndex = 1
	end
	
	globMediumExplosions[globMediumExplosionIndex].activate(globMediumExplosions[globMediumExplosionIndex], self.x + 6, self.y - 1)
	globMediumExplosionIndex = globMediumExplosionIndex + 1
	if globMediumExplosionIndex > globMediumExplosionMax then
		globMediumExplosionIndex = 1
	end
	
end

function EnemyS.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function EnemyS.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function EnemyS.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				print("Enemy dies, oid:"..self.oid)
				
				
				if globIsInDemoMode == false then 
					enemyDie7Snd:stop()
					enemyDie7Snd:play()
				end
				
				self.deactivate(self)
			else
				
				if globIsInDemoMode == false then 
					enemyHitSnd:stop()
					enemyHitSnd:play()
				end
			end
		end
	end
	
	--only if still active
	if self.active == true then
	
		if self.isStarting == true then
			self.executeStartMove(self)
		end
		
		if self.isDangling == true then
			self.executeDangleMove(self)
		end
		
		if self.isContinuing == true then
			self.executeContinueMove(self)
		end
		
	end
end

function EnemyS.executeStartMove(self)

	self.startTimer = self.startTimer - 1

	if self.startTimer > 0 then
	
		--scrolling movement pixel
		self.x = self.x - 1

		self.moveProgressX = self.moveProgressX - self.speedX
	
		while self.moveProgressX <= ((-1) * threshold) do --for negative x-Movement
			self.moveProgressX = self.moveProgressX + threshold
			self.x = self.x - 1
		end
	
		--if self.x < self.boundsLeft then --not on startmove
		--	self.deactivate(self)
		--end 
	
	else
		self.switchToDangle(self)
	end
	
end


function EnemyS.executeDangleMove(self)

	self.dangleTimer = self.dangleTimer - 1
	
	if self.dangleTimer > 0 then
	
		self.deflectProgress = self.deflectProgress + 1
		
		if self.deflectProgress < self.deflectMagnitude then
			--move
			
			if self.deflectUp == true then
				self.speedY = (-1)*2.4
			else
				self.speedY = 2.4
			end
			
			self.moveProgressY = self.moveProgressY + self.speedY
	
			while self.moveProgressY <= ((-1) * threshold) do --for negative x-Movement
				self.moveProgressY = self.moveProgressY + threshold
				self.y = self.y - 1
			end
			
			while self.moveProgressY >= threshold do --for negative x-Movement
				self.moveProgressY = self.moveProgressY - threshold
				self.y = self.y + 1
			end
			
			--while moving, shoot
			self.shootTimer = self.shootTimer - 1
			
			if self.shootTimer <= 0 then
				self.shootTimer = self.shootTimerMax
				self.plantShot(self)
			end
			
		else
			--change direction
			self.deflectProgress = 0
			self.deflectMagnitude = 2*self.deflection --for any vertical movement except the first one, use twice the deflection
			
			if self.deflectUp == true then
				self.deflectUp = false
			else
				self.deflectUp = true
			end
		end
		
	--self.deflection = 0	--magnitude/2 of deflection while dangling
	--self.deflectMagnitude = 0 --total magnitude in complete move
	--self.deflectProgress = 0
	--self.deflectUp = false

	else
		self.switchToContinue(self)
	end
end

function EnemyS.executeContinueMove(self)
	
	--scrolling movement pixel
	self.x = self.x - 1

	self.moveProgressX = self.moveProgressX - self.speedX
	
	while self.moveProgressX <= ((-1) * threshold) do --for negative x-Movement
		self.moveProgressX = self.moveProgressX + threshold
		self.x = self.x - 1
	end
	
	if self.x < self.boundsLeft then --here it applies
		self.deactivate(self)
	end 

end

function EnemyS.switchToDangle(self)

	self.isStarting = false
	self.isDangling = true
	self.isContinuing = false
	
	--dangleTimer has been set from outside so it doesnt have to be set here
	
	self.deflectMagnitude = self.deflection --start movement with 1x deflection, later progress to 2x deflection for full deflection movement
	self.deflectUp = true --always start deflection by moving up
	
	self.shootTimer = self.shootTimerMax --dont shoot immediately
end

function EnemyS.switchToContinue(self)

	self.isStarting = false
	self.isDangling = false
	self.isContinuing = true
end

function EnemyS.plantShot(self)
	
	local lclLaunchPX = 0
	local lclLaunchPY = 0
	
	local lclXSpd = 0.0
	local lclYSpd = 0.0
	
	
	lclLaunchPX = self.x + 3
	lclLaunchPY = self.y + 6

	lclXSpd = (-1)*3.5

	globEnmBulletsB[globEnmBulletsBIndex].activate(globEnmBulletsB[globEnmBulletsBIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
	globEnmBulletsBIndex = globEnmBulletsBIndex + 1
	if globEnmBulletsBIndex >= globEnmBulletsBIndexMax then
		globEnmBulletsBIndex = 1
	end
	
end

function EnemyS.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end







