require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"

EnemyT = Object:extend()


function EnemyT.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function EnemyT:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.boundsLeft  = (-1)*9
	
	
	self.refPtX = 0
	self.refPtY = 0
	
	self.dirVecX = 0.0
	self.dirVecY = 0.0
	
	self.speed = 0.0
	self.moveProgressX = 0.0
	self.moveProgressY = 0.0
	
	--animation
	self.anmIndex = 0 --invalid value
	self.anmIndexMax = 4
	self.anmStep = 0 --ineffective value
	self.anmProgress = 0
	self.anmThresh = 8
	
	--####################
	--# Shooting         #
	--####################
	
	self.plantShotFlag    = false

	self.oid = 0
	
	self.randomSpeedFlag = false
	
end

function EnemyT.setOid(self, aOid)
	self.oid = aOid
end

function EnemyT.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function EnemyT.setRandomSpeedFlag(self, aRndSpdFlg)
	self.randomSpeedFlag = aRndSpdFlg
end

function EnemyT.activate(self, aRefY)
	
	self.active = true
	
	self.anmIndex = 1 --valid value
	self.anmStep = 1  --effective value
	
	self.currentGph = enmTGph[self.anmIndex]
	
	self.scoreReward = 100
	
	self.enmWidth  = 9
	self.enmHeight = 9
	
	self.hitPoints = 2
	
	self.speed = 1.5
	
	if self.randomSpeedFlag == true then
		self.speed = 1.1 + 0.6*love.math.random()
	end
	
	local lclRefY = 0
	if aRefY <= 0 then
		lclRefY = love.math.random(150)
	else
		lclRefY = aRefY
	end
	
	self.refPtX = 0
	self.refPtY = lclRefY
	
	self.dirVecX = self.refPtX - self.x
	self.dirVecY = self.refPtY - self.y
	
	local lclVectorLength = math.sqrt(self.dirVecX * self.dirVecX + self.dirVecY * self.dirVecY)
	
	self.dirVecX = self.dirVecX/lclVectorLength
	self.dirVecY = self.dirVecY/lclVectorLength
	
	self.dirVecX = self.dirVecX*self.speed
	self.dirVecY = self.dirVecY*self.speed
	
end

function EnemyT.plantExplosion(self)

	globSmallExplosions[globSmallExplosionIndex].activate(globSmallExplosions[globSmallExplosionIndex], self.x - 3, self.y - 3)
	globSmallExplosionIndex = globSmallExplosionIndex + 1
	if globSmallExplosionIndex > globSmallExplosionMax then
		globSmallExplosionIndex = 1
	end
	
end

function EnemyT.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function EnemyT.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function EnemyT.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
		
			--self.hitPoints = self.hitPoints - self.takenHits --no
			self.hitPoints = self.hitPoints - 1 --always only take 1 hitpoint damage so that regardless of players firepower it will always take 2 hits to kill this enemy
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				print("Enemy dies, oid:"..self.oid)
				
				
				if globIsInDemoMode == false then 
					enemyDie2Snd:stop()
					enemyDie2Snd:play()
				end
				
				self.deactivate(self)
			else
				self.currentGph = enmCGph[self.anmIndex]
				
				if globIsInDemoMode == false then 
					enemyHit2Snd:stop()
					enemyHit2Snd:play()
				end
			end
		end
	end
	
	--only if still active
	if self.active == true then
		self.doAnimation(self)
		self.executeFlyMove(self) --always as long as active: move along the x axis
		
		if self.plantShotFlag == true then
			self.plantShot(self)
			self.plantShotFlag = false --flamethrower effect when line is left out :-)
		end
	end
end

function EnemyT.prepareShoot(self, aSpdX, aSpdY, aAimPlyr, aParam1, aParam2)
	print("Prepare shooting")
	
	--keep info that shooting needs to occur separate from actual shooting.
	--do the shooting in next update
	self.plantShotFlag = true
	
end

function EnemyT.doAnimation(self)

	self.anmProgress = self.anmProgress + self.anmStep
	
	if self.anmProgress >= self.anmThresh then
	
		self.anmProgress = 0
		
		self.anmIndex = self.anmIndex + 1
		
		if self.anmIndex > self.anmIndexMax then
			self.anmIndex = 1
		end
		
		if self.hitPoints == 2 then
			self.currentGph = enmTGph[self.anmIndex]
		else
			self.currentGph = enmCGph[self.anmIndex]
		end
		
	end

end

function EnemyT.executeFlyMove(self)

	--scrolling movement pixel
	--self.x = self.x - 1 --not here

	self.moveProgressX = self.moveProgressX + self.dirVecX
		
	while self.moveProgressX >= threshold do --for positive x-Movement
		self.moveProgressX = self.moveProgressX - threshold
		self.x = self.x + 1
	end
	while self.moveProgressX <= ((-1) * threshold) do --for negative x-Movement
		self.moveProgressX = self.moveProgressX + threshold
		self.x = self.x - 1
	end
	
	self.moveProgressY = self.moveProgressY + self.dirVecY
		
	while self.moveProgressY >= threshold do --for positive y-Movement (not used in this game)
		self.moveProgressY = self.moveProgressY - threshold
		self.y = self.y + 1
	end
	while self.moveProgressY <= ((-1) * threshold) do --for negative y-Movement (standard case)
		self.moveProgressY = self.moveProgressY + threshold
		self.y = self.y - 1
	end
	
	if self.x < self.boundsLeft then
		self.deactivate(self)
	end 
	
end

function EnemyT.plantShot(self)
	
	dummy = 1/0 --make it crash, since this enemy should never actually get a shooting command
	
end

function EnemyT.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end







