require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"

EnemyU = Object:extend()


function EnemyU.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function EnemyU:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.boundsLeft  = (-1)*15
	
	self.speed = 0.0 --take from levelsettings
	self.startProgressX = 0.0
	self.startProgressY = 0.0
	self.continueProgressX = 0.0
	self.continueProgressY = 0.0
	
	self.isStarting = false
	self.isHalting = false
	self.haltTimer = 0
	self.isContinuing = false
	
	self.startDestPtX = 0
	self.startDestPtY = 0
	self.continueDestPtX = 0
	self.continueDestPtY = 0
	
	--####################
	--# Shooting         #
	--####################
	
	self.plantShotFlag    = false
	
	self.oid = 0

	
end

function EnemyU.setStartDestCoordinate(self, aDestPtX, aDestPtY)
	self.startDestPtX = aDestPtX
	self.startDestPtY = aDestPtY
end

function EnemyU.setContinueDestCoordinate(self, aDestPtY)
	self.continueDestPtX = (-1)*30 --fix value for x endpoint
	self.continueDestPtY = aDestPtY
end

function EnemyU.setOid(self, aOid)
	self.oid = aOid
end

function EnemyU.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function EnemyU.activate(self, aSpd)
	
	self.active = true
	
	self.isStarting = true
	
	self.currentGph = enmUGph
	
	self.scoreReward = 110
	
	self.enmWidth  = 15
	self.enmHeight = 17
	
	self.hitPoints = 5
	
	self.speed = aSpd
	
end

function EnemyU.plantExplosion(self)

	globMediumExplosions[globMediumExplosionIndex].activate(globMediumExplosions[globMediumExplosionIndex], self.x - 2, self.y - 2)
	globMediumExplosionIndex = globMediumExplosionIndex + 1
	if globMediumExplosionIndex > globMediumExplosionMax then
		globMediumExplosionIndex = 1
	end
	
end

function EnemyU.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function EnemyU.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function EnemyU.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				print("Enemy dies, oid:"..self.oid)
				
				
				if globIsInDemoMode == false then 
					enemyDie4Snd:stop()
					enemyDie4Snd:play()
				end
				
				self.deactivate(self)
			else
				
				if globIsInDemoMode == false then 
					enemyHitSnd:stop()
					enemyHitSnd:play()
				end
			end
		end
	end
	
	--only if still active
	if self.active == true then
	
		if self.plantShotFlag == true then
			self.plantShotFlag = false
			self.plantShot(self)
		end
	
		if self.isStarting == true then
			self.executeStartMove(self)
		end
		
		if self.isHalting == true then
			self.executeHaltMove(self)
		end
		
		if self.isContinuing == true then
			self.executeContinueMove(self)
		end
		
	end
end

function EnemyU.prepareShoot(self, aSpdX, aSpdY, aAimPlyr, aParam1, aParam2)
	self.plantShotFlag = true
end

function EnemyU.executeStartMove(self)

	local lclDirectionVectorX = self.startDestPtX - self.x
	local lclDirectionVectorY = self.startDestPtY - self.y --need no offsety
		
	local lclVectorLength = math.sqrt(lclDirectionVectorX * lclDirectionVectorX + lclDirectionVectorY * lclDirectionVectorY)
	
	if lclVectorLength < self.speed*1.5 then
		self.x = self.startDestPtX
		self.y = self.startDestPtY
		self.switchToHalt(self)
	else
		local lclSpeedX = (lclDirectionVectorX/lclVectorLength)*self.speed
		local lclSpeedY = (lclDirectionVectorY/lclVectorLength)*self.speed
		
		self.startProgressX = self.startProgressX + lclSpeedX
		
		while self.startProgressX >= threshold do --for positive x-Movement
			self.startProgressX = self.startProgressX - threshold
			self.x = self.x + 1
		end
		while self.startProgressX <= ((-1) * threshold) do --for negative x-Movement
			self.startProgressX = self.startProgressX + threshold
			self.x = self.x - 1
		end
		
		self.startProgressY = self.startProgressY + lclSpeedY
		
		while self.startProgressY >= threshold do --for positive y-Movement (not used in this game)
			self.startProgressY = self.startProgressY - threshold
			self.y = self.y + 1
		end
		while self.startProgressY <= ((-1) * threshold) do --for negative y-Movement (standard case)
			self.startProgressY = self.startProgressY + threshold
			self.y = self.y - 1
		end
		
	end
	
end


function EnemyU.executeHaltMove(self)

	self.haltTimer = self.haltTimer - 1
	
	if self.haltTimer == 10 then
		self.plantShot(self)
	elseif self.haltTimer == 0 then
		self.switchToContinue(self)
	end
end

function EnemyU.executeContinueMove(self)

	local lclDirectionVectorX = self.continueDestPtX - self.x
	local lclDirectionVectorY = self.continueDestPtY - self.y --need no offsety
		
	local lclVectorLength = math.sqrt(lclDirectionVectorX * lclDirectionVectorX + lclDirectionVectorY * lclDirectionVectorY)
	
	if lclVectorLength < self.speed*1.5 then
		self.x = self.continueDestPtX
		self.y = self.continueDestPtY
		self.deactivate(self) --just to be extra save,could theoretically get here when moving with very high speed, but deactivation usually happens below before reaching destination
	else
		local lclSpeedX = (lclDirectionVectorX/lclVectorLength)*self.speed
		local lclSpeedY = (lclDirectionVectorY/lclVectorLength)*self.speed
		
		self.continueProgressX = self.continueProgressX + lclSpeedX
		
		while self.continueProgressX >= threshold do --for positive x-Movement
			self.continueProgressX = self.continueProgressX - threshold
			self.x = self.x + 1
		end
		while self.continueProgressX <= ((-1) * threshold) do --for negative x-Movement
			self.continueProgressX = self.continueProgressX + threshold
			self.x = self.x - 1
		end
		
		self.continueProgressY = self.continueProgressY + lclSpeedY
		
		while self.continueProgressY >= threshold do --for positive y-Movement (not used in this game)
			self.continueProgressY = self.continueProgressY - threshold
			self.y = self.y + 1
		end
		while self.continueProgressY <= ((-1) * threshold) do --for negative y-Movement (standard case)
			self.continueProgressY = self.continueProgressY + threshold
			self.y = self.y - 1
		end
		
		if self.x < self.boundsLeft then
			self.deactivate(self)
		end
	end

end

function EnemyU.switchToHalt(self)

	self.isStarting = false
	self.isHalting = true
	self.isContinuing = false
	
	self.haltTimer = 20
end

function EnemyU.switchToContinue(self)

	self.isStarting = false
	self.isHalting = false
	self.isContinuing = true
end

function EnemyU.plantShot(self)
	
	local lclLaunchPX = 0
	local lclLaunchPY = 0
	
	local lclXSpd = (-1)*2.8
	local lclYSpd = 0.0
	
	
	lclLaunchPX = self.x 
	lclLaunchPY = self.y + 3

	globEnmBulletsB[globEnmBulletsBIndex].activate(globEnmBulletsB[globEnmBulletsBIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
	globEnmBulletsBIndex = globEnmBulletsBIndex + 1
	if globEnmBulletsBIndex >= globEnmBulletsBIndexMax then
		globEnmBulletsBIndex = 1
	end
	
	
	lclLaunchPX = self.x 
	lclLaunchPY = self.y + 11

	globEnmBulletsB[globEnmBulletsBIndex].activate(globEnmBulletsB[globEnmBulletsBIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
	globEnmBulletsBIndex = globEnmBulletsBIndex + 1
	if globEnmBulletsBIndex >= globEnmBulletsBIndexMax then
		globEnmBulletsBIndex = 1
	end
	
	if globIsInDemoMode == false then 
		shotSnd:stop()
		shotSnd:play()
	end
	
end

function EnemyU.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end







