require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"

EnemyV = Object:extend()


function EnemyV.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function EnemyV:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.boundsLeft  = (-1)*46
	
	self.speedX = 0.0 --take from levelsettings
	self.moveProgressX = 0.0
	
	
	self.isStarting = false
	self.startTimer = 0
	self.isHalting = false
	self.haltTimer = 0
	self.isContinuing = false
	
	self.cycles = 0
	
	
	--####################
	--# Shooting         #
	--####################
	
	--self.plantShotFlag    = false
	self.shotTimer = 0
	
	--animation
	self.anmIndex = 0 --invalid value
	self.anmIndexMax = 2
	self.anmStep = 0 --ineffective value
	self.anmProgress = 0
	self.anmThresh = 10
	
	self.oid = 0

	
end

function EnemyV.setStartTimer(self, aTimer)
	self.startTimer = aTimer
end

function EnemyV.setCycles(self, aCycles)
	self.cycles = aCycles
end

function EnemyV.setOid(self, aOid)
	self.oid = aOid
end

function EnemyV.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function EnemyV.activate(self, aSpd)
	
	self.active = true
	
	self.isStarting = true
	
	self.anmIndex = 1 --valid value
	self.anmStep = 1 --effective value
	
	self.currentGph = enmVGph[self.anmIndex]
	
	self.scoreReward = 400
	
	self.enmWidth  = 46
	self.enmHeight = 38
	
	self.hitPoints = 120
	
	self.speedX = aSpd
	
end

function EnemyV.plantExplosion(self)

	globBigExplosions[globBigExplosionIndex].activate(globBigExplosions[globBigExplosionIndex], self.x - 7, self.y - 7)
	globBigExplosionIndex = globBigExplosionIndex + 1
	if globBigExplosionIndex > globBigExplosionMax then
		globBigExplosionIndex = 1
	end
	
	globBigExplosions[globBigExplosionIndex].activate(globBigExplosions[globBigExplosionIndex], self.x - 7, self.y + 13)
	globBigExplosionIndex = globBigExplosionIndex + 1
	if globBigExplosionIndex > globBigExplosionMax then
		globBigExplosionIndex = 1
	end
	
	globBigExplosions[globBigExplosionIndex].activate(globBigExplosions[globBigExplosionIndex], self.x + 13, self.y - 7)
	globBigExplosionIndex = globBigExplosionIndex + 1
	if globBigExplosionIndex > globBigExplosionMax then
		globBigExplosionIndex = 1
	end
	
	globBigExplosions[globBigExplosionIndex].activate(globBigExplosions[globBigExplosionIndex], self.x + 13, self.y + 13)
	globBigExplosionIndex = globBigExplosionIndex + 1
	if globBigExplosionIndex > globBigExplosionMax then
		globBigExplosionIndex = 1
	end
	
end

function EnemyV.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function EnemyV.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function EnemyV.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				print("Enemy dies, oid:"..self.oid)
				
				
				if globIsInDemoMode == false then 
					enemyDie9Snd:stop()
					enemyDie9Snd:play()
				end
				
				self.deactivate(self)
			else
				
				if globIsInDemoMode == false then 
					enemyHitSnd:stop()
					enemyHitSnd:play()
				end
			end
		end
	end
	
	--only if still active
	if self.active == true then
	
		self.doAnimation(self)
	
		if self.isStarting == true then
			self.executeStartMove(self)
		end
		
		if self.isHalting == true then
			self.executeHaltMove(self)
		end
		
		if self.isContinuing == true then
			self.executeContinueMove(self)
		end
		
	end
end

function EnemyV.prepareShoot(self, aSpdX, aSpdY, aAimPlyr, aParam1, aParam2)
	--do nothing
end

function EnemyV.doAnimation(self)

	self.anmProgress = self.anmProgress + self.anmStep
	
	if self.anmProgress >= self.anmThresh then
		self.anmProgress = 0 
		self.anmIndex = self.anmIndex + 1
		
		if self.anmIndex > self.anmIndexMax then
			self.anmIndex = 1
		end
		
		self.currentGph = enmVGph[self.anmIndex]
	end

end

function EnemyV.executeStartMove(self)

	self.startTimer = self.startTimer - 1

	if self.startTimer > 0 then
	
		--scrolling movement pixel
		self.x = self.x - 1

		self.moveProgressX = self.moveProgressX - self.speedX
	
		while self.moveProgressX <= ((-1) * threshold) do --for negative x-Movement
			self.moveProgressX = self.moveProgressX + threshold
			self.x = self.x - 1
		end
	
		--if self.x < self.boundsLeft then --not on startmove
		--	self.deactivate(self)
		--end 
	
	else
		self.switchToHalt(self)
	end
	
end


function EnemyV.executeHaltMove(self)

	self.haltTimer = self.haltTimer - 1
	self.shotTimer = self.shotTimer - 1
	
	if self.haltTimer > 0 then
	
		if self.shotTimer <= 0 then
			self.shotTimer = 15
			self.plantShot(self)
		end
	else
		self.switchToContinue(self)
	end
end

function EnemyV.executeContinueMove(self)
	
	--scrolling movement pixel
	self.x = self.x - 1

	self.moveProgressX = self.moveProgressX - self.speedX
	
	while self.moveProgressX <= ((-1) * threshold) do --for negative x-Movement
		self.moveProgressX = self.moveProgressX + threshold
		self.x = self.x - 1
	end
	
	if self.x < self.boundsLeft then --here it applies
		self.deactivate(self)
	end 

end

function EnemyV.switchToHalt(self)

	self.isStarting = false
	self.isHalting = true
	self.isContinuing = false
	
	self.haltTimer = 15 + 15*self.cycles
	self.shotTimer = 15
end

function EnemyV.switchToContinue(self)

	self.isStarting = false
	self.isHalting = false
	self.isContinuing = true
end

function EnemyV.plantShot(self)
	
	local lclLaunchPX = 0
	local lclLaunchPY = 0
	
	local lclXSpd = 0.0
	local lclYSpd = 0.0
	
	
	lclLaunchPX = self.x + 20
	lclLaunchPY = self.y + 17
	
	local lclDirVecX = (-1)*20
	local lclDirVecY = (-1)*25 + love.math.random(50)
	
	local lclVectorLength = math.sqrt(lclDirVecX * lclDirVecX + lclDirVecY * lclDirVecY)
	
	lclXSpd = lclDirVecX/lclVectorLength
	lclYSpd = lclDirVecY/lclVectorLength
	
	lclXSpd = lclXSpd*2.6
	lclYSpd = lclYSpd*2.6

	globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
	globEnmBulletsAIndex = globEnmBulletsAIndex + 1
	if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
		globEnmBulletsAIndex = 1
	end
end

function EnemyV.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end







