require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"

EnemyX = Object:extend()


function EnemyX.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function EnemyX:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.boundsLeft  = (-1)*26
	
	self.speedX = 0.0 --take from levelsettings
	self.moveProgressX = 0.0
	
	
	self.isStarting = false
	self.startTimer = 0
	self.isHalting = false
	--self.haltTimer = 0 --not needed
	self.isContinuing = false
	
	
	--####################
	--# Shooting         #
	--####################
	
	--self.plantShotFlag    = false
	
	self.oid = 0

	
end

function EnemyX.setStartTimer(self, aTimer)
	self.startTimer = aTimer
end

function EnemyX.setOid(self, aOid)
	self.oid = aOid
end

function EnemyX.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function EnemyX.activate(self, aSpd)
	
	self.active = true
	
	self.isStarting = true
	
	self.currentGph = enmXGph
	
	self.scoreReward = 220
	
	self.enmWidth  = 26
	self.enmHeight = 11
	
	self.hitPoints = 12
	
	self.speedX = aSpd
	
end

function EnemyX.plantExplosion(self)

	globSmallExplosions[globSmallExplosionIndex].activate(globSmallExplosions[globSmallExplosionIndex], self.x, self.y)
	globSmallExplosionIndex = globSmallExplosionIndex + 1
	if globSmallExplosionIndex > globSmallExplosionMax then
		globSmallExplosionIndex = 1
	end
	
	globSmallExplosions[globSmallExplosionIndex].activate(globSmallExplosions[globSmallExplosionIndex], self.x + 14, self.y)
	globSmallExplosionIndex = globSmallExplosionIndex + 1
	if globSmallExplosionIndex > globSmallExplosionMax then
		globSmallExplosionIndex = 1
	end
	
end

function EnemyX.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function EnemyX.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function EnemyX.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				print("Enemy dies, oid:"..self.oid)
				
				
				if globIsInDemoMode == false then 
					enemyDie6Snd:stop()
					enemyDie6Snd:play()
				end
				
				self.deactivate(self)
			else
				
				if globIsInDemoMode == false then 
					enemyHitSnd:stop()
					enemyHitSnd:play()
				end
			end
		end
	end
	
	--only if still active
	if self.active == true then
	
		if self.isStarting == true then
			self.executeStartMove(self)
		end
		
		if self.isHalting == true then
			self.executeHaltMove(self)
		end
		
		if self.isContinuing == true then
			self.executeContinueMove(self)
		end
		
	end
end

function EnemyX.prepareShoot(self, aSpdX, aSpdY, aAimPlyr, aParam1, aParam2)
	--do nothing
end

function EnemyX.executeStartMove(self)

	self.startTimer = self.startTimer - 1

	if self.startTimer > 0 then
	
		--scrolling movement pixel
		self.x = self.x - 1

		self.moveProgressX = self.moveProgressX - self.speedX
	
		while self.moveProgressX <= ((-1) * threshold) do --for negative x-Movement
			self.moveProgressX = self.moveProgressX + threshold
			self.x = self.x - 1
		end
	
		--if self.x < self.boundsLeft then --not on startmove
		--	self.deactivate(self)
		--end 
	
	else
		self.switchToHalt(self)
	end
	
end


function EnemyX.executeHaltMove(self)

	self.plantShot(self)

	self.switchToContinue(self)
end

function EnemyX.executeContinueMove(self)
	
	--scrolling movement pixel
	self.x = self.x - 1

	self.moveProgressX = self.moveProgressX - self.speedX
	
	while self.moveProgressX <= ((-1) * threshold) do --for negative x-Movement
		self.moveProgressX = self.moveProgressX + threshold
		self.x = self.x - 1
	end
	
	if self.x < self.boundsLeft then --here it applies
		self.deactivate(self)
	end 

end

function EnemyX.switchToHalt(self)

	self.isStarting = false
	self.isHalting = true
	self.isContinuing = false
	
	--self.haltTimer = 80 --not needed, only 1 frame halt
end

function EnemyX.switchToContinue(self)

	self.isStarting = false
	self.isHalting = false
	self.isContinuing = true
	
	self.speedX = 2.9
end

function EnemyX.plantShot(self)
	
	local lclLaunchPX = 0
	local lclLaunchPY = 0
	
	local lclXSpd = 0.0
	local lclYSpd = 0.0
	
	
	lclLaunchPX = self.x + 4
	lclLaunchPY = self.y - 4
	
	lclXSpd = (-1)*1.2
	lclYSpd = (-1)*0.75

	globEnmBulletsF[globEnmBulletsFIndex].activate(globEnmBulletsF[globEnmBulletsFIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
	globEnmBulletsFIndex = globEnmBulletsFIndex + 1
	if globEnmBulletsFIndex >= globEnmBulletsFIndexMax then
		globEnmBulletsFIndex = 1
	end
	
	lclLaunchPX = self.x + 4
	lclLaunchPY = self.y + 8
	
	lclXSpd = (-1)*1.2
	lclYSpd = 0.75
	
	globEnmBulletsF[globEnmBulletsFIndex].activate(globEnmBulletsF[globEnmBulletsFIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
	globEnmBulletsFIndex = globEnmBulletsFIndex + 1
	if globEnmBulletsFIndex >= globEnmBulletsFIndexMax then
		globEnmBulletsFIndex = 1
	end
	
	lclLaunchPX = self.x + 7
	lclLaunchPY = self.y - 8
	
	lclXSpd = (-1)*1.0
	lclYSpd = (-1)*1.5
	
	globEnmBulletsF[globEnmBulletsFIndex].activate(globEnmBulletsF[globEnmBulletsFIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
	globEnmBulletsFIndex = globEnmBulletsFIndex + 1
	if globEnmBulletsFIndex >= globEnmBulletsFIndexMax then
		globEnmBulletsFIndex = 1
	end
	
	lclLaunchPX = self.x + 7
	lclLaunchPY = self.y + 12
	
	lclXSpd = (-1)*1.0
	lclYSpd = 1.5
	
	globEnmBulletsF[globEnmBulletsFIndex].activate(globEnmBulletsF[globEnmBulletsFIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
	globEnmBulletsFIndex = globEnmBulletsFIndex + 1
	if globEnmBulletsFIndex >= globEnmBulletsFIndexMax then
		globEnmBulletsFIndex = 1
	end
	
	if globIsInDemoMode == false then 
		shotSnd:stop()
		shotSnd:play()
	end
	
end

function EnemyX.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end







