require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"

EnemyY = Object:extend()


function EnemyY.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function EnemyY:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.boundsLeft  = (-1)*10
	
	self.speedX = 0.0 --take from levelsettings
	self.moveProgressX = 0.0
	self.speedY = 0.0
	self.moveProgressY = 0.0
	
	self.magnitudeY = 0.0
	self.isUpsideDown = false
	
	self.bounceOffCounter = 0
	self.bounceBoundsLeft = 0
	self.bounceBoundsRight = 190
	self.bounceBoundsTop = 0
	self.bounceBoundsBottom = 138
	
	--####################
	--# Shooting         #
	--####################
	
	self.plantShotFlag    = false
	
	
	self.oid = 0

	
end

function EnemyY.setOid(self, aOid)
	self.oid = aOid
end

function EnemyY.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function EnemyY.setMagnitude(self, aMagn)
	self.magnitudeY = aMagn
end

function EnemyY.setUpsideDown(self, aUpDownFlag)
	self.isUpsideDown = aUpDownFlag
end


function EnemyY.activate(self, aSpd)
	
	self.active = true
	
	if self.isUpsideDown == true then
		self.currentGph = enmYGph[2]
	else
		self.currentGph = enmYGph[1]
	end
	
	self.scoreReward = 100
	
	self.enmWidth  = 10
	self.enmHeight = 12
	
	self.hitPoints = 6
	
	self.speedX = aSpd
	
end

function EnemyY.plantExplosion(self)

	globMediumExplosions[globMediumExplosionIndex].activate(globMediumExplosions[globMediumExplosionIndex], self.x - 4, self.y)
	globMediumExplosionIndex = globMediumExplosionIndex + 1
	if globMediumExplosionIndex > globMediumExplosionMax then
		globMediumExplosionIndex = 1
	end
	
end

function EnemyY.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function EnemyY.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function EnemyY.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				print("Enemy dies, oid:"..self.oid)
				
				
				if globIsInDemoMode == false then 
					enemyDie1Snd:stop()
					enemyDie1Snd:play()
				end
				
				self.deactivate(self)
			else
				if globIsInDemoMode == false then 
					enemyHitSnd:stop()
					enemyHitSnd:play()
				end
			end
		end
	end
	
	--only if still active
	if self.active == true then
		self.executeJumpMove(self) --always as long as active: move along the x axis
		
		if self.plantShotFlag == true then
			self.plantShot(self)
			self.plantShotFlag = false --flamethrower effect when line is left out :-)
		end
	end
end

function EnemyY.prepareShoot(self, aSpdX, aSpdY, aAimPlyr, aParam1, aParam2)
	print("Prepare shooting")
	
	--keep info that shooting needs to occur separate from actual shooting.
	--do the shooting in next update
	self.plantShotFlag = true
	
end

function EnemyY.executeJumpMove(self)

	self.moveProgressX = self.moveProgressX - self.speedX
	
	while self.moveProgressX <= ((-1) * threshold) do --for negative x-Movement
		self.moveProgressX = self.moveProgressX + threshold
		self.x = self.x - 1
	end
	
	while self.moveProgressX >= threshold do --for negative x-Movement
		self.moveProgressX = self.moveProgressX - threshold
		self.x = self.x + 1
	end
	
	if self.bounceOffCounter == 0 then
		if self.x <= self.bounceBoundsLeft then
			self.bounceOffCounter = 1
			self.speedX = self.speedX*(-1)
		end
	elseif self.bounceOffCounter == 1 then
		if self.x >= self.bounceBoundsRight then
			self.bounceOffCounter = 2
			self.speedX = self.speedX*(-1)
		end
	end
	
	if self.isUpsideDown == true then
	
		self.speedY = self.speedY - 0.2
		
		self.moveProgressY = self.moveProgressY + self.speedY
		
		while self.moveProgressY <= ((-1) * threshold) do --for negative x-Movement
			self.moveProgressY = self.moveProgressY + threshold
			self.y = self.y - 1
		end
	
		while self.moveProgressY >= threshold do --for negative x-Movement
			self.moveProgressY = self.moveProgressY - threshold
			self.y = self.y + 1
		end
	
		if self.y <= self.bounceBoundsTop then
			self.speedY = self.magnitudeY
		end
	else
		self.speedY = self.speedY - 0.2
		
		self.moveProgressY = self.moveProgressY - self.speedY
		
		while self.moveProgressY <= ((-1) * threshold) do --for negative x-Movement
			self.moveProgressY = self.moveProgressY + threshold
			self.y = self.y - 1
		end
	
		while self.moveProgressY >= threshold do --for negative x-Movement
			self.moveProgressY = self.moveProgressY - threshold
			self.y = self.y + 1
		end
	
		if self.y >= self.bounceBoundsBottom then
			self.speedY = self.magnitudeY
		end
	end
	
	
	if self.x < self.boundsLeft then
		self.deactivate(self)
	end 
	
end

function EnemyY.plantShot(self)
	
	local lclLaunchPX = self.x + 3 
	local lclLaunchPY = self.y + 4
	
	--self.aimAtPlyrFlag    = false
	--self.shotSpdX = 0.0
	--self.shotSpdY = 0.0
	
	local lclXSpd = 0.0
	local lclYSpd = 0.0
	local lclSkip = false
	
	
	lclXSpd = globPlShip[1].x + 8 - lclLaunchPX
	lclYSpd = globPlShip[1].y + 6 - lclLaunchPY
		
	local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
		
	if lclVectorLength < epsilon then
		lclSkip = true
	else
		lclXSpd = lclXSpd/lclVectorLength
		lclYSpd = lclYSpd/lclVectorLength
			
		lclXSpd = lclXSpd*2.3
		lclYSpd = lclYSpd*2.3
	end
	
	if lclSkip == false then

		globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
		globEnmBulletsAIndex = globEnmBulletsAIndex + 1
		if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
			globEnmBulletsAIndex = 1
		end
	
	end
	
end

function EnemyY.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end







