require "luas.shortcuts"
require "luas.extendedLoader"
require "luas.boundingBox"

EnemyZ = Object:extend()


function EnemyZ.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function EnemyZ:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0 --taken hit since last update? insert number hitpoints to be taken

	self.scoreReward = 0                    --see defineColorType
	
	self.enmWidth  = 0 
	self.enmHeight = 0 
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.boundsLeft  = (-1)*50
	
	self.speedX = 0.0 --take from levelsettings
	self.moveProgressX = 0.0
	
	
	self.isStarting = false
	self.startTimer = 0
	self.isHalting = false
	self.haltTimer = 0
	self.isContinuing = false
	
	self.cycles = 0
	
	
	--####################
	--# Shooting         #
	--####################
	
	self.plantShotFlag    = false
	self.shotSpdX = 0.0
	self.shotSpdY = 0.0
	--self.shotTimer = 0
	
	--animation
	self.anmIndex = 0 --invalid value
	self.anmIndexMax = 2
	self.anmStep = 0 --ineffective value
	self.anmProgress = 0
	self.anmThresh = 10
	
	self.oid = 0

	
end

function EnemyZ.setStartTimer(self, aTimer)
	self.startTimer = aTimer
end

function EnemyZ.setCycles(self, aCycles)
	self.cycles = aCycles
end

function EnemyZ.setOid(self, aOid)
	self.oid = aOid
end

function EnemyZ.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function EnemyZ.activate(self, aSpd)
	
	self.active = true
	
	self.isStarting = true
	
	self.anmIndex = 1 --valid value
	self.anmStep = 1 --effective value
	
	self.currentGph = enmZGph[self.anmIndex]
	
	self.scoreReward = 300
	
	self.enmWidth  = 50
	self.enmHeight = 20
	
	self.hitPoints = 100
	
	self.speedX = aSpd
	
end

function EnemyZ.plantExplosion(self)

	globBigExplosions[globBigExplosionIndex].activate(globBigExplosions[globBigExplosionIndex], self.x, self.y - 7)
	globBigExplosionIndex = globBigExplosionIndex + 1
	if globBigExplosionIndex > globBigExplosionMax then
		globBigExplosionIndex = 1
	end
	
	globBigExplosions[globBigExplosionIndex].activate(globBigExplosions[globBigExplosionIndex], self.x + 15, self.y - 7)
	globBigExplosionIndex = globBigExplosionIndex + 1
	if globBigExplosionIndex > globBigExplosionMax then
		globBigExplosionIndex = 1
	end
	
end

function EnemyZ.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function EnemyZ.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function EnemyZ.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then
				self.plantExplosion(self)
				
				globAddScore(self.scoreReward)
				--bugfix: add score BEFORE deactivation
				
				print("Enemy dies, oid:"..self.oid)
				
				
				if globIsInDemoMode == false then 
					enemyDie10Snd:stop()
					enemyDie10Snd:play()
				end
				
				self.deactivate(self)
			else
				
				if globIsInDemoMode == false then 
					enemyHitSnd:stop()
					enemyHitSnd:play()
				end
			end
		end
	end
	
	--only if still active
	if self.active == true then
	
		self.doAnimation(self)
		
		if self.plantShotFlag == true then
			self.plantShotFlag = false
			self.plantShot(self)
		end
	
		if self.isStarting == true then
			self.executeStartMove(self)
		end
		
		if self.isHalting == true then
			self.executeHaltMove(self)
		end
		
		if self.isContinuing == true then
			self.executeContinueMove(self)
		end
		
	end
end

function EnemyZ.prepareShoot(self, aSpdX, aSpdY, aAimPlyr, aParam1, aParam2)
	self.plantShotFlag = true
	self.shotSpdX = aSpdX
	self.shotSpdY = aSpdY
end

function EnemyZ.doAnimation(self)

	self.anmProgress = self.anmProgress + self.anmStep
	
	if self.anmProgress >= self.anmThresh then
		self.anmProgress = 0 
		self.anmIndex = self.anmIndex + 1
		
		if self.anmIndex > self.anmIndexMax then
			self.anmIndex = 1
		end
		
		self.currentGph = enmZGph[self.anmIndex]
	end

end

function EnemyZ.executeStartMove(self)

	self.startTimer = self.startTimer - 1

	if self.startTimer > 0 then
	
		--scrolling movement pixel
		self.x = self.x - 1

		self.moveProgressX = self.moveProgressX - self.speedX
	
		while self.moveProgressX <= ((-1) * threshold) do --for negative x-Movement
			self.moveProgressX = self.moveProgressX + threshold
			self.x = self.x - 1
		end
	
		--if self.x < self.boundsLeft then --not on startmove
		--	self.deactivate(self)
		--end 
	
	else
		self.switchToHalt(self)
	end
	
end


function EnemyZ.executeHaltMove(self)

	self.haltTimer = self.haltTimer - 1
	
	if self.haltTimer <= 0 then
		self.switchToContinue(self)
	end
end

function EnemyZ.executeContinueMove(self)
	
	--scrolling movement pixel
	self.x = self.x - 1

	self.moveProgressX = self.moveProgressX - self.speedX
	
	while self.moveProgressX <= ((-1) * threshold) do --for negative x-Movement
		self.moveProgressX = self.moveProgressX + threshold
		self.x = self.x - 1
	end
	
	if self.x < self.boundsLeft then --here it applies
		self.deactivate(self)
	end 

end

function EnemyZ.switchToHalt(self)

	self.isStarting = false
	self.isHalting = true
	self.isContinuing = false
	
	self.haltTimer = 15 + 15*self.cycles
	--self.shotTimer = 15
end

function EnemyZ.switchToContinue(self)

	self.isStarting = false
	self.isHalting = false
	self.isContinuing = true
end

function EnemyZ.plantShot(self)
	
	local lclLaunchPX = 0
	local lclLaunchPY = 0
	
	local lclXSpd = 0.0
	local lclYSpd = 0.0
	
	
	lclLaunchPX = self.x + 9
	lclLaunchPY = self.y + 3
	
	lclXSpd = self.shotSpdX
	lclYSpd = self.shotSpdY

	globEnmBulletsG[globEnmBulletsGIndex].activate(globEnmBulletsG[globEnmBulletsGIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
		
	globEnmBulletsGIndex = globEnmBulletsGIndex + 1
	if globEnmBulletsGIndex >= globEnmBulletsGIndexMax then
		globEnmBulletsGIndex = 1
	end
	
	if globIsInDemoMode == false then 
		shotSnd:stop()
		shotSnd:play()
	end
end

function EnemyZ.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end







