require "luas.shortcuts"
require "luas.boundingBox"
require "luas.extendedLoader"

ENMBULLET_INACTIVE   = 0	--type could be completely different projectiles in 1 class (or do different classes)
ENMBULLET_ACTIVE     = 1	--for this simple example, just active/inactive


EnmBulletC = Object:extend()


function EnmBulletC.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:resetType()
	
	--simplified, just 1 box (is used for all basic enemy bullet types here if there are types)
	self.reservedBBox = BoundingBox(enmShotCBoxGph)

end


function EnmBulletC:resetType()

	self.type = ENMBULLET_INACTIVE
	--following will be determined by type that is given on activation
	self.width = 0
	self.height= 0
	self.currentBBox       = nil
	self.currentGph        = nil
	
	self.x = 0
	self.y = 0
	
	self.xSpeedStep     = 0.0
	self.xSpeedProgress = 0.0
	self.xSpeedThresh= threshold
	
	self.ySpeedStep     = 0.0
	self.ySpeedProgress = 0.0
	self.ySpeedThresh= threshold
	
	--boundaries (basic values, getting fine adjustment according to bulletsize below)
	self.boundsLeft  = 0
	self.boundsRight = 200
	self.boundsTop   = 0
	self.boundsBtm   = 150
	
	self.bombTimer = 0

end


--may add bullet-type to parameterlist or do different classes for different bullets. 
--
function EnmBulletC.activate(self, aPX, aPY, aSX, aSY) --need loopdirection

	
	self.type = ENMBULLET_ACTIVE --SIMPLIFIED only one bullettype, activate
	
	--following will be determined by type that is given on activation
	self.width = 7 --for other shooters there could be variation between bullettype sizes, here it isn't
	self.height= 5
	
	self.boundsLeft  = self.boundsLeft - self.width
	self.boundsTop   = self.boundsTop - self.height
	
	self.currentBBox = self.reservedBBox --SIMPLIFIED only 1 box to access
	self.currentGph  = enmShotCGph --to valid graphic
		
	--set bullet position
	self.x = aPX
	self.y = aPY
		
	--first boundingbox adjustment right on activation, otherwise it will sit on wrong place for 1 frame
	self.adjustBoundingBox(self)
		
	--set bullet speeds
	self.xSpeedStep     = aSX
	self.ySpeedStep     = aSY
	
	self.bombTimer = 30
	
end


function EnmBulletC:deactivate()
	--set inactive, remove current-fields of boundingbox, sizes, graphics ect
	
	--set defaults
	self:resetType()
end

function EnmBulletC.doUpdate(self)

	
	if self.type ~= ENMBULLET_INACTIVE then --when bullet not inactive
		
		
		--##############################
		--# bullet movements
		--##############################
		
		--bullet movement in x-directions
		self.xSpeedProgress = self.xSpeedProgress + self.xSpeedStep
		while self.xSpeedProgress >= self.xSpeedThresh do --for positive x-Movement
			self.xSpeedProgress = self.xSpeedProgress - self.xSpeedThresh
			self.x = self.x + 1
		end
		while self.xSpeedProgress <= ((-1) * self.xSpeedThresh) do --for negative x-Movement
			self.xSpeedProgress = self.xSpeedProgress + self.xSpeedThresh
			self.x = self.x - 1
		end
		--bullet movements in y directions 
		self.ySpeedProgress = self.ySpeedProgress + self.ySpeedStep 
		
		while self.ySpeedProgress >= self.ySpeedThresh do --for positive y-Movement (not used in this game)
			self.ySpeedProgress = self.ySpeedProgress - self.ySpeedThresh
			self.y = self.y + 1
		end
		while self.ySpeedProgress <= ((-1) * self.ySpeedThresh) do --for negative y-Movement (standard case)
			self.ySpeedProgress = self.ySpeedProgress + self.ySpeedThresh
			self.y = self.y - 1
		end
		
		--##############################
		--# after movement adjust bounding box
		--##############################
		
		self.adjustBoundingBox(self)
		
		
		--######################################
		--# deactivation when this bomb blows up
		--######################################
		
		if self.bombTimer > 1 then
			self.bombTimer = self.bombTimer - 1
		elseif self.bombTimer == 1 then
			self.bombTimer = 0
			self.blowUp(self)
		end
		
		
		--##############################
		--# bullet deactivation on boundaries
		--##############################
		
		if self.x < self.boundsLeft then
			self:deactivate()
		end
		if self.x > self.boundsRight then
			self:deactivate()
		end
		if self.y < self.boundsTop then
			self:deactivate()
		end
		if self.y > self.boundsBtm then
			self:deactivate()
		end
		
		
	end
	
	--for collision-handling: see gamestates!

end

function EnmBulletC.blowUp(self)

	local lclLaunchPX = self.x + 1
	local lclLaunchPY = self.y + 1
	
	local lclXSpd = 0.0
	local lclYSpd = 0.0
	local lclTotalSpeed = 3.5
	
	--left bullet
	lclXSpd = (-1)*1.0
	lclYSpd = 0.0
	local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
	lclXSpd = lclXSpd/lclVectorLength
	lclYSpd = lclYSpd/lclVectorLength	
	lclXSpd = lclXSpd*lclTotalSpeed
	lclYSpd = lclYSpd*lclTotalSpeed
	--
	globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
	--	
	globEnmBulletsAIndex = globEnmBulletsAIndex + 1
	if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
		globEnmBulletsAIndex = 1
	end

	--left upper bullet 
	lclXSpd = (-1)*1.0
	lclYSpd = (-1)*1.0
	local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
	lclXSpd = lclXSpd/lclVectorLength
	lclYSpd = lclYSpd/lclVectorLength	
	lclXSpd = lclXSpd*lclTotalSpeed
	lclYSpd = lclYSpd*lclTotalSpeed
	--
	globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
	--	
	globEnmBulletsAIndex = globEnmBulletsAIndex + 1
	if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
		globEnmBulletsAIndex = 1
	end
	
	--upper bullet
	lclXSpd = 0.0
	lclYSpd = (-1)*1.0
	local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
	lclXSpd = lclXSpd/lclVectorLength
	lclYSpd = lclYSpd/lclVectorLength	
	lclXSpd = lclXSpd*lclTotalSpeed
	lclYSpd = lclYSpd*lclTotalSpeed
	--
	globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
	--	
	globEnmBulletsAIndex = globEnmBulletsAIndex + 1
	if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
		globEnmBulletsAIndex = 1
	end
	
	--upper right bullet
	lclXSpd = 1.0
	lclYSpd = (-1)*1.0
	local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
	lclXSpd = lclXSpd/lclVectorLength
	lclYSpd = lclYSpd/lclVectorLength	
	lclXSpd = lclXSpd*lclTotalSpeed
	lclYSpd = lclYSpd*lclTotalSpeed
	--
	globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
	--	
	globEnmBulletsAIndex = globEnmBulletsAIndex + 1
	if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
		globEnmBulletsAIndex = 1
	end
	
	--right bullet
	lclXSpd = 1.0
	lclYSpd = 0.0
	local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
	lclXSpd = lclXSpd/lclVectorLength
	lclYSpd = lclYSpd/lclVectorLength	
	lclXSpd = lclXSpd*lclTotalSpeed
	lclYSpd = lclYSpd*lclTotalSpeed
	--
	globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
	--	
	globEnmBulletsAIndex = globEnmBulletsAIndex + 1
	if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
		globEnmBulletsAIndex = 1
	end
	
	--lower right bullet
	lclXSpd = 1.0
	lclYSpd = 1.0
	local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
	lclXSpd = lclXSpd/lclVectorLength
	lclYSpd = lclYSpd/lclVectorLength	
	lclXSpd = lclXSpd*lclTotalSpeed
	lclYSpd = lclYSpd*lclTotalSpeed
	--
	globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
	--	
	globEnmBulletsAIndex = globEnmBulletsAIndex + 1
	if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
		globEnmBulletsAIndex = 1
	end
	
	--lower bullet
	lclXSpd = 0.0
	lclYSpd = 1.0
	local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
	lclXSpd = lclXSpd/lclVectorLength
	lclYSpd = lclYSpd/lclVectorLength	
	lclXSpd = lclXSpd*lclTotalSpeed
	lclYSpd = lclYSpd*lclTotalSpeed
	--
	globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
	--	
	globEnmBulletsAIndex = globEnmBulletsAIndex + 1
	if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
		globEnmBulletsAIndex = 1
	end
	
	--lower left bullet
	lclXSpd = (-1)*1.0
	lclYSpd = 1.0
	local lclVectorLength = math.sqrt(lclXSpd * lclXSpd + lclYSpd * lclYSpd)
	lclXSpd = lclXSpd/lclVectorLength
	lclYSpd = lclYSpd/lclVectorLength	
	lclXSpd = lclXSpd*lclTotalSpeed
	lclYSpd = lclYSpd*lclTotalSpeed
	--
	globEnmBulletsA[globEnmBulletsAIndex].activate(globEnmBulletsA[globEnmBulletsAIndex], lclLaunchPX, lclLaunchPY, lclXSpd, lclYSpd)
	--	
	globEnmBulletsAIndex = globEnmBulletsAIndex + 1
	if globEnmBulletsAIndex >= globEnmBulletsAIndexMax then
		globEnmBulletsAIndex = 1
	end
	
	self:deactivate()
	
end

function EnmBulletC.adjustBoundingBox(self)

	--just in case include all boundingboxes (although bullet should not change during lifecycle)
	--for i= 1,3,1 do
		self.reservedBBox.doUpdate(self.reservedBBox, self.x, self.y)
	--end
end


function EnmBulletC.draw(self)
	if self.type ~= ENMBULLET_INACTIVE then
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
		
		--debug
		--self.currentBBox.draw(self.currentBBox)
	end
end


function EnmBulletC:CollisionCheck(aBox)

	if self.type ~= ENMBULLET_INACTIVE then
		if self.currentBBox == nil then
			return false
		else
			return self.currentBBox:CollisionTest(aBox)
		end
	else 
		return false
	end
end
