require "luas.shortcuts"
require "luas.boundingBox"
require "luas.extendedLoader"

ENMBULLET_INACTIVE   = 0	--type could be completely different projectiles in 1 class (or do different classes)
ENMBULLET_ACTIVE     = 1	--for this simple example, just active/inactive

--TODO have loopdir constants, use em in enemy too


EnmBulletF = Object:extend()


function EnmBulletF.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:resetType()
	self:resetAnimation()
	
	--simplified, just 1 box (is used for all basic enemy bullet types here if there are types)
	self.reservedBBox = BoundingBox(enmShotFBoxGph) --pixelbased collisiontest, no boundingbox
	
	--create container for two bullet frames. 
	self.Gphs={}
	for i= 1,24,1 do
		self.Gphs[i] = nil --don't know the color yet.
	end
	
	self.Gphs[1] = enmShotFGph[1] --only yellow frames
	self.Gphs[2] = enmShotFGph[1]
	self.Gphs[3] = enmShotFGph[1]
	self.Gphs[4] = enmShotFGph[1]
	self.Gphs[5] = enmShotFGph[1]
	self.Gphs[6] = enmShotFGph[1]
	self.Gphs[7] = enmShotFGph[2]
	self.Gphs[8] = enmShotFGph[2]
	self.Gphs[9] = enmShotFGph[2]
	self.Gphs[10] = enmShotFGph[2]
	self.Gphs[11] = enmShotFGph[2] --only yellow frames
	self.Gphs[12] = enmShotFGph[2]
	self.Gphs[13] = enmShotFGph[3]
	self.Gphs[14] = enmShotFGph[3]
	self.Gphs[15] = enmShotFGph[3]
	self.Gphs[16] = enmShotFGph[3]
	self.Gphs[17] = enmShotFGph[3]
	self.Gphs[18] = enmShotFGph[3]
	self.Gphs[19] = enmShotFGph[4]
	self.Gphs[20] = enmShotFGph[4]
	self.Gphs[21] = enmShotFGph[4]
	self.Gphs[22] = enmShotFGph[4]
	self.Gphs[23] = enmShotFGph[4]
	self.Gphs[24] = enmShotFGph[4]

end

function EnmBulletF:resetType()

	self.type = ENMBULLET_INACTIVE
	--following will be determined by type that is given on activation
	self.width = 0
	self.height= 0
	self.currentBBox       = nil --no boundingbox, per pixel collision handling
	self.currentGph        = nil
	
	self.x = 0
	self.y = 0
	
	self.xSpeedStep     = 0.0
	self.xSpeedProgress = 0.0
	self.xSpeedThresh= threshold
	
	self.ySpeedStep     = 0.0
	self.ySpeedProgress = 0.0
	self.ySpeedThresh= threshold
	
	--boundaries (basic values, getting fine adjustment according to bulletsize below)
	self.boundsLeft  = (-1)*7
	self.boundsRight = 200
	self.boundsTop   = (-1)*7
	self.boundsBtm   = 150

end


function EnmBulletF:resetAnimation()
	--index for animations
	self.anmStep     = 25--to invalid value
	self.anmThresh   = 24  
end


--may add bullet-type to parameterlist or do different classes for different bullets. 
--
function EnmBulletF.activate(self, aPX, aPY, aSX, aSY) --need loopdirection
	
	self.type = ENMBULLET_ACTIVE  --SIMPLIFIED only one bullettype, activate
	
	--following will be determined by type that is given on activation
	self.width = 7 --for other shooters there could be variation between bullettype sizes, here it isn't
	self.height= 7
		
	self.boundsLeft  = (-7)
	self.boundsRight = 200
	self.boundsTop   = (-7)
	self.boundsBtm   = 150
	
	self.anmStep  = 1
	
	self.currentBBox = self.reservedBBox --no boundingbox, per pixel collision
	self.currentGph  = self.Gphs[self.anmStep] --to valid graphic
		
	--set bullet position
	self.x = aPX
	self.y = aPY
		
	--no boundingbox
	--self.adjustBoundingBox(self)
		
	--set bullet speeds
	self.xSpeedStep     = aSX
	self.ySpeedStep     = aSY
	
	print("Activate big bullet")
	
end


function EnmBulletF:deactivate()
	--set inactive, remove current-fields of boundingbox, sizes, graphics ect
	
	--set defaults
	self:resetType()
	self:resetAnimation()
end

function EnmBulletF.doUpdate(self)

	--##############################
	--# calculate bullet animation
	--##############################
	
	if self.type ~= ENMBULLET_INACTIVE then --when bullet not inactive

		
		if self.anmStep <= self.anmThresh then
			self.anmStep = self.anmStep + 1
		end
	
		if self.anmStep > self.anmThresh then
			self.anmStep = 1 --reset
		end
		
		
		--##############################
		--# apply bullet animation
		--##############################
		
		self.currentGph = self.Gphs[self.anmStep] --SIMPLIFIED, no bullet types
		
		
		--##############################
		--# bullet movements
		--##############################
		
		--bullet movement in x-directions
		self.xSpeedProgress = self.xSpeedProgress + self.xSpeedStep
		while self.xSpeedProgress >= self.xSpeedThresh do --for positive x-Movement
			self.xSpeedProgress = self.xSpeedProgress - self.xSpeedThresh
			self.x = self.x + 1
		end
		while self.xSpeedProgress <= ((-1) * self.xSpeedThresh) do --for negative x-Movement
			self.xSpeedProgress = self.xSpeedProgress + self.xSpeedThresh
			self.x = self.x - 1
		end
		--bullet movements in y directions 
		self.ySpeedProgress = self.ySpeedProgress + self.ySpeedStep 
		
		while self.ySpeedProgress >= self.ySpeedThresh do --for positive y-Movement (not used in this game)
			self.ySpeedProgress = self.ySpeedProgress - self.ySpeedThresh
			self.y = self.y + 1
		end
		while self.ySpeedProgress <= ((-1) * self.ySpeedThresh) do --for negative y-Movement (standard case)
			self.ySpeedProgress = self.ySpeedProgress + self.ySpeedThresh
			self.y = self.y - 1
		end
		
		--##############################
		--# after movement adjust bounding box
		--##############################
		
		self.adjustBoundingBox(self) -- no boundingbox, per pixel collision
		
		
		--##############################
		--# bullet deactivation on boundaries
		--##############################
		
		if self.x < self.boundsLeft then
			self:deactivate()
		end
		if self.x > self.boundsRight then
			self:deactivate()
		end
		if self.y < self.boundsTop then
			self:deactivate()
		end
		if self.y > self.boundsBtm then
			self:deactivate()
		end
		
		
	end
	
	--for collision-handling: see gamestates!

end

--no boundingbox, per pixel collision
--
function EnmBulletF.adjustBoundingBox(self)

	----just in case include all boundingboxes (although bullet should not change during lifecycle)
	----for i= 1,3,1 do
		self.reservedBBox.doUpdate(self.reservedBBox, self.x, self.y)
	----end
end


function EnmBulletF.draw(self)
	if self.type ~= ENMBULLET_INACTIVE then
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
		
		--debug
		--self.currentBBox.draw(self.currentBBox)
	end
end







