require "luas.shortcuts"
require "luas.boundingBox"
require "luas.extendedLoader"

ENMBULLET_INACTIVE   = 0	--type could be completely different projectiles in 1 class (or do different classes)
ENMBULLET_ACTIVE     = 1	--for this simple example, just active/inactive


EnmBulletG = Object:extend()


function EnmBulletG.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:resetType()
	
	--simplified, just 1 box (is used for all basic enemy bullet types here if there are types)
	self.reservedBBox = BoundingBox(enmShotABoxGph)

end


function EnmBulletG:resetType()

	self.type = ENMBULLET_INACTIVE
	--following will be determined by type that is given on activation
	self.width = 0
	self.height= 0
	self.currentBBox       = nil
	self.currentGph        = nil
	
	self.x = 0
	self.y = 0
	
	self.xSpeedStep     = 0.0
	self.xSpeedProgress = 0.0
	self.xSpeedThresh= threshold
	
	self.ySpeedStep     = 0.0
	self.ySpeedProgress = 0.0
	self.ySpeedThresh= threshold
	
	--boundaries (basic values, getting fine adjustment according to bulletsize below)
	self.boundsBtm   = 150

end


--may add bullet-type to parameterlist or do different classes for different bullets. 
--
function EnmBulletG.activate(self, aPX, aPY, aSX, aSY) --need loopdirection

	
	self.type = ENMBULLET_ACTIVE --SIMPLIFIED only one bullettype, activate
	
	--following will be determined by type that is given on activation
	self.width = 4 --for other shooters there could be variation between bullettype sizes, here it isn't
	self.height= 4
	
	self.currentBBox = self.reservedBBox --SIMPLIFIED only 1 box to access
	self.currentGph  = enmShotAGph --to valid graphic
		
	--set bullet position
	self.x = aPX
	self.y = aPY
		
	--first boundingbox adjustment right on activation, otherwise it will sit on wrong place for 1 frame
	self.adjustBoundingBox(self)
		
	--set bullet speeds
	self.xSpeedStep     = aSX
	self.ySpeedStep     = aSY
	
end


function EnmBulletG:deactivate()
	--set inactive, remove current-fields of boundingbox, sizes, graphics ect
	
	--set defaults
	self:resetType()
end

function EnmBulletG.doUpdate(self)

	
	if self.type ~= ENMBULLET_INACTIVE then --when bullet not inactive
		
		self.ySpeedStep = self.ySpeedStep + 0.2 --accelerate to make this shot have gravity
		
		--##############################
		--# bullet movements
		--##############################
		
		--bullet movement in x-directions
		self.xSpeedProgress = self.xSpeedProgress + self.xSpeedStep
		while self.xSpeedProgress >= self.xSpeedThresh do --for positive x-Movement
			self.xSpeedProgress = self.xSpeedProgress - self.xSpeedThresh
			self.x = self.x + 1
		end
		while self.xSpeedProgress <= ((-1) * self.xSpeedThresh) do --for negative x-Movement
			self.xSpeedProgress = self.xSpeedProgress + self.xSpeedThresh
			self.x = self.x - 1
		end
		--bullet movements in y directions 
		self.ySpeedProgress = self.ySpeedProgress + self.ySpeedStep 
		
		while self.ySpeedProgress >= self.ySpeedThresh do --for positive y-Movement (not used in this game)
			self.ySpeedProgress = self.ySpeedProgress - self.ySpeedThresh
			self.y = self.y + 1
		end
		while self.ySpeedProgress <= ((-1) * self.ySpeedThresh) do --for negative y-Movement (standard case)
			self.ySpeedProgress = self.ySpeedProgress + self.ySpeedThresh
			self.y = self.y - 1
		end
		
		--##############################
		--# after movement adjust bounding box
		--##############################
		
		self.adjustBoundingBox(self)
		
		
		--##############################
		--# bullet deactivation on boundaries
		--##############################
		
		if self.y > self.boundsBtm then
			self:deactivate()
		end
		
		
	end
	
	--for collision-handling: see gamestates!

end

function EnmBulletG.adjustBoundingBox(self)

	--just in case include all boundingboxes (although bullet should not change during lifecycle)
	--for i= 1,3,1 do
		self.reservedBBox.doUpdate(self.reservedBBox, self.x, self.y)
	--end
end


function EnmBulletG.draw(self)
	if self.type ~= ENMBULLET_INACTIVE then
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
		
		--debug
		--self.currentBBox.draw(self.currentBBox)
	end
end


function EnmBulletG:CollisionCheck(aBox)

	if self.type ~= ENMBULLET_INACTIVE then
		if self.currentBBox == nil then
			return false
		else
			return self.currentBBox:CollisionTest(aBox)
		end
	else 
		return false
	end
end
