require "luas.shortcuts"
require "luas.boundingBox"
require "luas.extendedLoader"
require "luas.playerBullet"
--require "luas/globals"  --player can't require globals because globals requires player and that creates
                          --a circle-require (loop). but globals initializes variables globally with 
						  --static function calls so accessing the stuff from globals works anyway

PlayerShip = Object:extend()

function PlayerShip.new(self)

	self.active = false

	self.enteredGame = false
	
	self.inputType = GLOB_INPUT_WAITING

	self.playerNumColor = 0           --undefined (define in defineColorType)
	
	self.respawnedVulnerableFlag = false
	self.RespawnCountDownIndex   = 0  --standard: player starts invulnerable for short period of time, only for 2nd and 3rd player, see below
	self.RespawnCountDownMax     = 140
	self.RespawnCountDownResume  = 1
	
	self.shipWidth  = 16
	self.shipHeight = 11 
	
	self.currentBBox = BoundingBox(shipBoxGph)
	--self.currentBBox[1] = BoundingBox(shipBoxGph[1]) --no boundingbox for player in this game
	--self.currentBBox[2] = BoundingBox(shipBoxGph[2])
	self.currentMouseBox = BoundingBox(mouseBoxGph) --boundingbox-like for directional mouse control
	
	self.currentGph = nil --undefined, needs to be defined with defineColor
	self.isVisible  = true

	self.moveSpeed      = 2.5 
	self.dx			    = 0.0 
	self.dy             = 0.0
	
	self.x              = globPlayerStartPosX
	self.y              = globPlayerStartPosY
	
	--adjust boundingbox right at the beginning or it will
	--be at the wrong place in the first frame
	
	self.adjustFunctionBoxes(self) --does self.adjustBoundingBox(self) + self.adjustMouseBox(self) --same for mousebox
	
	--may introduce adjustFunctionBoxes
	
	self.boundsLeft   = globPlayerLeftBounds
	self.boundsRight  = globPlayerRightBounds
	self.boundsTop    = globPlayerTopBounds
	self.boundsBottom = globPlayerBottomBounds
	
	self.leadPlBulletStep     = 1
	self.leadPlBulletProgress = 12 --will be able to shoot
	self.leadPlBulletThresh   = 12
	self.leadPlBulletMax      = 4 
	self.shootCondition1=false
	self.shootCondition2=false
	
	self.leftPress  = false
	self.rightPress = false
	self.upPress    = false
	self.downPress  = false
	self.firePress  = false
	
	self.hasMouseXY = false
	self.mouseX = 0
	self.mouseY = 0
	
	print("Playership Constructor Call")
	
	
	--self.debugRightMouseWasDown= false
	
	--score in this case is not for the whole game but for the individual 
	--player. It is a score counted from jointime onward used to calculate extra lifes
	self.playerScorePrevious = 0
	self.playerScoreCurrent  = 0
	self.SCORE_EXTRALIFE_1   = 10000
	self.SCORE_EXTRALIFE_2   = 20000
	self.SCORE_EXTRALIFE_3   = 30000
	
	--frame for toggling of sprite visibility
	self.ToggleFrame = {}
	self.ToggleFrameIndex    = 1
	self.ToggleFrameIndexMax = 8 
	--for i=1,self.ToggleFrameIndexMax,1 do		--dont use loop,set manually
	self.ToggleFrame[1] = true
	self.ToggleFrame[2] = true
	self.ToggleFrame[3] = false
	self.ToggleFrame[4] = false
	self.ToggleFrame[5] = false
	self.ToggleFrame[6] = false
	self.ToggleFrame[7] = true
	self.ToggleFrame[8] = true
	--end
	
	--self.deathAnmProgress = 0 -- default: handle death animation by yPosition of player, offset determined here
	
	self.dyingCountDown = 0
	self.dyingCountDownMax = 180--240
	self.doResumeChecks = false
	
	--self.demoMoveDirX = 0 --don't use
	--self.demoMoveDirY = 0
	
	--self.faceDirLeft = true --not needed
	
	--animation 
	self.anmIndex    = 0 --invalid values
	self.anmIndexMax = 2
	self.anmStep     = 1
	self.anmThresh   = 15
	
	
	self.power = 0 --power of the shots,invalid value
	self.powerMax = 4
	
	
	self.satelites = 0
	self.satelitesMax = 2
	self.secondaryShotTimer = 0
	self.secondaryShotTimerMax = 20
	
end

function PlayerShip.doPowerUp(self)
	if self.power < self.powerMax then
		self.power = self.power + 1
	end
end

function PlayerShip.addSatelite(self)
	if self.satelites < self.satelitesMax then
		self.satelites = self.satelites + 1
	end
end

function PlayerShip.getCurrentInputType(self)

	return self.inputType
end

function PlayerShip.addScore(self, aScorePlus)
	
	if self.enteredGame == true and self.active == true then
	
		if self.inputType ~= GLOB_INPUT_COMPUTER then--prevent adding score for computer-controlled players aka demoMode
	
			self.playerScoreCurrent = self.playerScoreCurrent + aScorePlus
			
			if self.playerScoreCurrent > globMaxScore then
				self.playerScoreCurrent = globMaxScore
			end
	
			--hardwire for gaining a maximum of 2 additional lifes with score
			if self.playerScorePrevious < self.SCORE_EXTRALIFE_1 and self.playerScoreCurrent >= self.SCORE_EXTRALIFE_1 then
				globNumLifes[self.playerNumColor] = globNumLifes[self.playerNumColor] + 1
				
				if globIsInDemoMode == false then --play sound not in DemoMode
					extraLifeSnd:stop()
					extraLifeSnd:play()
				end
			end
			if self.playerScorePrevious < self.SCORE_EXTRALIFE_2 and self.playerScoreCurrent >= self.SCORE_EXTRALIFE_2 then
				globNumLifes[self.playerNumColor] = globNumLifes[self.playerNumColor] + 1
				
				if globIsInDemoMode == false then --play sound not in DemoMode
					extraLifeSnd:stop()
					extraLifeSnd:play()
				end
			end
			if self.playerScorePrevious < self.SCORE_EXTRALIFE_3 and self.playerScoreCurrent >= self.SCORE_EXTRALIFE_3 then
				globNumLifes[self.playerNumColor] = globNumLifes[self.playerNumColor] + 1
				
				if globIsInDemoMode == false then --play sound not in DemoMode
					extraLifeSnd:stop()
					extraLifeSnd:play()
				end
			end
			
		
			self.playerScorePrevious = self.playerScoreCurrent
		end
	end
	

end

function PlayerShip.defineColorType(self, aColorNum) --multiplayer needs different shipcolors
	
	--define color
	self.playerNumColor = aColorNum
	--apply color
	--moved to activate
	--if self.faceDirLeft == true then 
	--	self.currentGph = playerShip_L_Gph[self.playerNumColor][1]--frame 1
	--else --hypothetically, case should never occur since its facing left on constructor
	--	self.currentGph = playerShip_R_Gph[self.playerNumColor][1]--frame 1
	--end
	
	print("Player is defined, PlayerNumber "..self.playerNumColor)
 
end

function PlayerShip.enterGame(self, aInputType)
	
	print("Player enters game:")
	print("Player number is "..self.playerNumColor)
	
	
	if self.playerNumColor == 1 then --color of first player
		self.respawnedVulnerableFlag = true --first player starts the game instantly vulnerable(as opposed to 
		                                    --those who join later and may otherwise be unfairly placed on a bullet)
		self.RespawnCountDownIndex   = 0
	end
	
	if self.playerNumColor <= 0 then
		--must not happen, need playercolor aka playernumber 
		--before player can enter and inputdevice be set
		dummy=1/0
	end
	
	if aInputType ~= GLOB_INPUT_WAITING then --playernumber aka color must be defined and input-type not passive...
		self.enteredGame = true
		self.inputType = aInputType
		--on entering game, always activate. 
		self.activate(self)
	
	elseif self.playerNumColor == 1 and aInputType == GLOB_INPUT_WAITING then --must not happen, 
		--because player 1 needs a valid input-device at this point. other players may join later.
		dummy = 1/0 
	end
	
	local debugVul = 0
	if self.respawnedVulnerableFlag ==true then
		debugVul = 1
	end
	local debugEnt = 0
	if self.enteredGame ==true then
		debugEnt = 1
	end
	print("Initial Vulnerability is "..debugVul.." Vulnerability Countdown starts with "..self.RespawnCountDownIndex)
	print("Player self.enteredGame is "..debugEnt..", input Type is "..self.inputType)
	
end

function PlayerShip.activate(self)
	if self.enteredGame then
		self.active = true
		
		self.anmIndex = 1 
		self.anmStep  = 1 
		
		self.currentGph = playerShipGph[self.playerNumColor][self.anmIndex]
		
		self.power = 1 --power of the shots,starting off with 1
		
		self.satelites = 0 --debug
		
	else
		--player not supposed to get activated if not entered game
		dummy = 1/0
	end
end

function PlayerShip.deactivate(self)
	self.active = false
	--TODO consider reset function, not essential in this game because player can't recover from gameover
	--and will be completely re-instantiated with gamestate change so unlike other objects, no re-activation.
	--
	--also ALWAYS keep self.enteredGame true on deactivation even with reset which means player had its chance
end

--function PlayerShip.setDeathAnmProgress(self, aAnmProgress) --not used like that
	--could as well be real death animation such as explosion, 
	--here it is a sink-into-ground Progress
	
	--no, don't mix up update action with the action that sets the parameter
	--self.y = globPlayerStartPosY - aAnmProgress
	
--	self.deathAnmProgress = aAnmProgress
	
--end

--function PlayerShip.doSmallUpdate(self) --not for death animation which is handled by gamestate

--	self.y = globPlayerStartPosY - self.deathAnmProgress
	
	
	--if self.deathAnmProgress == 1 then
	--	print(" S   M   A   L   L            U   P   D   A   T   E    1")
	--elseif self.deathAnmProgress == 2 then
	--	print(" S   M   A   L   L            U   P   D   A   T   E    2")
	--end
	
--end

function PlayerShip.doUpdate(self)


	local lclBlockFireInput     = false --ignore fire button in this update (?)
	local lclSpecialUpdateOnly = false --ignore most actions in this update (?)

	if self.dyingCountDown > 0 then
		lclSpecialUpdateOnly = true
		
		--print("Player "..self.playerNumColor.." dying - block most update")
	end

	--TODO entryMute has to be transferred to player (2nd and 3rd player should 
	--also start out not instantly shooting but shooting of previously active player(s) must not be blocked)
	if globEntryMuteCountDownIndex > 0 and globGameOverMuteCountDownIndex > 0 then
		lclBlockFireInput = true
		
		--print("Player "..self.playerNumColor.." entry mute - block fire")
	end
	
	if self.enteredGame == false then --if not yet entered game, ...

		if self.playerNumColor > 1 then --does not apply to player 1 but to other players joining on the run
		
			--TODO...listen for firebutton to enable entering the game AND determining input device for this ship
			if lclSpecialUpdateOnly == false then --...but only if game isnt blocked by other player's death
				--TODO set self.enteredGame
				--TODO set input device
			end
		end
		
	
	else --enteredGame true, update other than waiting for game-enter...
		
		
		
		--#############################################
		--#  For Playership after dying do resumechecks
		--#############################################
				
		if self.doResumeChecks == true then
			
			self.doResumeChecks = false --only do the check once
				
			--self.playerGameOver
			--self.playerRespawns
			if globNumLifes[self.playerNumColor] > 0 then
			
				if globIsInDemoMode == false then--don't lose lifes when game is in demomode
					globNumLifes[self.playerNumColor] = globNumLifes[self.playerNumColor] - 1 --one life less 
					
					--decrease power (only in hard mode)
					if globDifficulty == GLOB_DIFFICULTY_HARD then
						if self.power > 1 then
							self.power = self.power - 1
						end
					end
					
				end
				
				self.respawnedVulnerableFlag = false
				self.isVisible = true --make re-visible
				self.active = true
				
			else
				globNumLifes[self.playerNumColor] = (-1) --player gameover
				
				print("Player game over")
			end
				
		end
		
		
		
		--with entered game flag set
		
		if lclSpecialUpdateOnly == false then
		
			if self.active == true then --as long as active
			
		
				if globNumLifes[self.playerNumColor] < 0 then --check if lifes used up, if so deactivate
			
					self.deactivate(self) --in next loop will not be active and will neither do normal nor special update part
			
				else
			
				--###################################################################################
				--# known in this inner part : player is in game and active
				--###################################################################################
				--# let player do the real part of this update or the special part of death animation
				--###################################################################################
			
			
				--if lclSpecialUpdateOnly == false then --doesnt work here, move it up
				
				
					--####################################
					--#  Respawn Vulnerability Handling  #
					--####################################
		
					if self.respawnedVulnerableFlag == false and self.RespawnCountDownIndex == 0 then
					
						--respawn flag at invulnerable and respawn countdown inactive, set countdown
						self.RespawnCountDownIndex = self.RespawnCountDownMax
						--self.respawnedVulnerableFlag = false
						--self.RespawnCountDownIndex   = 140  --standard: player starts invulnerable for short period of time
						--self.RespawnCountDownMax     = 140
						--self.RespawnCountDownResume  = 1
						
						
					elseif self.RespawnCountDownIndex > self.RespawnCountDownResume then
					
						print("self.RespawnCountDownIndex "..self.RespawnCountDownIndex)
					
						--run countdown ...
						self.RespawnCountDownIndex = self.RespawnCountDownIndex - 1
						
						--and let it toggle visibility
						self.ToggleFrameIndex = self.ToggleFrameIndex + 1
						if self.ToggleFrameIndex == self.ToggleFrameIndexMax then
							self.ToggleFrameIndex = 1
						end
						self.isVisible = self.ToggleFrame[self.ToggleFrameIndex]
						
					elseif self.RespawnCountDownIndex == self.RespawnCountDownResume then
					
						--end countdown and reset respawn flag 
						self.RespawnCountDownIndex = 0
						self.respawnedVulnerableFlag = true
						
						--restore visibility
						self.isVisible = true
					else
						--keep or restore visibility
						self.isVisible = true
					end
				
				
					self.leftPress  = false
					self.rightPress = false
					self.upPress    = false
					self.downPress  = false
					self.firePress  = false
	
					--if self.inputType == GLOB_INPUT_KEYBOARD then --all controls at once
	
	
						if lkeyBrd("left") == true then
							self.leftPress = true
						end
						if lkeyBrd("right") == true then
							self.rightPress = true
						end
						if lkeyBrd("up") == true then
							self.upPress = true
						end
						if lkeyBrd("down") == true then
							self.downPress = true
						end
						
						if lclBlockFireInput == false then
							if lkeyBrd("z") == true or lkeyBrd("y") == true then
								self.firePress  = true
							end
						end

	
					--elseif self.inputType == GLOB_INPUT_GAMEPAD then --all controls at once
			
						if globJoystick then --only react when joystick 1 is actually in usb 
					
							if globJoystick:isGamepadDown("dpleft") == true then
								self.leftPress = true
							end
							if globJoystick:isGamepadDown("dpright") == true then
								self.rightPress = true
							end
							if globJoystick:isGamepadDown("dpup") == true then
								self.upPress = true
							end
							if globJoystick:isGamepadDown("dpdown") == true then
								self.downPress = true
							end
					
							if lclBlockFireInput == false then
						
								--down10 = globJoystick:isDown(10)
								--down11 = globJoystick:isDown(11)
								--down12 = globJoystick:isDown(12)
								--down13 = globJoystick:isDown(13)
								--if globJoystick:isDown(10) or globJoystick:isDown(11) or globJoystick:isDown(12) or globJoystick:isDown(13) then
								--	self.firePress  = true
								--end
								if globJoystick:isGamepadDown("a", "b", "x", "y") == true then
									self.firePress  = true
								end
						
							end
						end
						
					--elseif self.inputType == GLOB_INPUT_GAMEPAD2 then --all controls at once
					
						if globJoystick2 then --only react when joystick 1 is actually in usb 
					
							if globJoystick2:isGamepadDown("dpleft") == true then
								self.leftPress = true
							end
							if globJoystick2:isGamepadDown("dpright") == true then
								self.rightPress = true
							end
							if globJoystick2:isGamepadDown("dpup") == true then
								self.upPress = true
							end
							if globJoystick2:isGamepadDown("dpdown") == true then
								self.downPress = true
							end
					
							if lclBlockFireInput == false then
								
								if globJoystick2:isGamepadDown("a", "b", "x", "y") == true then
									self.firePress = true
								end
						
							end
						end
					
					--elseif self.inputType == GLOB_INPUT_GAMEPAD3 then --all inputs at once
					
						if globJoystick3 then --only react when joystick 1 is actually in usb 
					
							if globJoystick3:isGamepadDown("dpleft") == true then
								self.leftPress = true
							end
							if globJoystick3:isGamepadDown("dpright") == true then
								self.rightPress = true
							end
							if globJoystick3:isGamepadDown("dpup") == true then
								self.upPress = true
							end
							if globJoystick3:isGamepadDown("dpdown") == true then
								self.downPress = true
							end
					
							if lclBlockFireInput == false then
								
								if globJoystick3:isGamepadDown("a", "b", "x", "y") == true then
									self.firePress = true
								end
						
							end
						end
	
					--elseif self.inputType == GLOB_INPUT_MOUSE then --all controls at once
			
						--move ship with right mouse button down relative to playership (and its mouse-box)
						local lclMoveCheck = lMouseDown("r") --fire with left mousebutton down 
				
						--if leftRightCheck == true then
							--print("Right mouse btn down is true")
						--else
							--print("Right mouse down is false")
						--end
						
						self.hasMouseXY = false
				
				
						if lclMoveCheck == true then
						
							self.hasMouseXY = true
				
							--for mousepointer left of mousebox trigger movement to the left
							--for mousepointer right of mousebox trigger movement to the right
					
							--problem here: mouseposition is absolute window position.
							--whereas player position is canvas position. need to get transformed mouse position
							--in canvas coordinates. calculating those should not be the business of the playerobject
							--likely put in globals.
					
							local lclMouseX = lMouseGetX()
							local lclMouseY = lMouseGetY()
							--transfrom mouse according to game resolution (window to big-pixel)
							lclMouseX = mouseX_Transformed(lclMouseX)
							lclMouseY = mouseY_Transformed(lclMouseY)
							
							--need mousepos below, so put them in class variables
							self.mouseX = lclMouseX
							self.mouseY = lclMouseY
					
							--print("MouseX is"..lclMouseX)
							--print("MouseBoxMinX is "..self.currentMouseBox.minXT)
							--print("MouseBoxMaxX is "..self.currentMouseBox.maxXT)
							
							if self.currentMouseBox:InBoxTest(self.mouseX, self.mouseY) == true then --mouse input only counts if outside mousebox!
								self.hasMouseXY = false
							end
					
							
						end
				
				
						if lclBlockFireInput == false then
	
							if lMouseDown("l") == true then
								self.firePress  = true --fire with left mousebutton down 
							end
							
						end
						
					--end --all controls at once end
					
					
					if self.hasMouseXY == true then
					
						local lclDirVectorX = 0
						local lclDirVectorY = 0
						local lclShipMiddleX = self.x + 8
						local lclShipMiddleY = self.y + 6
						
						lclDirVectorX = self.mouseX - lclShipMiddleX
						lclDirVectorY = self.mouseY - lclShipMiddleY
						
						local lclVectorLength = math.sqrt(lclDirVectorX * lclDirVectorX + lclDirVectorY * lclDirVectorY)
						
						lclDirVectorX = lclDirVectorX/lclVectorLength
						lclDirVectorY = lclDirVectorY/lclVectorLength
						
						lclDirVectorX = lclDirVectorX*self.moveSpeed
						lclDirVectorY = lclDirVectorY*self.moveSpeed
						
						--apply movement
						self.dx = self.dx + lclDirVectorX
						self.dy = self.dy + lclDirVectorY
						
						
					
					elseif self.leftPress == true or self.rightPress == true or self.upPress == true or self.downPress == true then
					
						local lclDirVectorX = 0
						local lclDirVectorY = 0
						
						
						local lclApplyLeftPress = false
						local lclApplyRightPress = false
						local lclApplyUpPress = false
						local lclApplyDownPress = false
					
						--add to left right movement
						if self.rightPress == true and self.leftPress == true then --TODO also in other projects:
					
							lclApplyLeftPress = false
							lclApplyRightPress = false
						
						elseif self.rightPress == true then 
					
							lclApplyRightPress = true
						
							--self.faceDirLeft = false --not needed
					
							--self.dx	= self.dx + self.moveSpeed
						elseif self.leftPress == true then 
					
							lclApplyLeftPress = true
					
							--self.faceDirLeft = true --not needed
					
							--self.dx	= self.dx - self.moveSpeed
						end
					
						if self.upPress == true and self.downPress == true then
					
							lclApplyUpPress = false
							lclApplyDownPress = false
					
						elseif self.upPress == true then
					
							lclApplyUpPress = true
					
						elseif self.downPress == true then
					
							lclApplyDownPress = true
						
						end
						
						if lclApplyLeftPress == true or lclApplyRightPress == true or lclApplyUpPress == true or lclApplyDownPress == true then --need this or it may freeze when leftright or updown are pressed at once
						
							if lclApplyLeftPress == true and lclApplyUpPress == false and lclApplyDownPress == false then
								lclDirVectorX = (-1)*1.0
							elseif lclApplyRightPress == true and lclApplyUpPress == false and lclApplyDownPress == false then
								lclDirVectorX = 1.0
							elseif lclApplyUpPress == true and lclApplyLeftPress == false and lclApplyRightPress == false then
								lclDirVectorY = (-1)*1.0
							elseif lclApplyDownPress == true and lclApplyLeftPress == false and lclApplyRightPress == false then
								lclDirVectorY = 1.0
							elseif lclApplyLeftPress == true and lclApplyUpPress == true then
								lclDirVectorX = (-1)*1.0
								lclDirVectorY = (-1)*1.0
							elseif lclApplyLeftPress == true and lclApplyDownPress == true then
								lclDirVectorX = (-1)*1.0
								lclDirVectorY = 1.0
							elseif lclApplyRightPress == true and lclApplyUpPress == true then
								lclDirVectorX = 1.0
								lclDirVectorY = (-1)*1.0
							elseif lclApplyRightPress == true and lclApplyDownPress == true then
								lclDirVectorX = 1.0
								lclDirVectorY = 1.0
							end
						
						
							local lclVectorLength = math.sqrt(lclDirVectorX * lclDirVectorX + lclDirVectorY * lclDirVectorY)
						
							lclDirVectorX = lclDirVectorX/lclVectorLength
							lclDirVectorY = lclDirVectorY/lclVectorLength
						
							lclDirVectorX = lclDirVectorX*self.moveSpeed
							lclDirVectorY = lclDirVectorY*self.moveSpeed
						
							--apply movement
							self.dx = self.dx + lclDirVectorX
							self.dy = self.dy + lclDirVectorY
						
						end
					end
					
		
					--apply movement results according to deltas
					while self.dx > threshold do
						self.x = self.x + 1
						self.dx = self.dx - threshold
					end
					while self.dx < (-1)*threshold do
						self.x = self.x - 1
						self.dx = self.dx + threshold
					end
					while self.dy > threshold do
						self.y = self.y + 1
						self.dy = self.dy - threshold
					end
					while self.dy < (-1)*threshold do
						self.y = self.y - 1
						self.dy = self.dy + threshold
					end
		
					--apply bounds
					if self.x > self.boundsRight then
						self.x = self.boundsRight
					end
					if self.x < self.boundsLeft then
						self.x = self.boundsLeft
					end
					if self.y > self.boundsBottom then
						self.y = self.boundsBottom
					end
					if self.y < self.boundsTop then
						self.y = self.boundsTop
					end
					
		
					--# adjust positions of the boundingbox and mousebox
					--##################################################
			
			
					self.adjustFunctionBoxes(self) --does: self.adjustBoundingBox(self) + self.adjustMouseBox(self)
					
					
					
		
		
					--# prepare trigger for shooting
					--####################################
	
					local shootAttempt = false
			
					if self.firePress == true then
		
						--while shooting button pressed count up until trigger reloaded
						self.leadPlBulletProgress = self.leadPlBulletProgress + self.leadPlBulletStep
		
						if self.leadPlBulletProgress > self.leadPlBulletThresh then
							self.leadPlBulletProgress = self.leadPlBulletThresh
						end
						shootAttempt = true
				
						--print("shoot attempt")
					else
						--otherwise when shooting not pressed, shortcut to fully reloaded trigger
						self.leadPlBulletProgress = self.leadPlBulletThresh
					end
		
					--# with shooting trigger set, check shooting condition and shoot when allowed
					--############################################################################
	
					if shootAttempt == true then
						--fire pressed and trigger at maximum is first condition neccessary for shooting
						if self.leadPlBulletProgress == self.leadPlBulletThresh then
							self.shootCondition1=true
					
							--print("condition 1")
						end
		
			 
						--number of leadbullets less than maximum is the second condition neccessary for shooting
						--Step 1: iterate over global bulletcontainer maximum, check how many bullets are in it
						local bCount=0
						for i= 1,globLeadPlBulletIndexMax,1 do
					
							if globLeadPlBullets[self.playerNumColor][i].type ~= PLBULLET_INACTIVE then --TODO check if correct bullet container is accessed
								if globLeadPlBullets[self.playerNumColor][i].isLeadBullet == true then
									bCount = bCount + 1
								end
							end
						end
						--Step 2: check if bullet-maximum of player is reached, if there is room for new bullet, set condition 2
						--print("bCount"..bCount)
						if bCount < self.leadPlBulletMax then
							self.shootCondition2=true
					
							--print("condition 2")
						end
		
						if self.shootCondition1 == true and self.shootCondition2 == true then
		
							--print("shoot")
		
							self.fireShot(self)
		
							self.leadPlBulletProgress = 0--reset the trigger
						end
		
						--do not carry shooting conditions over to the next update
						self.shootCondition1=false
						self.shootCondition2=false
						
						
						--secondary shot
						if self.satelites > 0 then
							if self.secondaryShotTimer == 0 then
								self.secondaryShotTimer = self.secondaryShotTimerMax
								self.fireSecondaryShot(self)
							end
						end
					end	
					
					--even when fire not pressed, reduce self.secondaryShotTimer so it has to be placed here outside the shoot-attempt
					if self.secondaryShotTimer > 0 then
						self.secondaryShotTimer = self.secondaryShotTimer - 1
					end
					
					
					--animation not needed
					
					self.anmStep = self.anmStep + 1
	
					if self.anmStep >= self.anmThresh then
	
						self.anmStep = 1
	
						self.anmIndex = self.anmIndex + 1
		
						if self.anmIndex > self.anmIndexMax then
							self.anmIndex = 1
						end
						

						self.currentGph = playerShipGph[self.playerNumColor][self.anmIndex]--frame 1

					end
					
					
			
			
					--Respawn handling old version (TODO may want to place new version here)
			
					--if self.RespawnCountDownIndex > self.RespawnCountDownResume then
			
					--	self.RespawnCountDownIndex = self.RespawnCountDownIndex -1
				
						--TODO toggle player visibility with RespawnCountDownIndex
				
					--elseif self.RespawnCountDownIndex == self.RespawnCountDownResume then --standard: player starts invulnerable for short period of time
					--	self.respawnedVulnerableFlag = true -- (re-)enable vulnerability after invulnerability countdown
					--	self.RespawnCountDownIndex   = 0
					--end
			
			
		
				--elseif lclSpecialUpdateOnly == true then

				--end
			
			
			
			
			
				end  --act on num of lifes > 0 end
		
			else --active false
			
				self.isVisible = false 
			
			end --act on active end
		
		
		
		elseif lclSpecialUpdateOnly == true then
				
			if self.dyingCountDown > 0 then
				self.dyingCountDown = self.dyingCountDown - 1
						
				if self.dyingCountDown == (self.dyingCountDownMax - 1) then --the frame after player dies
				
				
					--take away the satellites and throw them on the playfield
					
					local lclPosSChunk1X = self.x - 7 - 4
					local lclPosSChunk1Y = self.y - 4 - 3
					local lclPosSChunk2X = self.x - 7 - 4
					local lclPosSChunk2Y = self.y + self.shipHeight + 1 - 3
					
					if self.satelites > 0 then
						globSateliteChunk[globSateliteChunkIndex].activate(globSateliteChunk[globSateliteChunkIndex], lclPosSChunk1X, lclPosSChunk1Y, 4.0, (-1)*0.5)
						
						globSateliteChunkIndex = globSateliteChunkIndex + 1
						if globSateliteChunkIndex > globSateliteChunkIndexMax then
							globSateliteChunkIndex = 1
						end
					end
					
					if self.satelites > 1 then
					
						globSateliteChunk[globSateliteChunkIndex].activate(globSateliteChunk[globSateliteChunkIndex], lclPosSChunk2X, lclPosSChunk2Y, 4.0, 0.5)
						
						globSateliteChunkIndex = globSateliteChunkIndex + 1
						if globSateliteChunkIndex > globSateliteChunkIndexMax then
							globSateliteChunkIndex = 1
						end
					
					end
					
					self.satelites = 0 --take away the satelites
					
					
					
					self.plantPlayerExplosion(self)
					
					
						
					self.isVisible = false --neccessary but not enough to make invisible or else further playercollisions will be detected as the game runs
					self.active=false --need to put active to false
					
					if globIsInDemoMode == false then --play sound not in DemoMode
						playerExplosionSnd:stop()
						playerExplosionSnd:play() 
					end
							
				elseif self.dyingCountDown == 1 then
						
					--finalize dying, do resumechecks, gameoverchecks
					--cannot do the resumechecks here and make player visible again because of the problem that on playergameover
					--the player will be visible for a short period of time before deactiivation. Instead set a resumeCheck flag
					--and have the ship do the resume check outside of the lclSpecialUpdateOnly part
					self.doResumeChecks = true
						
					self.dyingCountDown = 0
				end
			end

		end
		
		
	
	end --act on enteredGame end
	
	
	--There are updates that won't actualize inputs. To avoid carry-over of values to next update, empty them out.
	self.leftPress  = false
	self.rightPress = false
	self.upPress    = false
	self.downPress  = false
	self.firePress  = false
	
	
end

function PlayerShip.adjustFunctionBoxes(self)
	
	--update any bounding-objects that have a specific function in this player-object
	self.adjustBoundingBox(self) --has no boundingbox in this game
	self.adjustMouseBox(self)
end

function PlayerShip.adjustBoundingBox(self) --has no boundingbox in this game


	self.currentBBox.doUpdate(self.currentBBox, self.x, self.y)
--	--1 frame, 1 box, simplified version
--	self.currentBBox[1].doUpdate(self.currentBBox[1], self.x, self.y)
--	self.currentBBox[2].doUpdate(self.currentBBox[2], self.x, self.y)
end

function PlayerShip.adjustMouseBox(self)

	self.currentMouseBox.doUpdate(self.currentMouseBox, self.x, self.y)
end

function PlayerShip.fireShot(self)
	print("Fire shot was executed!")
	
	--access bullets from globals
	--TODO careful, bullets have no firepower and no animation offset in this game parameter in this game
	
	--SIMPLIFIED
	local playerIdx = self.playerNumColor
	local bulletIdx = globLeadPlBulletIndex[self.playerNumColor] --fetch bulletindex for this players's bulletcontainer
	--with playercolor index
	
	globLeadPlBullets[playerIdx][bulletIdx].setAsLeadBullet(globLeadPlBullets[playerIdx][bulletIdx], true)
	globLeadPlBullets[playerIdx][bulletIdx].setPower(globLeadPlBullets[playerIdx][bulletIdx],self.power)
	
	globLeadPlBullets[playerIdx][bulletIdx].activate(globLeadPlBullets[playerIdx][bulletIdx], self.x + 10, self.y + 3, 3.5, 0.0) --slightly different x position than left
	
	globLeadPlBulletIndex[playerIdx] = globLeadPlBulletIndex[playerIdx] + 1
	if globLeadPlBulletIndex[playerIdx] > globLeadPlBulletIndexMax then
		globLeadPlBulletIndex[playerIdx] = 1
	end
	
	if globIsInDemoMode == false then 
		shipShotSnd:stop()
		shipShotSnd:play()
	end

	
end

function PlayerShip.fireSecondaryShot(self)
	
	--if self.satelites > 0 then --previously already checked for satelites > 0 when this function was called
		local playerIdx = self.playerNumColor
		local bulletIdx = globLeadPlBulletIndex[self.playerNumColor]
		
		globLeadPlBullets[playerIdx][bulletIdx].setAsLeadBullet(globLeadPlBullets[playerIdx][bulletIdx], false)
		globLeadPlBullets[playerIdx][bulletIdx].setPower(globLeadPlBullets[playerIdx][bulletIdx], 1)--satelites always shoot with power 1
		
		globLeadPlBullets[playerIdx][bulletIdx].activate(globLeadPlBullets[playerIdx][bulletIdx], self.x - 5, self.y - 5, 3.5, 0.0) --slightly different x position than left
	
		globLeadPlBulletIndex[playerIdx] = globLeadPlBulletIndex[playerIdx] + 1
		if globLeadPlBulletIndex[playerIdx] > globLeadPlBulletIndexMax then
			globLeadPlBulletIndex[playerIdx] = 1
		end
	--end
	
	if self.satelites > 1 then
		playerIdx = self.playerNumColor
		bulletIdx = globLeadPlBulletIndex[self.playerNumColor]
		
		globLeadPlBullets[playerIdx][bulletIdx].setAsLeadBullet(globLeadPlBullets[playerIdx][bulletIdx], false)
		globLeadPlBullets[playerIdx][bulletIdx].setPower(globLeadPlBullets[playerIdx][bulletIdx], 1)--satelites always shoot with power 1
		
		globLeadPlBullets[playerIdx][bulletIdx].activate(globLeadPlBullets[playerIdx][bulletIdx], self.x - 5, self.y + self.shipHeight, 3.5, 0.0) --slightly different x position than left
	
		globLeadPlBulletIndex[playerIdx] = globLeadPlBulletIndex[playerIdx] + 1
		if globLeadPlBulletIndex[playerIdx] > globLeadPlBulletIndexMax then
			globLeadPlBulletIndex[playerIdx] = 1
		end
	end
	
end

function PlayerShip.plantPlayerExplosion(self)
	
	globPlayerExplosion.activate(globPlayerExplosion, self.x, self.y)

end


function PlayerShip.putToDeath(self)

	self.dyingCountDown = self.dyingCountDownMax
end


function PlayerShip.draw(self)

	if self.enteredGame and self.isVisible then
	
		if self.currentGph ~= nil then
			lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
			
			if self.satelites > 0 then
				lg.draw(sateliteGph, self.x - 7, self.y - 4, 0, 1, 1)
			end
			
			if self.satelites > 1 then
				lg.draw(sateliteGph, self.x - 7, self.y + self.shipHeight + 1, 0, 1, 1)
			end
		else
			dummy=1/0 --ought not to be nil once drawing is attempted. if so, crash deliberately on this line
		end
		
		--debug
		--self.currentMouseBox.draw(self.currentMouseBox)
	end
end




