require "luas.shortcuts"
require "luas.boundingBox"
require "luas.extendedLoader"

S_CHUNK_INACTIVE   = 0	--type could be completely different projectiles in 1 class (or do different classes)
S_CHUNK_ACTIVE     = 1	--for this simple example, just active/inactive


SateliteChunk = Object:extend()


function SateliteChunk.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:resetType()

end


function SateliteChunk:resetType()

	self.type = S_CHUNK_INACTIVE
	--following will be determined by type that is given on activation
	self.width = 0
	self.height= 0
	
	self.x = 0
	self.y = 0
	
	self.xSpeedStep     = 0.0
	self.xSpeedProgress = 0.0
	self.xSpeedThresh= threshold
	
	self.ySpeedStep     = 0.0
	self.ySpeedProgress = 0.0
	self.ySpeedThresh= threshold
	
	--boundaries (basic values, getting fine adjustment according to bulletsize below)
	self.boundsLeft  = (-13)
	self.boundsRight = 187
	self.boundsTop   = (-9)
	self.boundsBtm   = 150
	
	self.anmStep       = 3 --use for index of frame (here set invalid)
	self.anmThresh     = 2
	
	self.frames = {}
	
	self.timer = 0
	self.timerMax = 30

end


--may add bullet-type to parameterlist or do different classes for different bullets. 
--
function SateliteChunk.activate(self, aPX, aPY, aSX, aSY) --need loopdirection

	
	self.type = S_CHUNK_ACTIVE --SIMPLIFIED only one bullettype, activate
	
	--following will be determined by type that is given on activation
	self.width = 13 --for other shooters there could be variation between bullettype sizes, here it isn't
	self.height= 9
		
	--set bullet position
	self.x = aPX
	self.y = aPY
		
	--set bullet speeds
	self.xSpeedStep     = aSX
	self.ySpeedStep     = aSY
	
	self.anmStep = 1
	self.frames[1] = sateliteCollectibleGph[1]
	self.frames[2] = sateliteCollectibleGph[2]
	
	self.timer = self.timerMax
	
end


function SateliteChunk:deactivate()
	--set inactive, remove current-fields of boundingbox, sizes, graphics ect
	
	--set defaults
	self:resetType()
end

function SateliteChunk.doUpdate(self)

	
	if self.type ~= S_CHUNK_INACTIVE then --when bullet not inactive
	
		self.anmStep = self.anmStep + 1
		
		if self.anmStep > self.anmThresh then
			self.anmStep = 1
		end
		
		
		--##############################
		--# bullet movements
		--##############################
		
		--bullet movement in x-directions
		self.xSpeedProgress = self.xSpeedProgress + self.xSpeedStep
		while self.xSpeedProgress >= self.xSpeedThresh do --for positive x-Movement
			self.xSpeedProgress = self.xSpeedProgress - self.xSpeedThresh
			self.x = self.x + 1
		end
		while self.xSpeedProgress <= ((-1) * self.xSpeedThresh) do --for negative x-Movement
			self.xSpeedProgress = self.xSpeedProgress + self.xSpeedThresh
			self.x = self.x - 1
		end
		--bullet movements in y directions 
		self.ySpeedProgress = self.ySpeedProgress + self.ySpeedStep 
		
		while self.ySpeedProgress >= self.ySpeedThresh do --for positive y-Movement (not used in this game)
			self.ySpeedProgress = self.ySpeedProgress - self.ySpeedThresh
			self.y = self.y + 1
		end
		while self.ySpeedProgress <= ((-1) * self.ySpeedThresh) do --for negative y-Movement (standard case)
			self.ySpeedProgress = self.ySpeedProgress + self.ySpeedThresh
			self.y = self.y - 1
		end
		
		
		--##############################
		--# bullet deactivation on boundaries
		--##############################
		
		if self.x < self.boundsLeft then
			self:deactivate()
		end
		if self.x > self.boundsRight then
			self.x = self.boundsRight --stop at right edge of screen
		end
		if self.y < self.boundsTop then
			self:deactivate()
		end
		if self.y > self.boundsBtm then
			self:deactivate()
		end
		
		
		if self.type ~= S_CHUNK_INACTIVE then --test again cause it could be deactivated by now, have more conditions here
		
			if self.timer > 0 then
				self.timer = self.timer - 1 
			end
			
			if self.timer <= 0 then
				self.plantCollectible(self)
				self:deactivate()
			end
		
		end
		
		
	end
	
	--for collision-handling: see gamestates!

end

function SateliteChunk.plantCollectible(self)
	
	
	globSateliteCollectible[globSateliteCollectibleIndex].putStartPosition(globSateliteCollectible[globSateliteCollectibleIndex], self.x, self.y)
	globSateliteCollectible[globSateliteCollectibleIndex].activate(globSateliteCollectible[globSateliteCollectibleIndex])
	
	globSateliteCollectibleIndex = globSateliteCollectibleIndex + 1
	if globSateliteCollectibleIndex > globSateliteCollectibleIndexMax then
		globSateliteCollectibleIndex = 1
	end
	
end



function SateliteChunk.draw(self)
	if self.type ~= S_CHUNK_INACTIVE then
		lg.draw(self.frames[self.anmStep], self.x, self.y, 0, 1, 1)
		
		--debug
		--self.currentBBox.draw(self.currentBBox)
	end
end

