require "luas.shortcuts"
require "luas.extendedLoader"

SateliteCollectible = Object:extend()


function SateliteCollectible.new(self) --, aPosX, aPosY) position only known at activation time
	
	self:reset() --lots of other values initialized here and also reset to defaults on deactivation

end


function SateliteCollectible:reset()

	self.active = false	
	
	self.hitPoints = 1 --basic before activation
	
	self.takenHits = 0
	
	self.width  = 0 
	self.height = 0 
	
	self.currentGph  = nil
	
	self.x = 0
	self.y = 0
	
	self.startPosX = 0 
	self.startPosY = 0
	
	self.boundsLeft  = (-1)*13
	
	self.anmStep = 0 --ineffective values
	self.anmProgress = 0
	self.anmThresh = 1
	self.anmIndex = 0 --invalid values
	self.anmIndexMax = 2
	
	self.moveStep = 0
	self.moveThresh = 3

	
end

function SateliteCollectible.putStartPosition(self, aPx, aPy)

	self.x = aPx
	self.y = aPy
	
	self.startPosX = aPx 
	self.startPosY = aPy
end

function SateliteCollectible.activate(self)
	
	self.active = true
	
	self.hitPoints = 1
	
	self.anmStep = 1 --effective values
	self.anmIndex = 1 --valid values
	
	self.currentGph = sateliteCollectibleGph[self.anmIndex]
	
	self.enmWidth  = 13
	self.enmHeight = 9
	
	self.moveStep = 1
	
end

function SateliteCollectible.takeHit(self, aDmg)

	self.takenHits = self.takenHits + aDmg --adding up damage from multiple bullets in 1 update possible
end

function SateliteCollectible.deactivate(self)

	--self.active = false --is not enough, will cause carry-over of behaviors such as matrix/roaming mode
	
	--clean it up on deactivation
	self:reset()
end

function SateliteCollectible.doUpdate(self)


	if self.active == true then
	
		if self.takenHits > 0 then --flag set by gamemode, need to take hp and check for death
			self.hitPoints = self.hitPoints - self.takenHits
			
			self.takenHits = 0 --otherwise will take more hitpoints in next update
		
			if self.hitPoints <= 0 then

				
				--if globIsInDemoMode == false then 
				--	impactSnd:stop()
				--	impactSnd:play()
				--end
				
				self.deactivate(self)
			end
		end
	end
	
	--only if still active
	if self.active == true then
		self.doAnimation(self)
		self.executeFlyMove(self) --always as long as active: move along the x axis
	end
end

function SateliteCollectible.doAnimation(self)

	self.anmProgress = self.anmProgress + self.anmStep
	
	if self.anmProgress >= self.anmThresh then
		self.anmProgress = 0
		
		self.anmIndex = self.anmIndex + 1
		
		if self.anmIndex > self.anmIndexMax then
			self.anmIndex = 1
		end
		
		self.currentGph = sateliteCollectibleGph[self.anmIndex]
	end

end

function SateliteCollectible.executeFlyMove(self)

	--scrolling movement pixel
	self.moveStep = self.moveStep + 1
	if self.moveStep > self.moveThresh then
		self.moveStep = 1
		
		self.x = self.x - 1
	end
	
	if self.x < self.boundsLeft then
		self.deactivate(self)
	end 
	
end

function SateliteCollectible.draw(self)

	if self.active then	
		
		lg.draw(self.currentGph, self.x, self.y, 0, 1, 1)
	end
end







