require "luas.shortcuts"
require "luas.globals"


GameMode = Object:extend()

function GameMode.new(self)

	self.leaveStateReady  = false
	self.leaveStateWithDevice = GLOB_INPUT_WAITING
	
		--
	self.quitWithKeyboardFirePress = false
	self.quitWithGamepadFirePress  = false
	self.quitWithGamepad2FirePress = false
	self.quitWithGamepad3FirePress = false
	self.quitWithMouseFirePress    = false
	self.blockMouseOnWindowbarY    = 0
	--
	--
	self.leaveStateGameOver = false      --SEPARATE FROM GAMEOVER FLAG: enabling leaving State with Gameover Condition
	self.leaveStateGameOverCountdown = 0 --SEPARATE FROM GAMEOVER FLAG: start with gameover, block return to demo for short time
	self.LEAVE_STATE_GAME_OVER_DELAY = 40--SEPARATE FROM GAMEOVER FLAG: min time before return to demo
	--
	self.gameOverFlag         = false	--use at timepoint of gameover to display Gameover Writing
	--addition
	self.autoLeaveStateGameOverCountdown = 0 --SEPARATE FROM GAMEOVER FLAG: start with gameover, block return to demo for short time
	self.AUTO_LEAVE_STATE_GAME_OVER_DELAY = 330
	--self.gameOverAnmStep      = 1		--do not animate gameover for this game
	--self.gameOverAnmProgress  = 0   	
	--self.gameOverAnmTresh     = 10     	
	--self.gameOverFrame 		  = 1	

	self.levelBackGph   = backGroundGph
	--self.levelBackFlickerGph   = backGroundFlickerGph
	
	self.bkg_x = 0
	self.bkg_y = 0  
	
	self.levelLoaded  = false --start of not having loaded the level
	self.levelCleared = false --start of not having not cleared the level
	
	--for level transition allow time to pass, in between may take additional steps at key values of countdown
	--such as smooth fade in and fade out of the background grid color, change level display, reload level ect
	self.levelClearedCountDown = 0
	
	self.inputControlField = {}  --can't access maxplayers from globals
	--for i= 1,globMaxPlayers,1 do --globMaxPlayers not yet defined, causes inconsistency, but...
		--self.inputControlField[i] = GLOB_INPUT_WAITING 
		self.inputControlField[1] = GLOB_INPUT_WAITING
		self.inputControlField[2] = GLOB_INPUT_WAITING --but it needs initialisation
		self.inputControlField[3] = GLOB_INPUT_WAITING
	--end
	
	self.playerTookHit = {} 
	self.playerTookHit[1] = false--can't access maxplayers from globals
	self.playerTookHit[2] = false
	self.playerTookHit[3] = false--with this construct can let multiple players get hit and die simultaneously
	self.playerDyingCountDown       = 0
	self.playerDyingCountDownMax    = 6
	
	-- with this, force smallUpdates only during playerDying Phase
	self.doUpdatesFlag = true
	--
	--analogue to smallUpdate, players and objects can have smallRedraw automatism that does take care of the 
	--flickering with a inbuilt framecycle -> since automatism does not affect draw function, do not need to take
	--care so much about returning to proper point in altered-frame cycle (see white background error in other game)
	
	
	--don't let player join when mouse-event on window-bar instead of playfield is worked through
	self.blockMouseOnWindowbarY = 0
	
	self.debug = 0
	
	
	self.showIntro = true
	self.introTimer = 160
	
	self.startedScroll = false --to have the first frame with zero scroll-progress
	self.scrollProgress = 0--wave 12: 5875--wave 11: 3810 --wave 10: 2170--wave 8: 6423--wave 7: 5351--wave 6: 4240--wave 5: 1639--wave 4 level 2:0--wave3 5750
	
	
	self.isPaused = false
	self.pauseCountDown = 0
	self.pauseCountDownMax = 20
	
	self.showEndingExplosions = false

end


function GameMode:update(dt)
	--do not depend on deltatime dt here, use doUpdate() instead
end

--replace update
function GameMode:doUpdate()	

	if self.showIntro == true then
	
		if globLevel < 4 then --globLevel < 4 is real intro
		
			self.introTimer = self.introTimer - 1
		
			if self.introTimer == 159 then
				levelIntroMusic:stop()
				levelIntroMusic:play()
			end
		
			if self.introTimer == 1 then
				self.introTimer = 0
				self.showIntro = false
			end
		else --globLevel 4 is displaying the ending
		
			if globEnding.active == false then
				globEnding.activate(globEnding)
				self.showEndingExplosions = true
			end
		
			globEnding.doUpdate(globEnding)
		
			if globExplosionMothership ~= nil then
			
				if self.doUpdatesFlag == true then
			
					globExplosionMothership.doUpdate(globExplosionMothership)
				end
			end
			
			--also need to update explosions here or there will be still images
			for i=1,globSmallExplosionMax,1 do
				if globSmallExplosions[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						globSmallExplosions[i].doUpdate(globSmallExplosions[i])
					end
		
				end
			end
			
			for i=1,globMediumExplosionMax,1 do
				if globMediumExplosions[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						globMediumExplosions[i].doUpdate(globMediumExplosions[i])
					end
		
				end
			end
			
		end
	else

	
		if self.isPaused == false then
																--Task AAM (part 2) update game mode

																
																
			--debug
			if lkeyBrd("d") == true then
				print("Scrollprogress:"..self.scrollProgress.." RunningOid:"..globRunningOID.." ")
			end
	
		
	
			--#######################################################
			--# do joining of 2nd or 3rd player  (NO-UPDATE-FUNCTION)
			--#######################################################
	
	
	
			--######################################
			--# do scrolling
			--######################################
	
			self.doScrolling(self)
	
			--#######################
			--# update player       #
			--#######################
		
			for i = 1,globMaxPlayers,1 do
	
				if self.doUpdatesFlag == true then

					--FULL UPDATE
			
					if self.debug == 0 then
						self.debug = self.debug + 1
						print("    +++ DOING   FIRST   NORMAL  UPDATE   IN   A   ROW +++  ")
					end
			
					globPlShip[i].doUpdate(globPlShip[i]) --different players need different shipcolors
				end
			end
	
	
			--#######################
			--# update player bullets
			--#######################
	
			for i=1,globMaxPlayers,1 do 
				for j=1,globLeadPlBulletIndexMax,1 do
		
					if self.doUpdatesFlag == true then

						--FULL UPDATE
						globLeadPlBullets[i][j].doUpdate(globLeadPlBullets[i][j])
					end
				end
			end
			
			--###################################################################
			--# do playerbullet-foreground collision handling (NO-UPDATE-FUNCTION)
			--###################################################################
			
			if self.doUpdatesFlag == true then

				self.playerBulletForegroundCollisionHandling(self) --no-update function
			end
	
			--###############################################################
			--# do playerbullet-enemy collision handling (NO-UPDATE-FUNCTION)
			--###############################################################
			
			if self.doUpdatesFlag == true then

				self.playerBulletEnmCollisionHandling(self) --no-update function
			end
	
	
			--#############################
			--# decide for enemies shooting
			--#############################
		
			--#######################
			--# update powerup
			--#######################
	
			if globPowerup ~= nil then
				if self.doUpdatesFlag == true then
					globPowerup.doUpdate(globPowerup)
				end
			end
			
			--########################
			--# update satelite chunks
			--########################
			
			for i = 1,globSateliteChunkIndexMax,1 do
				if globSateliteChunk[i] ~= nil then
					if self.doUpdatesFlag == true then
						globSateliteChunk[i].doUpdate(globSateliteChunk[i])
					end
				end
			end
			
			--###############################
			--# update satelite collectibles
			--##############################
			
			for i = 1,globSateliteCollectibleIndexMax,1 do
				if globSateliteCollectible[i] ~= nil then
					if self.doUpdatesFlag == true then
						globSateliteCollectible[i].doUpdate(globSateliteCollectible[i])
					end
				end
			end
	
			--#######################
			--# update enemies 
			--#######################
			
			for i = 1,globMaxEnemyA,1 do
				if globEnemyA[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyA[i].doUpdate(globEnemyA[i])
					end
				end
			end
			
			for i = 1,globMaxEnemyB,1 do
				if globEnemyB[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyB[i].doUpdate(globEnemyB[i])
					end
				end
			end
			
			for i = 1,globMaxEnemyC,1 do
				if globEnemyC[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyC[i].doUpdate(globEnemyC[i])
					end
				end
			end
			
			for i = 1,globMaxEnemyD,1 do
				if globEnemyD[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyD[i].doUpdate(globEnemyD[i])
					end
				end
			end
			
			for i = 1,globMaxEnemyE,1 do
				if globEnemyE[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyE[i].doUpdate(globEnemyE[i])
					end
				end
			end
			
			for i = 1,globMaxEnemyF,1 do
				if globEnemyF[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyF[i].doUpdate(globEnemyF[i])
					end
				end
			end
			
			if globEnemyG ~= nil then
				if self.doUpdatesFlag == true then
					globEnemyG.doUpdate(globEnemyG)
				end
			end
			
			for i = 1,globMaxEnemyH,1 do
				if globEnemyH[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyH[i].doUpdate(globEnemyH[i])
					end
				end
			end
			
			for i = 1,globMaxEnemyI,1 do
				if globEnemyI[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyI[i].doUpdate(globEnemyI[i])
					end
				end
			end
			
			for i = 1,globMaxEnemyJ,1 do
				if globEnemyJ[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyJ[i].doUpdate(globEnemyJ[i])
					end
				end
			end
	
			for i = 1,globMaxEnemyK,1 do
				if globEnemyK[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyK[i].doUpdate(globEnemyK[i])
					end
				end
			end
			
			for i = 1,globMaxEnemyL,1 do
				if globEnemyL[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyL[i].doUpdate(globEnemyL[i])
					end
				end
			end
			
			for i = 1,globMaxEnemyM,1 do
				if globEnemyM[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyM[i].doUpdate(globEnemyM[i])
					end
				end
			end
			
			for i = 1,globMaxEnemyN,1 do
				if globEnemyN[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyN[i].doUpdate(globEnemyN[i])
					end
				end
			end
			
			
			for i = 1,globMaxEnemyO,1 do
				if globEnemyO[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyO[i].doUpdate(globEnemyO[i])
					end
				end
			end
			
			for i = 1,globMaxEnemyP,1 do
				if globEnemyP[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyP[i].doUpdate(globEnemyP[i])
					end
				end
			end
			
			for i = 1,globMaxEnemyQ,1 do
				if globEnemyQ[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyQ[i].doUpdate(globEnemyQ[i])
					end
				end
			end
			
			for i = 1,globMaxEnemyR,1 do
				if globEnemyR[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyR[i].doUpdate(globEnemyR[i])
					end
				end
			end
			
			for i = 1,globMaxEnemyS,1 do
				if globEnemyS[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyS[i].doUpdate(globEnemyS[i])
					end
				end
			end
			
			for i = 1,globMaxEnemyT,1 do
				if globEnemyT[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyT[i].doUpdate(globEnemyT[i])
					end
				end
			end
			
			for i = 1,globMaxEnemyU,1 do
				if globEnemyU[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyU[i].doUpdate(globEnemyU[i])
					end
				end
			end
			
			for i = 1,globMaxEnemyV,1 do
				if globEnemyV[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyV[i].doUpdate(globEnemyV[i])
					end
				end
			end
			
			for i = 1,globMaxEnemyW,1 do
				if globEnemyW[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyW[i].doUpdate(globEnemyW[i])
					end
				end
			end
			
			for i = 1,globMaxEnemyX,1 do
				if globEnemyX[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyX[i].doUpdate(globEnemyX[i])
					end
				end
			end
			
			for i = 1,globMaxEnemyY,1 do
				if globEnemyY[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyY[i].doUpdate(globEnemyY[i])
					end
				end
			end
			
			for i = 1,globMaxEnemyZ,1 do
				if globEnemyZ[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						--FULL UPDATE
						globEnemyZ[i].doUpdate(globEnemyZ[i])
					end
				end
			end
			
	
			--#######################
			--# update big enemies 
			--#######################
			
			if globEnemyBoss1 ~= nil then
				if self.doUpdatesFlag == true then
				
					globEnemyBoss1.doUpdate(globEnemyBoss1)
				end
			end
			
			if globEnemyBoss2 ~= nil then
				if self.doUpdatesFlag == true then
				
					globEnemyBoss2.doUpdate(globEnemyBoss2)
				end
			end
			
			if globEnemyBoss3 ~= nil then
				if self.doUpdatesFlag == true then
				
					globEnemyBoss3.doUpdate(globEnemyBoss3)
				end
			end
			
			if globBossBack ~= nil then
				if self.doUpdatesFlag == true then
					globBossBack.doUpdate(globBossBack)
				end
			end
	

			--#######################
			--# update enemy bullets
			--#######################
			
			for i = 1, globEnmBulletsAIndexMax, 1 do
	
				if globEnmBulletsA[i] ~= nil then
		
					if self.doUpdatesFlag == true then

						--FULL UPDATE
						globEnmBulletsA[i].doUpdate(globEnmBulletsA[i])
					end
				end
			end
			
			for i = 1, globEnmBulletsBIndexMax, 1 do
	
				if globEnmBulletsB[i] ~= nil then
		
					if self.doUpdatesFlag == true then

						--FULL UPDATE
						globEnmBulletsB[i].doUpdate(globEnmBulletsB[i])
					end
				end
			end
			
			for i = 1, globEnmBulletsCIndexMax, 1 do
	
				if globEnmBulletsC[i] ~= nil then
		
					if self.doUpdatesFlag == true then

						--FULL UPDATE
						globEnmBulletsC[i].doUpdate(globEnmBulletsC[i])
					end
				end
			end
			
			for i = 1, globEnmBulletsDIndexMax, 1 do
	
				if globEnmBulletsD[i] ~= nil then
		
					if self.doUpdatesFlag == true then

						--FULL UPDATE
						globEnmBulletsD[i].doUpdate(globEnmBulletsD[i])
					end
				end
			end
			
			for i = 1, globEnmBulletsEIndexMax, 1 do
	
				if globEnmBulletsE[i] ~= nil then
		
					if self.doUpdatesFlag == true then

						--FULL UPDATE
						globEnmBulletsE[i].doUpdate(globEnmBulletsE[i])
					end
				end
			end
			
			for i = 1, globEnmBulletsFIndexMax, 1 do
	
				if globEnmBulletsF[i] ~= nil then
		
					if self.doUpdatesFlag == true then

						--FULL UPDATE
						globEnmBulletsF[i].doUpdate(globEnmBulletsF[i])
					end
				end
			end
			
			for i = 1, globEnmBulletsGIndexMax, 1 do
	
				if globEnmBulletsG[i] ~= nil then
		
					if self.doUpdatesFlag == true then

						--FULL UPDATE
						globEnmBulletsG[i].doUpdate(globEnmBulletsG[i])
					end
				end
			end
	
	
			--##########################
			--# update enemy BIG bullets
			--##########################

	
			--#######################
			--# update explosions
			--#######################
	
			
			for i=1,globSmallExplosionMax,1 do
				if globSmallExplosions[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						globSmallExplosions[i].doUpdate(globSmallExplosions[i])
					end
		
				end
			end
			
			for i=1,globMediumExplosionMax,1 do
				if globMediumExplosions[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						globMediumExplosions[i].doUpdate(globMediumExplosions[i])
					end
		
				end
			end
			
			for i=1,globBigExplosionMax,1 do
				if globBigExplosions[i] ~= nil then
		
					if self.doUpdatesFlag == true then
			
						globBigExplosions[i].doUpdate(globBigExplosions[i])
					end
		
				end
			end
			
			
			if globBoss1Explosion ~= nil then
		
				if self.doUpdatesFlag == true then
			
					globBoss1Explosion.doUpdate(globBoss1Explosion)
				end
			end
			
			if globBoss2Explosion ~= nil then
		
				if self.doUpdatesFlag == true then
			
					globBoss2Explosion.doUpdate(globBoss2Explosion)
				end
			end
			
			if globBoss3Explosion ~= nil then
		
				if self.doUpdatesFlag == true then
			
					globBoss3Explosion.doUpdate(globBoss3Explosion)
				end
			end
			
			if globExplosion1000 ~= nil then
			
				if self.doUpdatesFlag == true then
			
					globExplosion1000.doUpdate(globExplosion1000)
				end
			end
			
			--if globExplosionMothership ~= nil then --no, need to put this in the intro-part of this uber-update
			
			--end
			
	
	
			--##########################
			--# update Player explosions
			--##########################
			
			if globPlayerExplosion ~= nil then
			
				if self.doUpdatesFlag == true then
					globPlayerExplosion.doUpdate(globPlayerExplosion)
				end
			end
	
	
			--###########################################
			--#check for player collision with foreground
			--###########################################
			--
	
			if self.doUpdatesFlag == true then

				self.foregroundPlayerCollisionHandling(self)	
			end
	
			--#####################################
			--#check for player taking a bullet hit
			--#####################################
			--
			
			if self.doUpdatesFlag == true then

				self.enmBulletPlayerCollisionHandling(self)	
			end

	
			--######################################
			--#check for player colliding with enemy
			--######################################
			--
			
			if self.doUpdatesFlag == true then
				self.enemyPlayerCollisionHandling(self)
			end
			
			--########################################
			--#check for player colliding with powerup (also satelite collectibles)
			--########################################
			--
			
			if self.doUpdatesFlag == true then
				self.powerupPlayerCollisionHandling(self)
			end
			
			
			--##########################################################################
			--# fill level according to current global level number (NO-UPDATE-FUNCTION)
			--##########################################################################
	
			if self.scrollProgress < (globScrollWidth[globLevel] - resWidth) then --only when smaller than max, or else it will crash
				globFillLevel(self.scrollProgress)
			end
	
			--########################
			--#check for level cleared
			--########################
			
			if self.doUpdatesFlag == true then
				self.levelClearAndFadeoutHandling(self)
			end
	
			--########################
			--#update mask printer
			--########################
			
			--if self.doUpdatesFlag == true then 	--blocking updates for this one may cause irritating behavior 
											--with not actualizing lifes (when there is no correct respawn invulnerability)
				--FULL UPDATE
				globMaskPrinter.doUpdate(globMaskPrinter) --update as gamemode --TODO in demoMode make difference
			--end
	
	
			--#########################################################
			--# have background flicker on playerdeath only for so long
			--#########################################################
	
	
			--#########################################
			--#Do Gameover-checks and Gameover handling
			--#########################################
			
			if self.doUpdatesFlag == true then --in this game block gameovercheck during level transition, hence the doUpdatesFlag
	
				self.doGameOverCheck(self)
			end
			
	
		
		end
	
	
		self.checkPauseUnPause(self)
	
	
		--also unaffected by updateBlocker:
		--printing of gameover separate from mask, usually over all other objects
		if self.gameOverFlag == true then --printing of gameover separate from mask, usually over all other objects
			--
			if self.leaveStateGameOverCountdown < self.LEAVE_STATE_GAME_OVER_DELAY then
				self.leaveStateGameOverCountdown = self.leaveStateGameOverCountdown + 1
				
				if self.leaveStateGameOverCountdown == 20 then
					gameOverMusic:stop()
					gameOverMusic:play()
				end
			else
				--##################################################################################
				--#	Read and handle Controls that need to be processed in Gamestate (for State exit)
				--##################################################################################
				--
				self:keyProcess()
				
				--in this game, if no key is pressed to exit to main menu,
				--gameover will turn back to main menu automatically
				--
				if self.leaveStateReady == false then --only have automatic response if response was not already triggered by key input
					if self.autoLeaveStateGameOverCountdown < self.AUTO_LEAVE_STATE_GAME_OVER_DELAY then
						self.autoLeaveStateGameOverCountdown = self.autoLeaveStateGameOverCountdown + 1
						
						--if self.autoLeaveStateGameOverCountdown == 5 then --above
						--	gameOverMusic:stop()
						--	gameOverMusic:play()
						--end
					else
						self.leaveStateReady  = true
						self.leaveStateWithDevice = GLOB_INPUT_PASSIVE
					end
				else
					--quit with keypress needs to stop gameover music or else it will overlap with titlescreen music
					gameOverMusic:stop()
				end
			end
			--
		end
	
	
	
	
		--if self.doUpdatesFlag == false then 
											
		--	self.playerDeathHandling(self)
		
		--end
	
	end
	
end


function GameMode:keyProcess()
	--BUG: this will allow you to exit the gamestate on gameover with EVERY firebutton and not just with the firebutton of the device, which may be unwanted
	--
	self.quitWithKeyboardFirePress = lkeyBrd("z")
	self.quitWithMouseFirePress    = lMouseDown("l")
	--
	local lclOldMousePos = self.blockMouseOnWindowbarY
	self.blockMouseOnWindowbarY = lMouseGetY()
	
	if self.quitWithMouseFirePress == true then
		print("mouse start on coordinate"..self.blockMouseOnWindowbarY)
	end
	
	if self.blockMouseOnWindowbarY <= 0 then --if mousefire outside playfield (happens on windowbar gaining focus), 
		self.quitWithMouseFirePress = false  --then DON'T react to it
	elseif lclOldMousePos <= 0 and self.blockMouseOnWindowbarY > 0 then
		self.quitWithMouseFirePress = false --even when just re-entering window and mouseevent still keeps prending
		                                    --when is was a click on windowbar then DON'T REACT TO THAT FUCKING MOUSEPRESS
		--TODO: still fail, also see gameMode (checkPlayerJoin)
	end
	
	
	if globJoystick then
			
		--dleft  = globJoystick:isGamepadDown("dpleft")
		--dright = globJoystick:isGamepadDown("dpright")
		
		--TODO can do for now, but need to find start button
		--Btns 1,2,3 directional (Number for up-Button Missing?)
		--Btns 8, 9 on top
		--Btns 10, 11, 12, 13 Roundbuttons (Fire)
		--other buttons not yet found up until index 25
		
		
		self.quitWithGamepadFirePress = globJoystick:isGamepadDown("a", "b", "x", "y")
		
		
		--if globJoystick:isDown(1) then
		--else
		--end
	else
		--print("No Joystick")
	end
	
	
	if globJoystick2 then --condition for joystick plugged in, tested in globals
		
		self.quitWithGamepad2FirePress = globJoystick2:isGamepadDown("a", "b", "x", "y")

	else
		--print("No Joystick2")
	end
	
	if globJoystick3 then --condition for joystick plugged in, tested in globals
		
		self.quitWithGamepad3FirePress = globJoystick3:isGamepadDown("a", "b", "x", "y")

	else
		--print("No Joystick2")
	end
	
	--self.quitWithGamepadFirePress  = --TODO Gamepad
	--self.quitWithMouseFirePress    = --TODO mouse
	
	--device to be carried over to game (use for 1st player)
	if self.quitWithKeyboardFirePress == true then
	
		self.leaveStateReady  = true
		self.leaveStateWithDevice = GLOB_INPUT_KEYBOARD
		print("GameMode: Keyboard input detected")
		
		--swirl1Snd:stop()
		--swirl2Snd:stop()
		--swirl3Snd:stop()
		--swirl4Snd:stop()
		--swirl5Snd:stop()
		
	elseif self.quitWithGamepadFirePress == true then
	
		self.leaveStateReady  = true
		self.leaveStateWithDevice = GLOB_INPUT_GAMEPAD
		print("GameMode: Gamepad input detected")
		
		--swirl1Snd:stop() --make it silent for demomode
		--swirl2Snd:stop()
		--swirl3Snd:stop()
		--swirl4Snd:stop()
		--swirl5Snd:stop()
		
	elseif self.quitWithGamepad2FirePress == true then
	
		self.leaveStateReady  = true
		self.leaveStateWithDevice = GLOB_INPUT_GAMEPAD2
		print("GameMode: Gamepad input detected")
		
		--swirl1Snd:stop() --make it silent for demomode
		--swirl2Snd:stop()
		--swirl3Snd:stop()
		--swirl4Snd:stop()
		--swirl5Snd:stop()
	
	elseif self.quitWithGamepad3FirePress == true then
	
		self.leaveStateReady  = true
		self.leaveStateWithDevice = GLOB_INPUT_GAMEPAD3
		print("GameMode: Gamepad input detected")
		
		--swirl1Snd:stop() --make it silent for demomode
		--swirl2Snd:stop()
		--swirl3Snd:stop()
		--swirl4Snd:stop()
		--swirl5Snd:stop()
	
	elseif self.quitWithMouseFirePress == true then
	
		self.leaveStateReady  = true
		self.leaveStateWithDevice = GLOB_INPUT_MOUSE
		print("GameMode: Mouse input detected")
		
		--swirl1Snd:stop() --make it silent for demomode
		--swirl2Snd:stop()
		--swirl3Snd:stop()
		--swirl4Snd:stop()
		--swirl5Snd:stop()
		
	end
	
end



function GameMode.checkPlayerJoin(self)


end

function GameMode.playerBulletForegroundCollisionHandling(self)

	for i=1,globMaxPlayers,1 do 
		for j=1,globLeadPlBulletIndexMax,1 do
		
			if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
				local lclBulletHitTarget = false
				
				local imgdat = globLeadPlBullets[i][j].currentGph:getData()
				local fieldW = imgdat:getWidth()
				local fieldH = imgdat:getHeight()
						
				local r, g, b, a =  0
			
				local imgdat2 = levelForegroundGphs[globLevel]:getData()
						
				local r2, g2, b2, a2 =  0
		
				
				for m = 1,fieldW, 1 do
					for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
						r, g, b, a =  imgdat:getPixel(m-1, n-1)
						
						
						
						
						if self.scrollProgress + globLeadPlBullets[i][j].x + m - 1 < globScrollWidth[globLevel] then --avoid out of bounds horizontally
						
							if globLeadPlBullets[i][j].y + n - 1 >= 0 and globLeadPlBullets[i][j].y + n - 1 <= 149 then --avoid out of bounds vertically
						
								r2, g2, b2, a2 =  imgdat2:getPixel(self.scrollProgress + globLeadPlBullets[i][j].x + m - 1, globLeadPlBullets[i][j].y + n - 1)
								
								if lclBulletHitTarget == false then
									if a ~= 0 and a2 ~= 0 then
										
										lclBulletHitTarget = true
										
									end
								end
							end
						
						end
							
							
					end
				end
				
				if lclBulletHitTarget == true then
					globLeadPlBullets[i][j]:deactivate()
				end
			end
		
		end
	end
	
end

function GameMode.playerBulletEnmCollisionHandling(self)

	for i=1,globMaxPlayers,1 do 
		for j=1,globLeadPlBulletIndexMax,1 do
		
			for k = 1,globMaxEnemyA,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyA[k] ~= nil and globEnemyA[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyA[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyA[k].x + m 
									testY = globEnemyA[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyA[k].takeHit(globEnemyA[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			for k = 1,globMaxEnemyB,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyB[k] ~= nil and globEnemyB[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyB[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyB[k].x + m 
									testY = globEnemyB[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyB[k].takeHit(globEnemyB[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			for k = 1,globMaxEnemyC,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyC[k] ~= nil and globEnemyC[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyC[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyC[k].x + m 
									testY = globEnemyC[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyC[k].takeHit(globEnemyC[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
			for k = 1,globMaxEnemyD,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyD[k] ~= nil and globEnemyD[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyD[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyD[k].x + m 
									testY = globEnemyD[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyD[k].takeHit(globEnemyD[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
			
			for k = 1,globMaxEnemyE,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyE[k] ~= nil and globEnemyE[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyE[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyE[k].x + m 
									testY = globEnemyE[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyE[k].takeHit(globEnemyE[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
			
			
			for k = 1,globMaxEnemyF,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyF[k] ~= nil and globEnemyF[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyF[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyF[k].x + m 
									testY = globEnemyF[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyF[k].takeHit(globEnemyF[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
			
			
			
			--Enemy G
			if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
				local lclBulletHitTarget = false
				
				if globEnemyG ~= nil and globEnemyG.active == true then
						
						
					--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
					local lclBlHit = false
						
						
					local imgdat = globEnemyG.currentGph:getData()
					local fieldW = imgdat:getWidth()
					local fieldH = imgdat:getHeight()
					local testX = 0
					local testY = 0
						
					local r, g, b, a =  0
	
					for m = 1,fieldW, 1 do
						for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
							r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
							if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
								testX = globEnemyG.x + m 
								testY = globEnemyG.y + n 
								lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
							end
						end
					end
						
						
					if lclBlHit == true then
							
						lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
						globEnemyG.takeHit(globEnemyG, globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
					end
						
				end
					
				if lclBulletHitTarget == true then
					globLeadPlBullets[i][j]:deactivate()
				end
						
			end
			
			
			
			
			
			for k = 1,globMaxEnemyH,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyH[k] ~= nil and globEnemyH[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyH[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyH[k].x + m 
									testY = globEnemyH[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyH[k].takeHit(globEnemyH[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
			
			
			
			for k = 1,globMaxEnemyI,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyI[k] ~= nil and globEnemyI[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyI[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyI[k].x + m 
									testY = globEnemyI[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyI[k].takeHit(globEnemyI[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
			
			
			for k = 1,globMaxEnemyJ,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyJ[k] ~= nil and globEnemyJ[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyJ[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyJ[k].x + m 
									testY = globEnemyJ[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyJ[k].takeHit(globEnemyJ[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
			
						
			for k = 1,globMaxEnemyK,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyK[k] ~= nil and globEnemyK[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyK[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyK[k].x + m 
									testY = globEnemyK[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyK[k].takeHit(globEnemyK[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
			
			for k = 1,globMaxEnemyL,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyL[k] ~= nil and globEnemyL[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyL[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyL[k].x + m 
									testY = globEnemyL[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyL[k].takeHit(globEnemyL[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
			
			
			for k = 1,globMaxEnemyM,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyM[k] ~= nil and globEnemyM[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyM[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyM[k].x + m 
									testY = globEnemyM[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyM[k].takeHit(globEnemyM[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
			
			
			for k = 1,globMaxEnemyN,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyN[k] ~= nil and globEnemyN[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyN[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyN[k].x + m 
									testY = globEnemyN[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyN[k].takeHit(globEnemyN[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
				
			for k = 1,globMaxEnemyO,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyO[k] ~= nil and globEnemyO[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyO[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyO[k].x + m 
									testY = globEnemyO[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyO[k].takeHit(globEnemyO[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
			for k = 1,globMaxEnemyP,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyP[k] ~= nil and globEnemyP[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyP[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyP[k].x + m 
									testY = globEnemyP[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyP[k].takeHit(globEnemyP[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
			
			for k = 1,globMaxEnemyQ,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyQ[k] ~= nil and globEnemyQ[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyQ[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyQ[k].x + m 
									testY = globEnemyQ[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyQ[k].takeHit(globEnemyQ[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
			
			
			for k = 1,globMaxEnemyR,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyR[k] ~= nil and globEnemyR[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyR[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyR[k].x + m 
									testY = globEnemyR[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyR[k].takeHit(globEnemyR[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
				
			for k = 1,globMaxEnemyS,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyS[k] ~= nil and globEnemyS[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyS[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyS[k].x + m 
									testY = globEnemyS[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyS[k].takeHit(globEnemyS[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
			
			for k = 1,globMaxEnemyT,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyT[k] ~= nil and globEnemyT[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyT[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyT[k].x + m 
									testY = globEnemyT[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyT[k].takeHit(globEnemyT[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
			
			for k = 1,globMaxEnemyU,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyU[k] ~= nil and globEnemyU[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyU[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyU[k].x + m 
									testY = globEnemyU[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyU[k].takeHit(globEnemyU[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
			
			for k = 1,globMaxEnemyV,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyV[k] ~= nil and globEnemyV[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyV[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyV[k].x + m 
									testY = globEnemyV[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyV[k].takeHit(globEnemyV[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
			
			for k = 1,globMaxEnemyW,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyW[k] ~= nil and globEnemyW[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyW[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyW[k].x + m 
									testY = globEnemyW[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyW[k].takeHit(globEnemyW[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
			
						
			for k = 1,globMaxEnemyX,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyX[k] ~= nil and globEnemyX[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyX[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyX[k].x + m 
									testY = globEnemyX[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyX[k].takeHit(globEnemyX[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
			
			
			
			for k = 1,globMaxEnemyY,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyY[k] ~= nil and globEnemyY[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyY[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyY[k].x + m 
									testY = globEnemyY[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyY[k].takeHit(globEnemyY[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			
			
			
			for k = 1,globMaxEnemyZ,1 do
			
				--TODO should separate bullettype from active flag
				if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
					local lclBulletHitTarget = false
				
					if globEnemyZ[k] ~= nil and globEnemyZ[k].active == true then
						
						
						--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
						local lclBlHit = false
						
						
						local imgdat = globEnemyZ[k].currentGph:getData()
						local fieldW = imgdat:getWidth()
						local fieldH = imgdat:getHeight()
						local testX = 0
						local testY = 0
						
						local r, g, b, a =  0
	
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
								if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
									testX = globEnemyZ[k].x + m 
									testY = globEnemyZ[k].y + n 
									lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
								end
							end
						end
						
						
						if lclBlHit == true then
							
							lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
							globEnemyZ[k].takeHit(globEnemyZ[k], globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
						end
						
					end
					
					if lclBulletHitTarget == true then
						globLeadPlBullets[i][j]:deactivate()
					end
						
				end
				
			end
			
			
			

			
			--Boss 1
			--
			if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
				local lclBulletHitTarget = false
				
				if globEnemyBoss1 ~= nil and globEnemyBoss1.active == true then
						
						
					--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
					local lclBlHit = false
						
						
					local imgdat = globEnemyBoss1.currentGph:getData()
					local fieldW = imgdat:getWidth()
					local fieldH = imgdat:getHeight()
					local testX = 0
					local testY = 0
						
					local r, g, b, a =  0
	
					for m = 1,fieldW, 1 do
						for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
							r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
							if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
								testX = globEnemyBoss1.x + m 
								testY = globEnemyBoss1.y + n 
								lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
							end
						end
					end
						
						
					if lclBlHit == true then
							
						lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
						globEnemyBoss1.takeHit(globEnemyBoss1, globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
					end
						
				end
					
				if lclBulletHitTarget == true then
					globLeadPlBullets[i][j]:deactivate()
				end
						
			end
				
				
				
			
			--Boss 2
			--
			if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
				local lclBulletHitTarget = false
				
				if globEnemyBoss2 ~= nil and globEnemyBoss2.active == true then
						
						
					--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
					local lclBlHit = false
						
						
					local imgdat = globEnemyBoss2.currentGph:getData()
					local fieldW = imgdat:getWidth()
					local fieldH = imgdat:getHeight()
					local testX = 0
					local testY = 0
						
					local r, g, b, a =  0
	
					for m = 1,fieldW, 1 do
						for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
							r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
							if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
								testX = globEnemyBoss2.x + m 
								testY = globEnemyBoss2.y + n 
								lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
							end
						end
					end
						
						
					if lclBlHit == true then
							
						lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
						globEnemyBoss2.takeHit(globEnemyBoss2, globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
					end
						
				end
					
				if lclBulletHitTarget == true then
					globLeadPlBullets[i][j]:deactivate()
				end
						
			end
			
			
			
			
			
			
			
			--Boss 3
			--
			if globLeadPlBullets[i][j] ~= nil and globLeadPlBullets[i][j].type ~= PLBULLET_INACTIVE then
				
				local lclBulletHitTarget = false
				
				if globEnemyBoss3 ~= nil and globEnemyBoss3.active == true then
						
						
					--local lclBlHit = globLeadPlBullets[i][j].currentBBox:CollisionTest(globEnmBots[k].currentBBox)
						
					local lclBlHit = false
						
						
					local imgdat = globEnemyBoss3.currentGph:getData()
					local fieldW = imgdat:getWidth()
					local fieldH = imgdat:getHeight()
					local testX = 0
					local testY = 0
						
					local r, g, b, a =  0
	
					for m = 1,fieldW, 1 do
						for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
							r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
							if a ~= 0 and lclBlHit == false then --test only for non-opaque pixel and only as long as no hit was detected
								testX = globEnemyBoss3.x + m 
								testY = globEnemyBoss3.y + n 
								lclBlHit = globLeadPlBullets[i][j].currentBBox.InBoxTest(globLeadPlBullets[i][j].currentBBox,testX, testY)
							end
						end
					end
						
						
					if lclBlHit == true then
							
						lclBulletHitTarget = true --this way 1 bullet can hit multiple targets at same place
							                          --and still be removed when at least 1 enemy was hit
													  
						globEnemyBoss3.takeHit(globEnemyBoss3, globLeadPlBullets[i][j].power + 1) --calculate "actual power" here, weakest shot has power variable 1 but deals 2 damage
					end
						
				end
					
				if lclBulletHitTarget == true then
					globLeadPlBullets[i][j]:deactivate()
				end
						
			end
			
			
			
			
			
			
			
		end
	end
	
end


function GameMode.prepareEnemyShooting_Lv1(self)


end


function GameMode.decideShootingLv2(self)


end

function GameMode.prepareEnemyShooting_Lv2(self)

end


function GameMode.levelClearAndFadeoutHandling(self)
	if globBossBeaten == true then
		globBossBeatTimer = globBossBeatTimer - 1
		
		if globBossBeatTimer == 1 then
			globBossBeaten = false
			globBossBeatTimer = 0
			
			globLevel = globLevel + 1
			--show intro to next level in next update
			self.showIntro = true
			self.introTimer = 160
			
			--reset scrolling
			self.startedScroll = false
			self.scrollProgress = 0
		end
	end
end

function GameMode.foregroundPlayerCollisionHandling(self)

	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
						
			local r, g, b, a =  0
			
			local imgdat2 = levelForegroundGphs[globLevel]:getData()
						
			local r2, g2, b2, a2 =  0
		
			local lclTookHit = false
			
			for m = 1,fieldW, 1 do
				for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
					r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
					r2, g2, b2, a2 =  imgdat2:getPixel(self.scrollProgress + globPlShip[i].x + m - 1, globPlShip[i].y + n - 1)
								
					if lclTookHit == false then
						if a ~= 0 and a2 ~= 0 then
										
							lclTookHit = true
										
						end
					end
					
					if lclTookHit == true then
															
						self.playerTookHit[i] = lclTookHit 					
					end
							
							
				end
			end
		
		end
	end
	
	--initialize player dying and flicker effect for dying
	for i = 1,globMaxPlayers,1 do
		if self.playerTookHit[i] == true then
		
			--self.playerDyingCountDown = self.playerDyingCountDownMax --not needed, no death flicker
			
			self.playerTookHit[i] = false
					
			globPlShip[i].putToDeath(globPlShip[i])
		end
	end

end

function GameMode.enmBulletPlayerCollisionHandling(self)

	--Bullet A
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globEnmBulletsAIndexMax,1 do
			
				if globEnmBulletsA[j] ~= nil and globEnmBulletsA[j].type ~= ENMBULLET_INACTIVE then
				
					local imgdat2 = globEnmBulletsA[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnmBulletsA[j].x + p, 0) --need to round because enemies on splines fire shots with unround positions
												testY2 = round(globEnmBulletsA[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
						globEnmBulletsA[j].deactivate(globEnmBulletsA[j])
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
	--Bullet B
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globEnmBulletsBIndexMax,1 do
			
				if globEnmBulletsB[j] ~= nil and globEnmBulletsB[j].type ~= ENMBULLET_INACTIVE then
				
					local imgdat2 = globEnmBulletsB[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = globPlShip[i].currentBBox:CollisionTest(globEnmBulletsB[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnmBulletsB[j].x + p, 0)
												testY2 = round(globEnmBulletsB[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
						globEnmBulletsB[j].deactivate(globEnmBulletsB[j])
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
	
	--Bullet C
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globEnmBulletsCIndexMax,1 do
			
				if globEnmBulletsC[j] ~= nil and globEnmBulletsC[j].type ~= ENMBULLET_INACTIVE then
				
					local imgdat2 = globEnmBulletsC[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = globPlShip[i].currentBBox:CollisionTest(globEnmBulletsC[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnmBulletsC[j].x + p, 0)
												testY2 = round(globEnmBulletsC[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
						globEnmBulletsC[j].deactivate(globEnmBulletsC[j])
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
	--Bullet D
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globEnmBulletsDIndexMax,1 do
			
				if globEnmBulletsD[j] ~= nil and globEnmBulletsD[j].type ~= ENMBULLET_INACTIVE then
				
					local imgdat2 = globEnmBulletsD[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = globPlShip[i].currentBBox:CollisionTest(globEnmBulletsD[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnmBulletsD[j].x + p, 0)
												testY2 = round(globEnmBulletsD[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
						globEnmBulletsD[j].deactivate(globEnmBulletsD[j])
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
	--Bullet E
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globEnmBulletsEIndexMax,1 do
			
				if globEnmBulletsE[j] ~= nil and globEnmBulletsE[j].type ~= ENMBULLET_INACTIVE then
				
					local imgdat2 = globEnmBulletsE[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = globPlShip[i].currentBBox:CollisionTest(globEnmBulletsE[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnmBulletsE[j].x + p, 0)
												testY2 = round(globEnmBulletsE[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
						globEnmBulletsE[j].deactivate(globEnmBulletsE[j])
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
	
	--Bullet F
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globEnmBulletsFIndexMax,1 do
			
				if globEnmBulletsF[j] ~= nil and globEnmBulletsF[j].type ~= ENMBULLET_INACTIVE then
				
					local imgdat2 = globEnmBulletsF[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = globPlShip[i].currentBBox:CollisionTest(globEnmBulletsF[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnmBulletsF[j].x + p, 0)
												testY2 = round(globEnmBulletsF[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
						globEnmBulletsF[j].deactivate(globEnmBulletsF[j])
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
	--Bullet G
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globEnmBulletsGIndexMax,1 do
			
				if globEnmBulletsG[j] ~= nil and globEnmBulletsG[j].type ~= ENMBULLET_INACTIVE then
				
					local imgdat2 = globEnmBulletsG[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = globPlShip[i].currentBBox:CollisionTest(globEnmBulletsG[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnmBulletsG[j].x + p, 0)
												testY2 = round(globEnmBulletsG[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
						globEnmBulletsG[j].deactivate(globEnmBulletsG[j])
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
	--initialize player dying and flicker effect for dying
	for i = 1,globMaxPlayers,1 do
		if self.playerTookHit[i] == true then
		
			--self.playerDyingCountDown = self.playerDyingCountDownMax --not needed, no death flicker
			
			self.playerTookHit[i] = false
					
			globPlShip[i].putToDeath(globPlShip[i])
		end
	end


end



function GameMode.enemyPlayerCollisionHandling(self)

	--Enemy A
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyA,1 do
			
				if globEnemyA[j] ~= nil and globEnemyA[j].active == true then
				
					local imgdat2 = globEnemyA[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyA[j].x + p
								testY2 = globEnemyA[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyA[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyA[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
	--Enemy B
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyB,1 do
			
				if globEnemyB[j] ~= nil and globEnemyB[j].active == true then
				
					local imgdat2 = globEnemyB[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyB[j].x + p
								testY2 = globEnemyB[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyB[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyB[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
	--Enemy C
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyC,1 do
			
				if globEnemyC[j] ~= nil and globEnemyC[j].active == true then
				
					local imgdat2 = globEnemyC[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyC[j].x + p
								testY2 = globEnemyC[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyC[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyC[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
	
	--Enemy D
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyD,1 do
			
				if globEnemyD[j] ~= nil and globEnemyD[j].active == true then
				
					local imgdat2 = globEnemyD[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyD[j].x + p
								testY2 = globEnemyD[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyD[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyD[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
		
	--Enemy E
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyE,1 do
			
				if globEnemyE[j] ~= nil and globEnemyE[j].active == true then
				
					local imgdat2 = globEnemyE[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyE[j].x + p
								testY2 = globEnemyE[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyE[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyE[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
		
		
	--Enemy F
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyF,1 do
			
				if globEnemyF[j] ~= nil and globEnemyF[j].active == true then
				
					local imgdat2 = globEnemyF[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyF[j].x + p
								testY2 = globEnemyF[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyF[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyF[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
			
		
	--Enemy G
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			
			
			if globEnemyG ~= nil and globEnemyG.active == true then
				
				local imgdat2 = globEnemyG.currentGph:getData()
				local fieldW2 = imgdat2:getWidth()
				local fieldH2 = imgdat2:getHeight()
				local testX2 = 0
				local testY2 = 0
						
				local r2, g2, b2, a2 =  0
				
				
					
				local lclTookHit = false
					
				local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
				--first test
				for p = 1,fieldW2, 1 do
					for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
						r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
						if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
							testX2 = globEnemyG.x + p
							testY2 = globEnemyG.y + q
								
							lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
						end
					end
				end
					
				--more accurate test
				if lclTestForHit == true then
						
					for m = 1,fieldW, 1 do
						for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
							r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
							for p = 1,fieldW2, 1 do
								for q = 1,fieldH2, 1 do
								
									r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
									if lclTookHit == false then
										if a ~= 0 and a2 ~= 0 then
										
											testX = globPlShip[i].x + m 
											testY = globPlShip[i].y + n 
											
											testX2 = round(globEnemyG.x + p, 0) --some enemies have unround position due to movement splines, so round
											testY2 = round(globEnemyG.y + q, 0)
											
											if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
												lclTookHit = true
											end
										
										end
									end
									
								end
							end
							
							
						end
					end
					
				end
					
					
				if lclTookHit == true then
															--see above:
					self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
				end
					
			end
		
		
		end
		
	end
	
	
	
	
	
		
		
	--Enemy H
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyH,1 do
			
				if globEnemyH[j] ~= nil and globEnemyH[j].active == true then
				
					local imgdat2 = globEnemyH[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyH[j].x + p
								testY2 = globEnemyH[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyH[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyH[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
		
	--Enemy I
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyI,1 do
			
				if globEnemyI[j] ~= nil and globEnemyI[j].active == true then
				
					local imgdat2 = globEnemyI[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyI[j].x + p
								testY2 = globEnemyI[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyI[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyI[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
		
	--Enemy J
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyJ,1 do
			
				if globEnemyJ[j] ~= nil and globEnemyJ[j].active == true then
				
					local imgdat2 = globEnemyJ[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyJ[j].x + p
								testY2 = globEnemyJ[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyJ[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyJ[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
		
	--Enemy K
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyK,1 do
			
				if globEnemyK[j] ~= nil and globEnemyK[j].active == true then
				
					local imgdat2 = globEnemyK[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyK[j].x + p
								testY2 = globEnemyK[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyK[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyK[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
	--Enemy L
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyL,1 do
			
				if globEnemyL[j] ~= nil and globEnemyL[j].active == true then
				
					local imgdat2 = globEnemyL[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyL[j].x + p
								testY2 = globEnemyL[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyL[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyL[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
	
	--Enemy M
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyM,1 do
			
				if globEnemyM[j] ~= nil and globEnemyM[j].active == true then
				
					local imgdat2 = globEnemyM[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyM[j].x + p
								testY2 = globEnemyM[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyM[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyM[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
		
	--Enemy N
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyN,1 do
			
				if globEnemyN[j] ~= nil and globEnemyN[j].active == true then
				
					local imgdat2 = globEnemyN[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyN[j].x + p
								testY2 = globEnemyN[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyN[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyN[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
		
	--Enemy O
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyO,1 do
			
				if globEnemyO[j] ~= nil and globEnemyO[j].active == true then
				
					local imgdat2 = globEnemyO[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyO[j].x + p
								testY2 = globEnemyO[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyO[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyO[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
		
	--Enemy P
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyP,1 do
			
				if globEnemyP[j] ~= nil and globEnemyP[j].active == true then
				
					local imgdat2 = globEnemyP[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyP[j].x + p
								testY2 = globEnemyP[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyP[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyP[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
	
		
	--Enemy Q
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyQ,1 do
			
				if globEnemyQ[j] ~= nil and globEnemyQ[j].active == true then
				
					local imgdat2 = globEnemyQ[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyQ[j].x + p
								testY2 = globEnemyQ[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyQ[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyQ[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
		
	--Enemy R
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyR,1 do
			
				if globEnemyR[j] ~= nil and globEnemyR[j].active == true then
				
					local imgdat2 = globEnemyR[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyR[j].x + p
								testY2 = globEnemyR[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyR[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyR[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
	
		
	--Enemy S
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyS,1 do
			
				if globEnemyS[j] ~= nil and globEnemyS[j].active == true then
				
					local imgdat2 = globEnemyS[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyS[j].x + p
								testY2 = globEnemyS[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyS[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyS[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
	
		
	--Enemy T
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyT,1 do
			
				if globEnemyT[j] ~= nil and globEnemyT[j].active == true then
				
					local imgdat2 = globEnemyT[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyT[j].x + p
								testY2 = globEnemyT[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyT[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyT[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
		
	--Enemy U
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyU,1 do
			
				if globEnemyU[j] ~= nil and globEnemyU[j].active == true then
				
					local imgdat2 = globEnemyU[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyU[j].x + p
								testY2 = globEnemyU[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyU[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyU[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
		
	--Enemy V
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyV,1 do
			
				if globEnemyV[j] ~= nil and globEnemyV[j].active == true then
				
					local imgdat2 = globEnemyV[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyV[j].x + p
								testY2 = globEnemyV[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyV[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyV[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
		
	--Enemy W
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyW,1 do
			
				if globEnemyW[j] ~= nil and globEnemyW[j].active == true then
				
					local imgdat2 = globEnemyW[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyW[j].x + p
								testY2 = globEnemyW[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyW[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyW[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
		
	--Enemy X
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyX,1 do
			
				if globEnemyX[j] ~= nil and globEnemyX[j].active == true then
				
					local imgdat2 = globEnemyX[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyX[j].x + p
								testY2 = globEnemyX[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyX[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyX[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
	
		
	--Enemy Y
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyY,1 do
			
				if globEnemyY[j] ~= nil and globEnemyY[j].active == true then
				
					local imgdat2 = globEnemyY[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyY[j].x + p
								testY2 = globEnemyY[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyY[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyY[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
		
	--Enemy Z
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			for j= 1,globMaxEnemyZ,1 do
			
				if globEnemyZ[j] ~= nil and globEnemyZ[j].active == true then
				
					local imgdat2 = globEnemyZ[j].currentGph:getData()
					local fieldW2 = imgdat2:getWidth()
					local fieldH2 = imgdat2:getHeight()
					local testX2 = 0
					local testY2 = 0
						
					local r2, g2, b2, a2 =  0
				
				
					
					local lclTookHit = false
					
					local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
					--first test
					for p = 1,fieldW2, 1 do
						for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
							r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
							if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX2 = globEnemyZ[j].x + p
								testY2 = globEnemyZ[j].y + q
								
								lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
							end
						end
					end
					
					--more accurate test
					if lclTestForHit == true then
						
						for m = 1,fieldW, 1 do
							for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
								r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
								
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
										if lclTookHit == false then
											if a ~= 0 and a2 ~= 0 then
										
												testX = globPlShip[i].x + m 
												testY = globPlShip[i].y + n 
											
												testX2 = round(globEnemyZ[j].x + p, 0) --some enemies have unround position due to movement splines, so round
												testY2 = round(globEnemyZ[j].y + q, 0)
											
												if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
													lclTookHit = true
												end
										
											end
										end
									
									end
								end
							
							
							end
						end
					
					end
					
					
					if lclTookHit == true then
															--see above:
						self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
					end
					
				end
		
			end
		
		end
		
	end
	
	
	
	
	
		
	--Boss 1
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			
			
			if globEnemyBoss1 ~= nil and globEnemyBoss1.active == true then
				
				local imgdat2 = globEnemyBoss1.currentGph:getData()
				local fieldW2 = imgdat2:getWidth()
				local fieldH2 = imgdat2:getHeight()
				local testX2 = 0
				local testY2 = 0
						
				local r2, g2, b2, a2 =  0
				
				
					
				local lclTookHit = false
					
				local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
				--first test
				for p = 1,fieldW2, 1 do
					for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
						r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
						if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
							testX2 = globEnemyBoss1.x + p
							testY2 = globEnemyBoss1.y + q
								
							lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
						end
					end
				end
					
				--more accurate test
				if lclTestForHit == true then
						
					for m = 1,fieldW, 1 do
						for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
							r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
							for p = 1,fieldW2, 1 do
								for q = 1,fieldH2, 1 do
								
									r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
									if lclTookHit == false then
										if a ~= 0 and a2 ~= 0 then
										
											testX = globPlShip[i].x + m 
											testY = globPlShip[i].y + n 
											
											testX2 = round(globEnemyBoss1.x + p, 0) --some enemies have unround position due to movement splines, so round
											testY2 = round(globEnemyBoss1.y + q, 0)
											
											if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
												lclTookHit = true
											end
										
										end
									end
									
								end
							end
							
							
						end
					end
					
				end
					
					
				if lclTookHit == true then
															--see above:
					self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
				end
					
			end
		
		
		end
		
	end
	
	
	
	
	
		
	--Boss 2
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			
			
			if globEnemyBoss2 ~= nil and globEnemyBoss2.active == true then
				
				local imgdat2 = globEnemyBoss2.currentGph:getData()
				local fieldW2 = imgdat2:getWidth()
				local fieldH2 = imgdat2:getHeight()
				local testX2 = 0
				local testY2 = 0
						
				local r2, g2, b2, a2 =  0
				
				
					
				local lclTookHit = false
					
				local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
				--first test
				for p = 1,fieldW2, 1 do
					for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
						r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
						if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
							testX2 = globEnemyBoss2.x + p
							testY2 = globEnemyBoss2.y + q
								
							lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
						end
					end
				end
					
				--more accurate test
				if lclTestForHit == true then
						
					for m = 1,fieldW, 1 do
						for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
							r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
							for p = 1,fieldW2, 1 do
								for q = 1,fieldH2, 1 do
								
									r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
									if lclTookHit == false then
										if a ~= 0 and a2 ~= 0 then
										
											testX = globPlShip[i].x + m 
											testY = globPlShip[i].y + n 
											
											testX2 = round(globEnemyBoss2.x + p, 0) --some enemies have unround position due to movement splines, so round
											testY2 = round(globEnemyBoss2.y + q, 0)
											
											if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
												lclTookHit = true
											end
										
										end
									end
									
								end
							end
							
							
						end
					end
					
				end
					
					
				if lclTookHit == true then
															--see above:
					self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
				end
					
			end
		
		
		end
		
	end
	
	
	
	
	
	
		
	--Boss 3
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil and globPlShip[i].active == true and globPlShip[i].respawnedVulnerableFlag == true then
		
			local imgdat = globPlShip[i].currentGph:getData()
			local fieldW = imgdat:getWidth()
			local fieldH = imgdat:getHeight()
			local testX = 0
			local testY = 0
						
			local r, g, b, a =  0
		
		
			
			
			if globEnemyBoss3 ~= nil and globEnemyBoss3.active == true then
				
				local imgdat2 = globEnemyBoss3.currentGph:getData()
				local fieldW2 = imgdat2:getWidth()
				local fieldH2 = imgdat2:getHeight()
				local testX2 = 0
				local testY2 = 0
						
				local r2, g2, b2, a2 =  0
				
				
					
				local lclTookHit = false
					
				local lclTestForHit = false--globPlShip[i].currentBBox:CollisionTest(globEnmBulletsA[j].currentBBox) --only test for hit with pixelcollision when there is boundingboxcollision
					
				--first test
				for p = 1,fieldW2, 1 do
					for q = 1,fieldH2, 1 do --TODO need to look where row and col are needed and why index is off
						r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
						if a2 ~= 0 and lclTestForHit == false then --test only for specific pixel color and only as long as no hit was detected
							testX2 = globEnemyBoss3.x + p
							testY2 = globEnemyBoss3.y + q
								
							lclTestForHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX2, testY2)
						end
					end
				end
					
				--more accurate test
				if lclTestForHit == true then
						
					for m = 1,fieldW, 1 do
						for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						
							r, g, b, a =  imgdat:getPixel(m-1, n-1)
							
							for p = 1,fieldW2, 1 do
								for q = 1,fieldH2, 1 do
								
									r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
								
									if lclTookHit == false then
										if a ~= 0 and a2 ~= 0 then
										
											testX = globPlShip[i].x + m 
											testY = globPlShip[i].y + n 
											
											testX2 = round(globEnemyBoss3.x + p, 0) --some enemies have unround position due to movement splines, so round
											testY2 = round(globEnemyBoss3.y + q, 0)
											
											if testX == testX2 and testY == testY2 then --pixels on top of each other, each with content that is not opaque
												lclTookHit = true
											end
										
										end
									end
									
								end
							end
							
							
						end
					end
					
				end
					
					
				if lclTookHit == true then
															--see above:
					self.playerTookHit[i] = lclTookHit 	--only actualize when value "true" was fetched, otherwise won't
															--keep the self.playerTookHit[i] value that was detected 
															--once there is negative test against the next active bullet
															--and that will cause messed up behavior where self.playerTookHit[i] is used
						
				end
					
			end
		
		
		end
		
	end
	
	
	
	
	--initialize player dying and flicker effect for dying
	for i = 1,globMaxPlayers,1 do
		if self.playerTookHit[i] == true then
		
			--self.playerDyingCountDown = self.playerDyingCountDownMax --not needed, no death flicker
			
			self.playerTookHit[i] = false
					
			globPlShip[i].putToDeath(globPlShip[i])
		end
	end

	
end


function GameMode.powerupPlayerCollisionHandling(self)
	
	--collecting powerup 
	for i=1,globMaxPlayers,1 do 
		
		if globPlShip[i] ~= nil and globPlShip[i].active == true then
		
			if globPowerup ~= nil and globPowerup.active == true then
				
				
				local lclTookHit = false
					
					
				local imgdat = globPowerup.currentGph:getData()
				local fieldW = imgdat:getWidth()
				local fieldH = imgdat:getHeight()
				local testX = 0
				local testY = 0
				local testHit = false
						
				local r, g, b, a =  0
				
				
				for m = 1,fieldW, 1 do
					for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
						r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
						if a ~= 0 and lclTookHit == false then --test only for specific pixel color and only as long as no hit was detected
							testX = globPowerup.x + m 
							testY = globPowerup.y + n 
								
							testHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX, testY)
								
							if testHit == true then
								local imgdat2 = globPlShip[i].currentGph:getData()
								local fieldW2 = imgdat2:getWidth()
								local fieldH2 = imgdat2:getHeight()
								local testX2 = 0
								local testY2 = 0
							
								local r2, g2, b2, a2 =  0
								
								for p = 1,fieldW2, 1 do
									for q = 1,fieldH2, 1 do
										r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
										
										if a2 ~= 0 and lclTookHit == false then
											local testX2 = globPlShip[i].x + p
											local testY2 = globPlShip[i].y + q
											
											if testX == testX2 and testY == testY2 then
												lclTookHit = true
											end
										end
									end
								end
							
							end
								
						end
					end
				end
				
				
				if lclTookHit == true then --collect powerup
					
					if globPowerup.type == 1 then
						if globPlShip[i].power < globPlShip[i].powerMax then
							globPlShip[i].doPowerUp(globPlShip[i])
						else
							globAddScore(1000)
							globExplosion1000.activate(globExplosion1000, globPowerup.x - 9, globPowerup.y - 3)
						end
					else
						if globPlShip[i].satelites < globPlShip[i].satelitesMax then
							globPlShip[i].addSatelite(globPlShip[i])
						else
							globAddScore(1000)
							globExplosion1000.activate(globExplosion1000, globPowerup.x - 9, globPowerup.y - 3)
						end
					end
														
						
					globPowerup.deactivate(globPowerup)
					
					if globIsInDemoMode == false then
						extraPowerSnd:stop()
						extraPowerSnd:play()
					end
						
				end
				
				
				
			end
				
		
				
		end
	end
	
	
	
	--collecting satelitecollectible
	for i=1,globMaxPlayers,1 do 
		
		if globPlShip[i] ~= nil and globPlShip[i].active == true then
		
			for j = 1,globSateliteCollectibleIndexMax,1 do
			
				if globSateliteCollectible[j] ~= nil and globSateliteCollectible[j].active == true then
				
					local lclTookHit = false
					
					
					local imgdat = globSateliteCollectible[j].currentGph:getData()
					local fieldW = imgdat:getWidth()
					local fieldH = imgdat:getHeight()
					local testX = 0
					local testY = 0
					local testHit = false
						
					local r, g, b, a =  0
				
				
					for m = 1,fieldW, 1 do
						for n = 1,fieldH, 1 do --TODO need to look where row and col are needed and why index is off
							r, g, b, a =  imgdat:getPixel(m-1, n-1)
								
							if a ~= 0 and lclTookHit == false then --test only for specific pixel color and only as long as no hit was detected
								testX = globSateliteCollectible[j].x + m 
								testY = globSateliteCollectible[j].y + n 
								
								testHit = globPlShip[i].currentBBox.InBoxTest(globPlShip[i].currentBBox,testX, testY)
								
								if testHit == true then
								
									local imgdat2 = globPlShip[i].currentGph:getData()
									local fieldW2 = imgdat2:getWidth()
									local fieldH2 = imgdat2:getHeight()
									local testX2 = 0
									local testY2 = 0
							
									local r2, g2, b2, a2 =  0
								
									for p = 1,fieldW2, 1 do
										for q = 1,fieldH2, 1 do
											r2, g2, b2, a2 =  imgdat2:getPixel(p-1, q-1)
										
											if a2 ~= 0 and lclTookHit == false then
												local testX2 = globPlShip[i].x + p
												local testY2 = globPlShip[i].y + q
											
												if testX == testX2 and testY == testY2 then
													lclTookHit = true
												end
											end
										end
									end
							
								end
								
							end
						end
					end
				
				
					if lclTookHit == true then --collect powerup
					
						globPlShip[i].addSatelite(globPlShip[i])
														
						
						globSateliteCollectible[j].deactivate(globSateliteCollectible[j])
						
					end
				
				end
			end
		end
	end
	
		
	
	
end


function GameMode.deathFlickerUseUp(self)

end

function GameMode.doScrolling(self)

	if self.startedScroll == false then
		self.startedScroll = true
	else
		if self.scrollProgress < (globScrollWidth[globLevel] - resWidth) then 
			self.scrollProgress = self.scrollProgress + 1
			--print("ScrollProgress "..self.scrollProgress)
		end
	end

end

function GameMode.doGameOverCheck(self)

	local lclNumPlayersOutOfGame = 0

	for i=1,globMaxPlayers,1 do 
	
		--for every player, irrespective if they died in this run or earlier, add player gameovers/ none-joins
		if globNumLifes[i] == (-1) or globNumLifes[i] == (-2) then
			lclNumPlayersOutOfGame = lclNumPlayersOutOfGame + 1
		end
		
	end
	
	if lclNumPlayersOutOfGame == globMaxPlayers then
	
		self.gameOverFlag = true
		
		globLevelMusic:stop()
	end
end


function GameMode.checkPauseUnPause(self)
	
	if self.pauseCountDown > 0 then
		self.pauseCountDown = self.pauseCountDown - 1
	else
		local lclKeybrdPPressed = false
		local lclGamepad1StartPressed = false
		local lclGamepad2StartPressed = false
		local lclGamepad3StartPressed = false
		
		lclKeybrdPPressed = lkeyBrd("p")
		
		if globJoystick then 
			lclGamepad1StartPressed = globJoystick:isGamepadDown("start")
		end
		
		if globJoystick2 then
			lclGamepad2StartPressed = globJoystick2:isGamepadDown("start")
		end
		
		if globJoystick3 then
			lclGamepad3StartPressed = globJoystick3:isGamepadDown("start")
		end
		
		if lclKeybrdPPressed == true or lclGamepad1StartPressed == true or lclGamepad2StartPressed == true or lclGamepad3StartPressed == true then
			
			if self.isPaused == true then
				self.isPaused = false
				
				globLevelMusic:resume()
			else
				self.isPaused = true
				
				globLevelMusic:pause()
				
				--swirl1Snd:stop()
				--swirl2Snd:stop()
				--swirl3Snd:stop()
				--swirl4Snd:stop()
				--swirl5Snd:stop()
				
				if globIsInDemoMode == false then --play sound not in DemoMode
					pauseSnd:stop()
					pauseSnd:play()
				end
			end
			
			self.pauseCountDown = self.pauseCountDownMax
		end
		
	end

end


function GameMode:draw()																				--Task AAT (part 4) draw canvas game mode

	--if self.playerDyingCountDown == 0 then
		lg.draw( self.levelBackGph, self.bkg_x, self.bkg_y, 0, 1, 1)
	--else
		--lg.draw( self.levelBackFlickerGph, self.bkg_x, self.bkg_y, 0, 1, 1)
	--end
	--lg.draw( self.levelGroundGph, self.bkg_x, self.bkg_y + 125, 0, 1, 1) --debug, otherwise below to omit other objects
	
	
	if globBossBack ~= nil then  --draw in the background
		globBossBack.draw(globBossBack)
	end
	
	
	--#######################
	--# draw enemies E + F
	--#######################
	
	for i = 1,globMaxEnemyE,1 do
		if globEnemyE[i] ~= nil then
			globEnemyE[i].draw(globEnemyE[i])
		end
	end
	
	for i = 1,globMaxEnemyF,1 do
		if globEnemyF[i] ~= nil then
			globEnemyF[i].draw(globEnemyF[i])
		end
	end
	
	
	--#######################
	--# foreground 
	--#######################
	
	lg.draw( levelForegroundGphs[globLevel], self.bkg_x - self.scrollProgress, self.bkg_y, 0, 1, 1)
	
	--#######################
	--# draw explosions
	--#######################
	
	for i=1,globSmallExplosionMax,1 do
	
		if globSmallExplosions[i] ~= nil then
			globSmallExplosions[i].draw(globSmallExplosions[i])
		end
	end
	
	for i=1,globMediumExplosionMax,1 do
	
		if globMediumExplosions[i] ~= nil then
			globMediumExplosions[i].draw(globMediumExplosions[i])
		end
	end
	
	for i=1,globBigExplosionMax,1 do
	
		if globBigExplosions[i] ~= nil then
			globBigExplosions[i].draw(globBigExplosions[i])
		end
	end
	
	if globBoss1Explosion ~= nil then
		globBoss1Explosion.draw(globBoss1Explosion)
	end
	
	if globBoss2Explosion ~= nil then
		globBoss2Explosion.draw(globBoss2Explosion)
	end
	
	if globBoss3Explosion ~= nil then
		globBoss3Explosion.draw(globBoss3Explosion)
	end
	
	if globPlayerExplosion ~= nil then
		globPlayerExplosion.draw(globPlayerExplosion)
	end
	
	if globExplosion1000 ~= nil then
		globExplosion1000.draw(globExplosion1000)
	end
	
	--explosion of mothership below
	
	--#######################
	--# draw player 
	--#######################
	
	for i=1,globMaxPlayers,1 do 
	
		if globPlShip[i] ~= nil then
			globPlShip[i].draw(globPlShip[i])
		end
	end
	
	--#######################
	--# draw enemy bullets
	--#######################
	
	for i = 1, globEnmBulletsAIndexMax, 1 do
	
		if globEnmBulletsA[i] ~= nil then
			globEnmBulletsA[i].draw(globEnmBulletsA[i])
		end
	end
	
	for i = 1, globEnmBulletsBIndexMax, 1 do
	
		if globEnmBulletsB[i] ~= nil then
			globEnmBulletsB[i].draw(globEnmBulletsB[i])
		end
	end
	
	for i = 1, globEnmBulletsCIndexMax, 1 do
	
		if globEnmBulletsC[i] ~= nil then
			globEnmBulletsC[i].draw(globEnmBulletsC[i])
		end
	end
	
	for i = 1, globEnmBulletsDIndexMax, 1 do
	
		if globEnmBulletsD[i] ~= nil then
			globEnmBulletsD[i].draw(globEnmBulletsD[i])
		end
	end
	
	for i = 1, globEnmBulletsEIndexMax, 1 do
	
		if globEnmBulletsE[i] ~= nil then
			globEnmBulletsE[i].draw(globEnmBulletsE[i])
		end
	end
	
	for i = 1, globEnmBulletsFIndexMax, 1 do
	
		if globEnmBulletsF[i] ~= nil then
			globEnmBulletsF[i].draw(globEnmBulletsF[i])
		end
	end
	
	for i = 1, globEnmBulletsGIndexMax, 1 do
	
		if globEnmBulletsG[i] ~= nil then
			globEnmBulletsG[i].draw(globEnmBulletsG[i])
		end
	end
	
	--#######################
	--# draw powerup 
	--#######################
	
	if globPowerup ~= nil then
		globPowerup.draw(globPowerup)
	end
	
	--########################
	--# draw satelite chunks
	--########################
			
	for i = 1,globSateliteChunkIndexMax,1 do
		if globSateliteChunk[i] ~= nil then
			globSateliteChunk[i].draw(globSateliteChunk[i])
		end
	end
			
	--###############################
	--# draw satelite collectibles
	--##############################
			
	for i = 1,globSateliteCollectibleIndexMax,1 do
		if globSateliteCollectible[i] ~= nil then
			globSateliteCollectible[i].draw(globSateliteCollectible[i])
		end
	end
	
	
	--#######################
	--# draw enemies 
	--#######################
	
	for i = 1,globMaxEnemyV,1 do --draw big enemy in the back end so other enemies stay visible
		if globEnemyV[i] ~= nil then
			globEnemyV[i].draw(globEnemyV[i])
		end
	end
	
	for i = 1,globMaxEnemyZ,1 do --draw big enemy in the back end so other enemies stay visible
		if globEnemyZ[i] ~= nil then
			globEnemyZ[i].draw(globEnemyZ[i])
		end
	end
	
	for i = 1,globMaxEnemyA,1 do
		if globEnemyA[i] ~= nil then
			globEnemyA[i].draw(globEnemyA[i])
		end
	end
	
	for i = 1,globMaxEnemyB,1 do
		if globEnemyB[i] ~= nil then
			globEnemyB[i].draw(globEnemyB[i])
		end
	end
	
	for i = 1,globMaxEnemyC,1 do
		if globEnemyC[i] ~= nil then
			globEnemyC[i].draw(globEnemyC[i])
		end
	end
	
	for i = 1,globMaxEnemyD,1 do
		if globEnemyD[i] ~= nil then
			globEnemyD[i].draw(globEnemyD[i])
		end
	end
	
	for i = 1,globMaxEnemyE,1 do
		if globEnemyE[i] ~= nil then
			globEnemyE[i].draw(globEnemyE[i])
		end
	end
	
	for i = 1,globMaxEnemyF,1 do
		if globEnemyF[i] ~= nil then
			globEnemyF[i].draw(globEnemyF[i])
		end
	end
	
	--need to draw enemies E and F before drawing foreground
	
	if globEnemyG ~= nil then
		globEnemyG.draw(globEnemyG)
	end
	
	for i = 1,globMaxEnemyH,1 do
		if globEnemyH[i] ~= nil then
			globEnemyH[i].draw(globEnemyH[i])
		end
	end
	
	for i = 1,globMaxEnemyI,1 do
		if globEnemyI[i] ~= nil then
			globEnemyI[i].draw(globEnemyI[i])
		end
	end
	
	for i = 1,globMaxEnemyJ,1 do
		if globEnemyJ[i] ~= nil then
			globEnemyJ[i].draw(globEnemyJ[i])
		end
	end
	
	for i = 1,globMaxEnemyK,1 do
		if globEnemyK[i] ~= nil then
			globEnemyK[i].draw(globEnemyK[i])
		end
	end
	
	for i = 1,globMaxEnemyL,1 do
		if globEnemyL[i] ~= nil then
			globEnemyL[i].draw(globEnemyL[i])
		end
	end
	
	for i = 1,globMaxEnemyM,1 do
		if globEnemyM[i] ~= nil then
			globEnemyM[i].draw(globEnemyM[i])
		end
	end
	
	for i = 1,globMaxEnemyN,1 do
		if globEnemyN[i] ~= nil then
			globEnemyN[i].draw(globEnemyN[i])
		end
	end
	
	for i = 1,globMaxEnemyO,1 do
		if globEnemyO[i] ~= nil then
			globEnemyO[i].draw(globEnemyO[i])
		end
	end
	
	for i = 1,globMaxEnemyQ,1 do --draw enemy q before enemy p to draw enemy p on top of it
		if globEnemyQ[i] ~= nil then
			globEnemyQ[i].draw(globEnemyQ[i])
		end
	end
	
	for i = 1,globMaxEnemyP,1 do
		if globEnemyP[i] ~= nil then
			globEnemyP[i].draw(globEnemyP[i])
		end
	end
	
	for i = 1,globMaxEnemyR,1 do 
		if globEnemyR[i] ~= nil then
			globEnemyR[i].draw(globEnemyR[i])
		end
	end
	
	for i = 1,globMaxEnemyS,1 do 
		if globEnemyS[i] ~= nil then
			globEnemyS[i].draw(globEnemyS[i])
		end
	end
	
	for i = 1,globMaxEnemyT,1 do 
		if globEnemyT[i] ~= nil then
			globEnemyT[i].draw(globEnemyT[i])
		end
	end
	
	for i = 1,globMaxEnemyU,1 do 
		if globEnemyU[i] ~= nil then
			globEnemyU[i].draw(globEnemyU[i])
		end
	end
	
	for i = 1,globMaxEnemyW,1 do 
		if globEnemyW[i] ~= nil then
			globEnemyW[i].draw(globEnemyW[i])
		end
	end
	
	for i = 1,globMaxEnemyX,1 do 
		if globEnemyX[i] ~= nil then
			globEnemyX[i].draw(globEnemyX[i])
		end
	end
	
	for i = 1,globMaxEnemyY,1 do 
		if globEnemyY[i] ~= nil then
			globEnemyY[i].draw(globEnemyY[i])
		end
	end
	
	
	
	if globEnemyBoss1 ~= nil then
		globEnemyBoss1.draw(globEnemyBoss1)
	end
	
	if globEnemyBoss2 ~= nil then
		globEnemyBoss2.draw(globEnemyBoss2)
	end
	
	if globEnemyBoss3 ~= nil then
		globEnemyBoss3.draw(globEnemyBoss3)
	end
	
	--if globBossBack ~= nil then  --not here, draw more in the background
	--	globBossBack.draw(globBossBack)
	--end
	
	
	--#######################
	--# draw player bullets
	--#######################

	for i=1,globMaxPlayers,1 do 
		
		for j=1,globLeadPlBulletIndexMax,1 do --globLeadPlBulletIndexMax
			globLeadPlBullets[i][j].draw(globLeadPlBullets[i][j])
		end
	end
	
	--#################################
	--# draw player explosion
	--#################################
	
	--done above
	
	--################################
	--# draw Maskprint/Statusbar
	--################################
	
	globMaskPrinter.draw(globMaskPrinter) 
	
	
	--################################
	--# draw Gameover
	--################################
	
	if self.gameOverFlag == true then
		lg.draw( gameOverOverlayGph, 0, 0, 0, 1, 1)
	end
	
	
	if self.isPaused == true then
	
		lg.draw( pauseGph, 0, 0, 0, 1, 1)
	end
	
	
	if self.showIntro then
		lg.draw( levelIntroGphs[globLevel], self.bkg_x, self.bkg_y, 0, 1, 1)
	end
	
	--need to draw explosion of mothership over everything
	
	if globEnding ~= nil then
		globEnding.draw(globEnding)
	end
	
	if globExplosionMothership ~= nil then
		globExplosionMothership.draw(globExplosionMothership)
	end
	
	if self.showEndingExplosions == true then
		for i=1,globSmallExplosionMax,1 do
	
			if globSmallExplosions[i] ~= nil then
				globSmallExplosions[i].draw(globSmallExplosions[i])
			end
		end
	
		for i=1,globMediumExplosionMax,1 do
	
			if globMediumExplosions[i] ~= nil then
				globMediumExplosions[i].draw(globMediumExplosions[i])
			end
		end
	end
	
	
end


